@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001 Ximain, Inc. (www.ximian.com)
+ *  Copyright 2001 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,43 +25,121 @@
 #include <config.h>
 #endif
 
-#include "gmime-stream.h"
 #include <string.h>
 
+#include "gmime-stream.h"
+
+#define d(x) x
+
+static void g_mime_stream_class_init (GMimeStreamClass *klass);
+static void g_mime_stream_init (GMimeStream *stream, GMimeStreamClass *klass);
+static void g_mime_stream_finalize (GObject *object);
+
+static gssize stream_read (GMimeStream *stream, char *buf, size_t len);
+static gssize stream_write (GMimeStream *stream, char *buf, size_t len);
+static int stream_flush (GMimeStream *stream);
+static int stream_close (GMimeStream *stream);
+static gboolean stream_eos (GMimeStream *stream);
+static int stream_reset (GMimeStream *stream);
+static off_t stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence);
+static off_t stream_tell (GMimeStream *stream);
+static gssize stream_length (GMimeStream *stream);
+static GMimeStream *stream_substream (GMimeStream *stream, off_t start, off_t end);
+
+
+static GObjectClass *parent_class = NULL;
+
+
+GType
+g_mime_stream_get_type (void)
+{
+	static GType type = 0;
+	
+	if (!type) {
+		static const GTypeInfo info = {
+			sizeof (GMimeStreamClass),
+			NULL, /* base_class_init */
+			NULL, /* base_class_finalize */
+			(GClassInitFunc) g_mime_stream_class_init,
+			NULL, /* class_finalize */
+			NULL, /* class_data */
+			sizeof (GMimeStream),
+			0,    /* n_preallocs */
+			(GInstanceInitFunc) g_mime_stream_init,
+		};
+		
+		type = g_type_register_static (G_TYPE_OBJECT, "GMimeStream", &info, 0);
+	}
+	
+	return type;
+}
+
+
+static void
+g_mime_stream_class_init (GMimeStreamClass *klass)
+{
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	
+	parent_class = g_type_class_ref (G_TYPE_OBJECT);
+	
+	object_class->finalize = g_mime_stream_finalize;
+	
+	klass->read = stream_read;
+	klass->write = stream_write;
+	klass->flush = stream_flush;
+	klass->close = stream_close;
+	klass->eos = stream_eos;
+	klass->reset = stream_reset;
+	klass->seek = stream_seek;
+	klass->tell = stream_tell;
+	klass->length = stream_length;
+	klass->substream = stream_substream;
+}
+
+static void
+g_mime_stream_init (GMimeStream *stream, GMimeStreamClass *klass)
+{
+	stream->super_stream = NULL;
+	
+	stream->position = 0;
+	stream->bound_start = 0;
+	stream->bound_end = 0;
+}
+
+static void
+g_mime_stream_finalize (GObject *object)
+{
+	GMimeStream *stream = (GMimeStream *) object;
+	
+	if (stream->super_stream)
+		g_mime_stream_unref (stream->super_stream);
+	
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 
 /**
  * g_mime_stream_construct:
  * @stream: stream
- * @stream_template: stream template
- * @type: stream type
  * @start: start boundary
  * @end: end boundary
  *
- * Initializes a new stream of type @type, using the virtual methods
- * from @stream_template, with bounds @start and @end.
+ * Initializes a new stream with bounds @start and @end.
  **/
 void
-g_mime_stream_construct (GMimeStream *stream, GMimeStream *stream_template, unsigned int type, off_t start, off_t end)
+g_mime_stream_construct (GMimeStream *stream, off_t start, off_t end)
 {
-	stream->super_stream = NULL;
-	stream->refcount = 1;
-	stream->type = type;
-	
 	stream->position = start;
 	stream->bound_start = start;
 	stream->bound_end = end;
-	
-	stream->destroy = stream_template->destroy;
-	stream->read = stream_template->read;
-	stream->write = stream_template->write;
-	stream->flush = stream_template->flush;
-	stream->close = stream_template->close;
-	stream->reset = stream_template->reset;
-	stream->seek = stream_template->seek;
-	stream->tell = stream_template->tell;
-	stream->eos = stream_template->eos;
-	stream->length = stream_template->length;
-	stream->substream = stream_template->substream;
+}
+
+
+static gssize
+stream_read (GMimeStream *stream, char *buf, size_t len)
+{
+	d(g_warning ("Invoked default stream_read implementation."));
+	return 0;
 }
 
 
@@ -75,13 +153,21 @@ g_mime_stream_construct (GMimeStream *stream, GMimeStream *stream_template, unsi
  *
  * Returns the number of bytes read or -1 on fail.
  **/
-ssize_t
+gssize
 g_mime_stream_read (GMimeStream *stream, char *buf, size_t len)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	g_return_val_if_fail (buf != NULL, -1);
 	
-	return stream->read (stream, buf, len);
+	return GMIME_STREAM_GET_CLASS (stream)->read (stream, buf, len);
+}
+
+
+static gssize
+stream_write (GMimeStream *stream, char *buf, size_t len)
+{
+	d(g_warning ("Invoked default stream_write implementation."));
+	return 0;
 }
 
 
@@ -95,13 +181,21 @@ g_mime_stream_read (GMimeStream *stream, char *buf, size_t len)
  *
  * Returns the number of bytes written or -1 on fail.
  **/
-ssize_t
+gssize
 g_mime_stream_write (GMimeStream *stream, char *buf, size_t len)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	g_return_val_if_fail (buf != NULL, -1);
 	
-	return stream->write (stream, buf, len);
+	return GMIME_STREAM_GET_CLASS (stream)->write (stream, buf, len);
+}
+
+
+static int
+stream_flush (GMimeStream *stream)
+{
+	d(g_warning ("Invoked default stream_flush implementation."));
+	return 0;
 }
 
 
@@ -116,9 +210,17 @@ g_mime_stream_write (GMimeStream *stream, char *buf, size_t len)
 int
 g_mime_stream_flush (GMimeStream *stream)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return stream->flush (stream);
+	return GMIME_STREAM_GET_CLASS (stream)->flush (stream);
+}
+
+
+static int
+stream_close (GMimeStream *stream)
+{
+	d(g_warning ("Invoked default stream_close implementation."));
+	return 0;
 }
 
 
@@ -133,9 +235,17 @@ g_mime_stream_flush (GMimeStream *stream)
 int
 g_mime_stream_close (GMimeStream *stream)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return stream->close (stream);
+	return GMIME_STREAM_GET_CLASS (stream)->close (stream);
+}
+
+
+static gboolean
+stream_eos (GMimeStream *stream)
+{
+	d(g_warning ("Invoked default stream_eos implementation."));
+	return stream->position < stream->bound_end;
 }
 
 
@@ -145,17 +255,26 @@ g_mime_stream_close (GMimeStream *stream)
  *
  * Tests the end-of-stream indicator for @stream.
  *
- * Returns TRUE on EOS or FALSE otherwise.
+ * Returns %TRUE on EOS or %FALSE otherwise.
  **/
 gboolean
 g_mime_stream_eos (GMimeStream *stream)
 {
-	g_return_val_if_fail (stream != NULL, TRUE);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), TRUE);
 	
 	if (stream->bound_end != -1 && stream->position >= stream->bound_end)
 		return TRUE;
 	
-	return stream->eos (stream);
+	return GMIME_STREAM_GET_CLASS (stream)->eos (stream);
+}
+
+
+static int
+stream_reset (GMimeStream *stream)
+{
+	d(g_warning ("Invoked default stream_reset implementation."));
+	stream->position = stream->bound_start;
+	return 0;
 }
 
 
@@ -170,9 +289,17 @@ g_mime_stream_eos (GMimeStream *stream)
 int
 g_mime_stream_reset (GMimeStream *stream)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return stream->reset (stream);
+	return GMIME_STREAM_GET_CLASS (stream)->reset (stream);
+}
+
+
+static off_t
+stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
+{
+	d(g_warning ("Invoked default stream_seek implementation."));
+	return -1;
 }
 
 
@@ -199,9 +326,17 @@ g_mime_stream_reset (GMimeStream *stream)
 off_t
 g_mime_stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return stream->seek (stream, offset, whence);
+	return GMIME_STREAM_GET_CLASS (stream)->seek (stream, offset, whence);
+}
+
+
+static off_t
+stream_tell (GMimeStream *stream)
+{
+	d(g_warning ("Invoked default stream_tell implementation."));
+	return stream->position;
 }
 
 
@@ -211,14 +346,22 @@ g_mime_stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
  *
  * Gets the current offset within the stream.
  *
- * Returns the current position within the stream.
+ * Returns the current position within the stream or -1 on fail.
  **/
 off_t
 g_mime_stream_tell (GMimeStream *stream)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return stream->tell (stream);
+	return GMIME_STREAM_GET_CLASS (stream)->tell (stream);
+}
+
+
+static gssize
+stream_length (GMimeStream *stream)
+{
+	d(g_warning ("Invoked default stream_length implementation."));
+	return -1;
 }
 
 
@@ -230,12 +373,20 @@ g_mime_stream_tell (GMimeStream *stream)
  *
  * Returns the length of the stream or -1 on fail.
  **/
-ssize_t
+gssize
 g_mime_stream_length (GMimeStream *stream)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return stream->length (stream);
+	return GMIME_STREAM_GET_CLASS (stream)->length (stream);
+}
+
+
+static GMimeStream *
+stream_substream (GMimeStream *stream, off_t start, off_t end)
+{
+	d(g_warning ("Invoked default stream_tell implementation."));
+	return NULL;
 }
 
 
@@ -254,11 +405,13 @@ g_mime_stream_substream (GMimeStream *stream, off_t start, off_t end)
 {
 	GMimeStream *sub;
 	
-	g_return_val_if_fail (stream != NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), NULL);
 	
-	sub = stream->substream (stream, start, end);
-	sub->super_stream = stream;
-	g_mime_stream_ref (stream);
+	sub = GMIME_STREAM_GET_CLASS (stream)->substream (stream, start, end);
+	if (sub) {
+		sub->super_stream = stream;
+		g_mime_stream_ref (stream);
+	}
 	
 	return sub;
 }
@@ -273,9 +426,9 @@ g_mime_stream_substream (GMimeStream *stream, off_t start, off_t end)
 void
 g_mime_stream_ref (GMimeStream *stream)
 {
-	g_return_if_fail (stream != NULL);
+	g_return_if_fail (GMIME_IS_STREAM (stream));
 	
-	stream->refcount++;
+	g_object_ref ((GObject *) stream);
 }
 
 
@@ -288,16 +441,9 @@ g_mime_stream_ref (GMimeStream *stream)
 void
 g_mime_stream_unref (GMimeStream *stream)
 {
-	g_return_if_fail (stream != NULL);
+	g_return_if_fail (GMIME_IS_STREAM (stream));
 	
-	if (stream->refcount <= 1) {
-		if (stream->super_stream)
-			g_mime_stream_unref (stream->super_stream);
-		
-		stream->destroy (stream);
-	} else {
-		stream->refcount--;
-	}
+	g_object_unref ((GObject *) stream);
 }
 
 
@@ -312,7 +458,7 @@ g_mime_stream_unref (GMimeStream *stream)
 void
 g_mime_stream_set_bounds (GMimeStream *stream, off_t start, off_t end)
 {
-	g_return_if_fail (stream != NULL);
+	g_return_if_fail (GMIME_IS_STREAM (stream));
 	
 	stream->bound_start = start;
 	stream->bound_end = end;
@@ -331,12 +477,12 @@ g_mime_stream_set_bounds (GMimeStream *stream, off_t start, off_t end)
  *
  * Writes @string to @stream.
  *
- * Returns the number of bytes written.
+ * Returns the number of bytes written or -1 on fail.
  **/
-ssize_t
+gssize
 g_mime_stream_write_string (GMimeStream *stream, const char *string)
 {
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	g_return_val_if_fail (string != NULL, -1);
 	
 	return g_mime_stream_write (stream, (char *) string, strlen (string));
@@ -351,16 +497,16 @@ g_mime_stream_write_string (GMimeStream *stream, const char *string)
  *
  * Write formatted output to a stream.
  *
- * Returns the number of bytes written.
+ * Returns the number of bytes written or -1 on fail.
  **/
-ssize_t
+gssize
 g_mime_stream_printf (GMimeStream *stream, const char *fmt, ...)
 {
 	va_list args;
 	char *string;
-	ssize_t ret;
+	gssize ret;
 	
-	g_return_val_if_fail (stream != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	g_return_val_if_fail (fmt != NULL, -1);
 	
 	va_start (args, fmt);
@@ -384,16 +530,16 @@ g_mime_stream_printf (GMimeStream *stream, const char *fmt, ...)
  *
  * Attempts to write stream @src to stream @dest.
  *
- * Returns the number of bytes written.
+ * Returns the number of bytes written or -1 on fail.
  **/
-ssize_t
+gssize
 g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
 {
-	ssize_t nread, nwritten, total = 0;
+	gssize nread, nwritten, total = 0;
 	char buf[4096];
 	
-	g_return_val_if_fail (src != NULL, -1);
-	g_return_val_if_fail (dest != NULL, -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (src), -1);
+	g_return_val_if_fail (GMIME_IS_STREAM (dest), -1);
 	
 	while (!g_mime_stream_eos (src)) {
 		nread = g_mime_stream_read (src, buf, sizeof (buf));
@@ -403,7 +549,7 @@ g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
 		if (nread > 0) {
 			nwritten = 0;
 			while (nwritten < nread) {
-				ssize_t len;
+				gssize len;
 				
 				len = g_mime_stream_write (dest, buf + nwritten, nread - nwritten);
 				if (len < 0)
@@ -428,18 +574,21 @@ g_mime_stream_write_to_stream (GMimeStream *src, GMimeStream *dest)
  *
  * Writes at most @count blocks described by @vector to @stream.
  *
- * Returns the number of bytes written.
+ * Returns the number of bytes written or -1 on fail.
  **/
-size_t
-g_mime_stream_writev (GMimeStream *stream, IOVector *vector, size_t count)
+gssize
+g_mime_stream_writev (GMimeStream *stream, GMimeStreamIOVector *vector, size_t count)
 {
-	size_t i, total = 0;
+	gssize total = 0;
+	int i;
+	
+	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
 	for (i = 0; i < count; i++) {
-		ssize_t n, nwritten = 0;
+		gssize n, nwritten = 0;
 		
 		while (nwritten < vector[i].len) {
-			n = g_mime_stream_write (stream, vector[i].data + nwritten,
+			n = g_mime_stream_write (stream, (char*)vector[i].data + nwritten,
 						 vector[i].len - nwritten);
 			if (n > 0)
 				nwritten += n;
