@@ -29,18 +29,19 @@ extern "C" {
 #pragma }
 #endif /* __cplusplus }*/
 
-#include <glib.h>
+#include <glib/gtypes.h>
 #include <time.h>
 #include <stdarg.h>
 
 typedef enum {
-        GMIME_PART_ENCODING_DEFAULT,
-        GMIME_PART_ENCODING_7BIT,
-        GMIME_PART_ENCODING_8BIT,
-        GMIME_PART_ENCODING_BASE64,
-        GMIME_PART_ENCODING_QUOTEDPRINTABLE,
+	GMIME_PART_ENCODING_DEFAULT,
+	GMIME_PART_ENCODING_7BIT,
+	GMIME_PART_ENCODING_8BIT,
+	GMIME_PART_ENCODING_BINARY,
+	GMIME_PART_ENCODING_BASE64,
+	GMIME_PART_ENCODING_QUOTEDPRINTABLE,
 	GMIME_PART_ENCODING_UUENCODE,
-        GMIME_PART_NUM_ENCODINGS
+	GMIME_PART_NUM_ENCODINGS
 } GMimePartEncodingType;
 
 #define BASE64_ENCODE_LEN(x) ((size_t) ((x) * 5 / 3) + 4)  /* conservative would be ((x * 4 / 3) + 4) */
