@@ -23,7 +23,7 @@
 #ifndef __GMIME_H__
 #define __GMIME_H__
 
-#include <glib.h>
+#include <glib/gtypes.h>
 #include "gmime-charset.h"
 #include "gmime-iconv.h"
 #include "gmime-iconv-utils.h"
