@@ -113,7 +113,7 @@ find_matching_sent (GMimeMessage* message)
 	data.message = message;
 	data.body = g_strstrip (g_mime_message_get_body (data.message, TRUE, &is_html));
 	data.match = NULL;
-	acache_path_foreach (PAN_SENT, find_matching_sent_foreach, &data);
+	acache_path_foreach (PAN_SENT.str, find_matching_sent_foreach, &data);
 	g_free (data.body);
 	return data.match;
 }
@@ -217,13 +217,14 @@ article_copy_articles_to_folder_cb (gpointer obj, gpointer arg, gpointer data)
 	task = TASK(obj);
 
 	/* copy them over if the task succeeded ... */
-	if (status == TASK_SUCCESS)
+	if (status == TASK_OK)
 	{
 		guint i;
 		for (i=0; i<task->identifiers->len; ++i)
 		{
 			MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (task->identifiers, i));
-			GMimeMessage * message = acache_get_message (acache_key, &mid, 1);
+			const PString * id = &mid->message_id;
+			GMimeMessage * message = acache_get_message (acache_key, &id, 1);
 			folder_add_message (folder, message);
 			g_object_unref (message);
 		}
@@ -258,7 +259,8 @@ article_copy_articles_to_folder (Group * folder, const Article ** articles, int
 		const Article * article = articles[i];
 		const char * acache_key = group_get_acache_key (article->group);
 		MessageIdentifier * mid = message_identifier_new_from_article (article);
-		GMimeMessage * message = acache_get_message (acache_key, &mid, 1);
+		const PString * id = &mid->message_id;
+		GMimeMessage * message = acache_get_message (acache_key, &id, 1);
 
 		if (message == NULL)
 			missing[missing_qty++] = mid;
@@ -300,7 +302,7 @@ article_copy_articles_to_folder (Group * folder, const Article ** articles, int
 ***/
 
 static void
-save_attachments_impl (Article ** articles, int len)
+save_attachments_impl (Article ** articles, guint len, gpointer unused)
 {
 	int i;
 	GSList * tasks = NULL;
@@ -344,120 +346,153 @@ save_attachments_impl (Article ** articles, int len)
 void
 article_action_selected_save_attachments (void)
 {
-	GPtrArray * sel;
 	debug_enter ("article_action_selected_save_attachments");
 
-	/* decode each selected article */
-	sel = articlelist_get_selected_articles_nolock ();
-	save_attachments_impl ((Article**)sel->pdata, sel->len);
-	g_ptr_array_free (sel, TRUE);
+	header_pane_forall_selected (save_attachments_impl, NULL, FALSE);
 
 	debug_exit ("article_action_selected_save_attachments");
 }
 
+typedef struct
+{
+	Group * group;
+	GSList * slists_of_articles;
+}
+SelectedSaveAsStruct;
+
+static void
+add_article_grouping_to_gslist (Article * article, gpointer user_data)
+{
+	GSList * l = NULL;
+	SelectedSaveAsStruct * data = (SelectedSaveAsStruct*) user_data;
+
+	/* get the first group */
+	if (data->group == NULL)
+		data->group = article->group;
+
+	/* build an array of this node and all its children. */
+	if (article->parts > 1)
+		l = g_slist_copy (article->threads);
+	l = g_slist_prepend (l, article);
+
+	/* add this to the gslist of articles */
+	data->slists_of_articles = g_slist_prepend (data->slists_of_articles, l);
+}
+
 void
 article_action_selected_save_as (void)
 {
-	guint i;
-	GPtrArray * sel;
-	Group * group = articlelist_get_group ();
-	GSList * gslists_of_articles = NULL;
-	debug_enter ("articlelist_selected_decode_as");
+	SelectedSaveAsStruct data;
+	data.group = NULL;
+	data.slists_of_articles = NULL;
 
-	g_return_if_fail (group != NULL);
+	/* build the list of articles */
+	header_pane_foreach_selected (add_article_grouping_to_gslist, &data);
 
-	/* loop through all the selected articles */
-	sel = articlelist_get_selected_articles_nolock ();
-	for (i=0; i!=sel->len; ++i)
-	{
-		GSList * l = NULL;
-		Article * article = ARTICLE(g_ptr_array_index(sel,i));
-
-		/* build an array of this node and all its children. */
-		if (article->parts > 1)
-			l = g_slist_copy (article->threads);
-		l = g_slist_prepend (l, article);
-
-		/* add this to the gslist of articles */
-		gslists_of_articles = g_slist_prepend (gslists_of_articles, l);
+	/* if any articles are selected, save them */
+	if (data.slists_of_articles != NULL) {
+		data.slists_of_articles = g_slist_reverse (data.slists_of_articles);
+		save_attachment_as (data.slists_of_articles, data.group->download_dir);
+		/* save_attach_as owns slists_of_articles now */
 	}
-
-	/* save_attach_as owns gslists_of_articles now */
-	gslists_of_articles = g_slist_reverse (gslists_of_articles);
-	save_attachment_as (group, gslists_of_articles);
-
-	g_ptr_array_free (sel, TRUE);
-	debug_exit ("articlelist_selected_decode_as");
 }
 
 /***
 ****   DELETING ARTICLES
 ***/
 
+static void
+add_articles_and_multiparts_to_array (Article * article, gpointer user_data)
+{
+	GPtrArray * articles = (GPtrArray *) user_data;
+
+	/* add the article */
+	g_ptr_array_add (articles, article);
+
+	/* if it's the first in a multipart, add the other parts */
+	if (article->part==1 && article->parts>1) {
+		GSList * l;
+		for (l=article->threads; l!=NULL; l=l->next)
+			if (ARTICLE(l->data)->part > 1)
+				g_ptr_array_add (articles, l->data);
+	}
+}
+
+void
+article_action_delete_articles (Article ** articles, guint article_qty, gpointer unused)
+{
+	debug_enter ("article_action_delete_articles");
+
+	/* delete the articles */
+	if (article_qty)
+	{
+		int i;
+		Group * group;
+		const PString ** ids = g_newa (const PString*, article_qty);
+		MessageIdentifier ** mids;
+
+		/* remove the headers */
+		mids = g_new (MessageIdentifier*, article_qty);
+		for (i=0; i<article_qty; ++i)
+			mids[i] = message_identifier_new_from_article (articles[i]);
+		message_identifiers_delete ((const MessageIdentifier**)mids, article_qty, SERVER_GROUPS_SUBSCRIBED);
+
+		/* remove the cached bodies */
+		for (i=0; i<article_qty; ++i)
+			ids[i] = &mids[i]->message_id;
+	       	group = articles[0]->group;
+		acache_expire_messages (group_get_acache_key(group), ids, article_qty);
+
+		/* cleanup */
+		for (i=0; i<article_qty; ++i)
+			g_object_unref (mids[i]);
+		g_free (mids);
+	}
+
+	debug_exit ("article_action_delete_articles");
+}
+
 void
 article_action_delete_selected_articles (void)
 {
 	GPtrArray * articles;
-	int i, len;
 	debug_enter ("article_action_delete_selected_articles");
 
 	/* if it's a multipart, delete the children too */
-       	articles = articlelist_get_active_articles_nolock ();
-	for (i=0, len=articles->len; i!=len; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		if (a->part==1 && a->parts>1) {
-			GSList * l;
-			for (l=a->threads; l!=NULL; l=l->next)
-				if (ARTICLE(l->data)->part > 1)
-					g_ptr_array_add (articles, l->data);
-		}
-	}
+	articles = g_ptr_array_new ();
+	header_pane_foreach_active (add_articles_and_multiparts_to_array, articles);
 
-	/* delete the articles */
+	/* any articles to delete? */
 	if (articles->len)
-	{
-		Group * group;
-		MessageIdentifier ** mids;
-
-		/* remove the headers */
-		mids = g_new (MessageIdentifier*, articles->len);
-		for (i=0; i<articles->len; ++i)
-			mids[i] = message_identifier_new_from_article (ARTICLE(g_ptr_array_index(articles,i)));
-		message_identifiers_delete ((const MessageIdentifier**)mids, articles->len, SERVER_GROUPS_SUBSCRIBED);
-
-		/* remove the cached bodies */
-	       	group = articlelist_get_group ();
-		acache_expire_message_ids (group_get_acache_key(group), (const MessageIdentifier**)mids, articles->len);
-
-		/* cleanup */
-		for (i=0; i<articles->len; ++i)
-			g_object_unref (mids[i]);
-		g_free (mids);
-	}
+		article_action_delete_articles ((Article**)articles->pdata, articles->len, NULL);
 
 	/* cleanup */
 	g_ptr_array_free (articles, TRUE);
 	debug_exit ("article_action_delete_selected_articles");
 }
 
-void
-article_action_edit_selected (void)
+static void
+edit_article (Article * article, gpointer unused)
 {
-        Article * a = NULL;
 	MessageIdentifier * mid = NULL;
 	GMimeMessage * message = NULL;
-	debug_enter ("article_action_edit_selected");
 
-	a = articlelist_get_selected_article_nolock ();
-	if (a != NULL)
-		mid = message_identifier_new_from_article (a);
-	if (mid != NULL)
-		message = acache_get_message (group_get_acache_key(a->group), &mid, 1);
+	if (article != NULL)
+		mid = message_identifier_new_from_article (article);
+	if (mid != NULL) {
+		const PString * id = &mid->message_id;
+		message = acache_get_message (group_get_acache_key(article->group), &id, 1);
+	}
 	if (message != NULL)
 		message_edit_window (message);
 
 	g_object_unref (mid);
-	debug_exit ("article_action_edit_selected");
+}
+
+void
+article_action_edit_selected (void)
+{
+	header_pane_first_selected (edit_article, NULL);
 }
 
 /***
@@ -474,7 +509,7 @@ static void
 article_action_something_thread (const Article * article, int action)
 {
 	ScoreAddItem add_items[2];
-	GPtrArray * articles;
+	char * section_name;
 	char * score_name;
 	char * thread_message_id;
 	struct {
@@ -498,6 +533,8 @@ article_action_something_thread (const Article * article, int action)
 	/* if the article's Message-ID is the top article, 
 	 * the article's Referencess contains the top article,
 	 * set the score to watch/ignore */
+
+	section_name = score_create_section_str(article);
 	add_items[0].on = TRUE;
 	add_items[0].negate = FALSE;
 	add_items[0].key = HEADER_MESSAGE_ID;
@@ -506,18 +543,13 @@ article_action_something_thread (const Article * article, int action)
 	add_items[1].negate = FALSE;
 	add_items[1].key = HEADER_REFERENCES;
 	add_items[1].value = filter_phrase_create_regex (thread_message_id, PHRASE_MATCH_STARTS_WITH);
-	score_add (score_name, NULL, do_list[action].score, TRUE, 31, AGGREGATE_TYPE_OR, add_items, 2);
-
-	/* rescore the thread */
-	articles = g_ptr_array_new ();
-	article_get_thread (article, articles);
-	ensure_articles_scored (article->group, (Article**)articles->pdata, articles->len);
+	score_add (score_name, section_name, do_list[action].score, TRUE, 31, AGGREGATE_TYPE_OR, add_items, 2, TRUE);
 
 	/* cleanup */
-	g_ptr_array_free (articles, TRUE);
 	g_free (add_items[1].value);
 	g_free (add_items[0].value);
 	g_free (score_name);
+	g_free (section_name);
 	g_free (thread_message_id);
 }
 
@@ -527,12 +559,15 @@ article_action_watch_thread (const Article * article)
 	article_action_something_thread (article, DO_WATCH);
 }
 
+static void
+watch_article (Article * article, gpointer user_data)
+{
+	article_action_watch_thread (article);
+}
 void
 article_action_watch_selected_thread (void)
 {
-	Article * article = articlelist_get_selected_article_nolock ();
-	if (article != NULL)
-		article_action_watch_thread (article);
+	header_pane_first_selected (watch_article, NULL);
 }
 
 void
@@ -540,11 +575,94 @@ article_action_ignore_thread (const Article * article)
 {
 	article_action_something_thread (article, DO_IGNORE);
 }
-
+static void
+ignore_article (Article * article, gpointer user_data)
+{
+	article_action_ignore_thread (article);
+}
 void
 article_action_ignore_selected_thread (void)
 {
-	Article * article = articlelist_get_selected_article_nolock ();
-	if (article != NULL)
-		article_action_ignore_thread (article);
+	header_pane_first_selected (ignore_article, NULL);
+}
+
+/**
+***  Mark read/unread
+**/
+
+static void
+header_pane_mark_articles_nolock (Article ** articles, guint article_qty, gpointer user_data)
+{
+	const gboolean read = user_data!=NULL;
+
+	articles_set_read (articles, article_qty, read);
+}
+
+void
+article_action_mark_selected_unread (void)
+{
+	header_pane_forall_active (header_pane_mark_articles_nolock, GINT_TO_POINTER(FALSE));
+}
+void 
+article_action_mark_selected_read (void)
+{
+	header_pane_forall_active (header_pane_mark_articles_nolock, GINT_TO_POINTER(TRUE));
+}
+
+/**
+***  Download
+**/
+
+static void
+queue_task_bodies (Article ** articles, guint article_qty, gpointer user_data)
+{
+	queue_add (TASK(task_bodies_new_from_articles ((const Article**)articles, article_qty)));
+}
+void
+article_action_download_selected (void)
+{
+	header_pane_forall_active (queue_task_bodies, NULL);
+}
+
+/**
+***  Cancel / Supersede
+**/
+
+static void
+cancel_article (Article * article, gpointer user_data)
+{
+	const PString * id = &article->message_id;
+	GMimeMessage * message = acache_get_message (group_get_acache_key(article->group), &id, 1);
+
+	if (message == NULL)
+		log_add (LOG_ERROR|LOG_URGENT, _("You must download the article before you can cancel it."));
+	else {
+		article_cancel (article->group->server, message);
+		g_object_unref (message);
+	}
+}
+void
+article_action_cancel_selected (void)
+{
+	header_pane_first_selected (cancel_article, NULL);
+}
+
+static void
+supersede_article (Article * article, gpointer user_data)
+{
+	const PString * id = &article->message_id;
+	GMimeMessage * message = acache_get_message (group_get_acache_key(article->group), &id, 1);
+
+	if (message == NULL)
+		log_add (LOG_ERROR|LOG_URGENT, _("You must download the article before you can supersede it."));
+	else {
+		article_supersede (message);
+		g_object_unref (message);
+	}
+}
+
+void
+article_action_supersede_selected (void)
+{
+	header_pane_first_selected (supersede_article, NULL);
 }
