@@ -60,7 +60,6 @@ typedef struct
 void spa_bits_to_base64 (unsigned char *, const unsigned char *, int);
 int spa_base64_to_bits (char *out, const char *in);
 void spa_build_auth_response (SPAAuthChallenge *challenge, 
-	SPAAuthResponse *response, char *user, char *password);
-void spa_build_auth_request (SPAAuthRequest *request, char *user, 
-	char *domain);
+	SPAAuthResponse *response, const char *user, const char *password);
+void spa_build_auth_request (SPAAuthRequest *request, const char *user, const char *domain);
 
