@@ -75,7 +75,7 @@ typedef struct
 {
 	AcachePath * path;
 	int refcount;
-	char * message_id;
+	PString * message_id;
 	size_t size;
 	time_t date;
 }
@@ -103,9 +103,9 @@ acache_get_bodies_added_callback (void)
 }
 
 static void
-fire_bodies_added (const char ** message_ids, int qty)
+fire_bodies_added (const PString ** mids, int qty)
 {
-	pan_callback_call (acache_get_bodies_added_callback(), message_ids, GINT_TO_POINTER(qty));
+	pan_callback_call (acache_get_bodies_added_callback(), mids, GINT_TO_POINTER(qty));
 }
 
 PanCallback*
@@ -117,7 +117,7 @@ acache_get_bodies_removed_callback (void)
 }
 
 static void
-fire_bodies_removed (const char ** message_ids, int qty)
+fire_bodies_removed (const PString ** message_ids, int qty)
 {
 	pan_callback_call (acache_get_bodies_removed_callback(), message_ids, GINT_TO_POINTER(qty));
 }
@@ -147,20 +147,21 @@ acache_get_folder_base (void)
  * so we transform them to a safer name.
  */
 static char*
-acache_message_id_to_basename (char * buf, int len, const char * message_id)
+acache_message_id_to_basename (char * buf, int len, const PString * message_id)
 {
 	const char * in;
 	char * out;
 	debug_enter ("acache_message_id_to_basename");
 
 	/* sanity clause */
-	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+	g_return_val_if_fail (message_id!=NULL, NULL);
+	g_return_val_if_fail (message_id->str!=NULL, NULL);
 	g_return_val_if_fail (buf!=NULL, NULL);
 	g_return_val_if_fail (len>0, NULL);
 
 	/* some characters in message-ids are illegal on older Windows boxes,
 	 * so we transform those illegal characters using URL encoding */
-	for (in=message_id, out=buf; *in; ++in) {
+	for (in=message_id->str, out=buf; *in; ++in) {
 		switch (*in) {
 			case '%': /* this is the escape character */
 			case '"': case '*': case '/': case ':': case '?': case '|':
@@ -185,8 +186,10 @@ acache_message_id_to_basename (char * buf, int len, const char * message_id)
  * Message-IDs are transformed via acache_message_id_to_basename()
  * to play nicely with some filesystems, so to extract the Message-ID
  * from a basename we need to reverse the transform.
+ *
+ * @return string length, or 0 on failure
  */
-static char*
+static int
 acache_basename_to_message_id (char * buf, int len, const char * basename)
 {
 	const char * in;
@@ -196,9 +199,9 @@ acache_basename_to_message_id (char * buf, int len, const char * basename)
 	debug_enter ("acache_basename_to_message_id");
 
 	/* sanity clause */
-	g_return_val_if_fail (is_nonempty_string(basename), NULL);
-	g_return_val_if_fail (buf!=NULL, NULL);
-	g_return_val_if_fail (len>0, NULL);
+	g_return_val_if_fail (is_nonempty_string(basename), 0);
+	g_return_val_if_fail (buf!=NULL, 0);
+	g_return_val_if_fail (len>0, 0);
 
 	/* remove the trailing ".msg" */
 	g_strlcpy (tmp_basename, basename, sizeof(tmp_basename));
@@ -225,18 +228,18 @@ acache_basename_to_message_id (char * buf, int len, const char * basename)
 
 	/* cleanup */
 	debug_exit ("acache_basename_to_message_id");
-	return buf;
+	return out - buf;
 }
 
 static char*
-acache_get_filename (char * buf, int len, const char * path, const char * message_id)
+acache_get_filename (char * buf, int len, const char * path, const PString * message_id)
 {
 	char basename[PATH_MAX] = { '\0' };
 
 	/* sanity clause */
-	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
 	g_return_val_if_fail (buf!=NULL, NULL);
 	g_return_val_if_fail (len>0, NULL);
+	g_return_val_if_fail (pstring_is_set (message_id),NULL);
 
 	/* build the filename */
 	*buf = '\0';
@@ -266,7 +269,7 @@ acache_get_key_path_hash (const char * key_path)
 	/* ensure there's a hash table for this path. */
 	retval = g_hash_table_lookup (key_to_messageids, key_path);
 	if (retval == NULL) {
-		retval = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+		retval = g_hash_table_new_full (pstring_hash, pstring_equal, (GDestroyNotify)pstring_free, NULL);
 		g_hash_table_insert (key_to_messageids, g_strdup(key_path), retval);
 	}
 	
@@ -274,12 +277,14 @@ acache_get_key_path_hash (const char * key_path)
 }
 
 static AcacheEntry*
-acache_lookup_entry (const char* key_path, const char* message_id)
+acache_lookup_entry (const char* key_path, const PString * message_id)
 {
 	GHashTable * message_ids;
 
 	g_return_val_if_fail (is_nonempty_string(key_path), NULL);
-	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+	g_return_val_if_fail (message_id!=NULL, NULL);
+	g_return_val_if_fail (message_id->len>0, NULL);
+	g_return_val_if_fail (message_id->str!=NULL, NULL);
 
 	message_ids = acache_get_key_path_hash (key_path);
 
@@ -287,26 +292,29 @@ acache_lookup_entry (const char* key_path, const char* message_id)
 }
 
 static AcacheEntry*
-acache_get_entry (const char * key_path, const char * message_id)
+acache_get_entry (const char * key_path, const PString * message_id)
 {
 	AcacheEntry * entry = NULL;
 	GHashTable * message_ids;
 
+	/* sanity clause */
 	g_return_val_if_fail (is_nonempty_string(key_path), NULL);
-	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+	g_return_val_if_fail (pstring_is_set (message_id), NULL);
 
 	message_ids = acache_get_key_path_hash (key_path);
 	entry = g_hash_table_lookup (message_ids, message_id);
 	if (entry == NULL)
 	{
+		PString * mid_dup = pstring_dup (message_id);
+
 		entry = (AcacheEntry*) memchunk_alloc (acache_entry_chunk);
 		entry->path = NULL;
 		entry->refcount = 0;
-		entry->message_id = g_strdup (message_id);
+		entry->message_id = mid_dup;
 		entry->size = 0;
 		entry->date = time (NULL);
-		g_hash_table_replace (message_ids, entry->message_id, entry);
-		debug1 (DEBUG_ACACHE, "Added new entry %d", entry->message_id);
+		g_hash_table_replace (message_ids, mid_dup, entry);
+		debug1 (DEBUG_ACACHE, "Added new entry %s", mid_dup->str);
 	}
 
 	return entry;
@@ -324,11 +332,11 @@ acache_remove_entry (AcacheEntry * entry)
 	/* sanity clause */
 	g_return_if_fail (entry!=NULL);
 	g_return_if_fail (entry->path!=NULL);
-	g_return_if_fail (is_nonempty_string(entry->message_id));
+	g_return_if_fail (pstring_is_set (entry->message_id));
 
 	/* notify listeners that this message is going away */
 	debug2 (DEBUG_ACACHE, "Removing %s from cache with refcount %d", entry->message_id, entry->refcount);
-	fire_bodies_removed ((const char**)&entry->message_id, 1);
+	fire_bodies_removed ((const PString**)&entry->message_id, 1);
 
 	/* remove the file */
 	acache_get_filename (filename, sizeof(filename), entry->path->path, entry->message_id);
@@ -359,11 +367,11 @@ acache_path_free (AcachePath * path)
 	g_free (path);
 }
 
-static gboolean
+static int
 get_message_id_from_file (char * msgid, int msgid_len, const char * filename)
 {
 	const char * basename;
-	gboolean retval = FALSE;
+	int retval = 0;
 
 	/* sanity clause */
 	g_return_val_if_fail (is_nonempty_string(filename), FALSE);
@@ -374,7 +382,7 @@ get_message_id_from_file (char * msgid, int msgid_len, const char * filename)
 	*msgid = '\0';
 	basename = g_basename (filename);
 	if (basename != NULL)
-		retval = acache_basename_to_message_id (msgid, msgid_len, basename) != NULL;
+		retval = acache_basename_to_message_id (msgid, msgid_len, basename);
 
 	return retval;
 }
@@ -401,9 +409,9 @@ acache_init (void)
 
 	/* add the folder server */
 	server = server_new ();
-	server->address = g_strdup ("Mock Server for Folders");
-	server->name = g_strdup (INTERNAL_SERVER_NAME);
-	serverlist_add_server (server);
+	pstring_set (&server->address, "Mock Server for Folders", -1);
+	pstring_copy (&server->name, &INTERNAL_SERVER_NAME);
+	serverlist_add_server (server, NULL);
 
 	/* ensure the folder directories exist */
 	message_base = acache_get_message_base ();
@@ -413,12 +421,12 @@ acache_init (void)
 
 	/* ensure pan.sent exists */
 	folder_base = acache_get_folder_base ();
-	path = g_build_filename (folder_base, PAN_SENT, NULL);
+	path = g_build_filename (folder_base, PAN_SENT.str, NULL);
 	pan_file_ensure_path_exists (path);
 	replace_gstr (&path, NULL);
 
 	/* ensure pan.sendlater exists */
-	path = g_build_filename (folder_base, PAN_SENDLATER, NULL);
+	path = g_build_filename (folder_base, PAN_SENDLATER.str, NULL);
 	pan_file_ensure_path_exists (path);
 	replace_gstr (&path, NULL);
 
@@ -495,22 +503,23 @@ compare_ppEntry_ppEntry_by_youth (gconstpointer a, gconstpointer b, gpointer unu
 }
 
 void
-acache_expire_messages (const char   *  path_key,
-                        const char   ** message_ids,
-	                int             message_id_qty)
+acache_expire_messages (const char                * path_key,
+                        const PString * const     * mids,
+                        int                         mid_qty)
 {
 	/* sanity clause */
 	g_return_if_fail (is_nonempty_string (path_key));
-	g_return_if_fail (message_ids!=NULL);
-	g_return_if_fail (message_id_qty>0);
+	g_return_if_fail (mid_qty >= 0);
+	g_return_if_fail (mids != NULL);
 
+	/* do the work */
 	g_static_mutex_lock (&entries_mutex);
 	{
 		int i;
 
-		for (i=0; i<message_id_qty; ++i)
+		for (i=0; i<mid_qty; ++i)
 		{
-			AcacheEntry * entry = acache_lookup_entry (path_key, message_ids[i]);
+			AcacheEntry * entry = acache_lookup_entry (path_key, mids[i]);
 			if (entry != NULL)
 				acache_remove_entry (entry);
 		}
@@ -520,26 +529,6 @@ acache_expire_messages (const char   *  path_key,
 	g_static_mutex_unlock (&entries_mutex);
 }
 
-void
-acache_expire_message_ids (const char                * path_key,
-                           const MessageIdentifier  ** mids,
-                           int                         mid_qty)
-{
-	int i;
-	const char ** message_ids;
-
-	/* sanity clause */
-	g_return_if_fail (is_nonempty_string (path_key));
-	g_return_if_fail (mid_qty >= 0);
-	g_return_if_fail (message_identifiers_are_valid ((const MessageIdentifier **)mids, mid_qty));
-
-	/* let acache_expire_messages() do all the work */
-	message_ids = g_newa (const char*, mid_qty);
-	for (i=0; i<mid_qty; ++i)
-		message_ids[i] = mids[i]->message_id;
-	acache_expire_messages (path_key, message_ids, mid_qty);
-}
-
 static int
 acache_expire_to_size (AcachePath * apath, double max_megs)
 {
@@ -616,20 +605,20 @@ acache_expire_all (void)
 
 /* this must be called inside an entries_mutex lock */
 static void
-acache_update_refcount_nolock (const char         * path_key,
-                               MessageIdentifier ** identifiers,
-                               int                  identifier_qty,
-                               int                  inc)
+acache_update_refcount_nolock (const char                * path_key,
+                               const PString * const     * mids,
+                               int                         mid_qty,
+                               int                         inc)
 {
 	int i;
 	debug_enter ("acache_update_refcount_nolock");
 
-	g_return_if_fail (identifier_qty >= 1);
-	g_return_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)identifiers, identifier_qty));
+	g_return_if_fail (mid_qty >= 1);
+	g_return_if_fail (mids != NULL);
 
-	for (i=0; i<identifier_qty; ++i)
+	for (i=0; i<mid_qty; ++i)
 	{
-		AcacheEntry * entry = acache_get_entry (path_key, identifiers[i]->message_id);
+		AcacheEntry * entry = acache_get_entry (path_key, mids[i]);
 
 		/* If we're checking it out,
 		 * then move it to the safe end of the least-recently-used kill heuristic */
@@ -644,42 +633,40 @@ acache_update_refcount_nolock (const char         * path_key,
 }
 
 void
-acache_checkout (const char          * key,
-                 MessageIdentifier  ** identifiers,
-                 int                   identifier_qty)
+acache_checkout (const char               * key,
+                 const PString * const    * mids,
+                 int                        mid_qty)
 {
 	debug_enter ("acache_checkout");
 
 	/* sanity clause */
 	g_return_if_fail (is_nonempty_string(key));
-	g_return_if_fail (identifiers!=NULL);
-	g_return_if_fail (identifier_qty >= 1);
-	g_return_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)identifiers, identifier_qty));
+	g_return_if_fail (mids!=NULL);
+	g_return_if_fail (mid_qty >= 1);
 
 	/* ref the entries */
 	g_static_mutex_lock (&entries_mutex);
-	acache_update_refcount_nolock (key, identifiers, identifier_qty, 1);
+	acache_update_refcount_nolock (key, mids, mid_qty, 1);
 	g_static_mutex_unlock (&entries_mutex);
 
 	debug_exit ("acache_checkout");
 }
 
 void
-acache_checkin (const char          * key,
-                MessageIdentifier  ** identifiers,
-                int                   identifier_qty)
+acache_checkin (const char                * key,
+                const PString * const     * mids,
+                int                         mid_qty)
 {
 	debug_enter ("acache_checkin");
 
 	/* sanity clause */
 	g_return_if_fail (is_nonempty_string (key));
-	g_return_if_fail (identifiers!=NULL);
-	g_return_if_fail (identifier_qty >= 1);
-	g_return_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)identifiers, identifier_qty));
+	g_return_if_fail (mids!=NULL);
+	g_return_if_fail (mid_qty >= 1);
 
 	/* unref the entries */
 	g_static_mutex_lock (&entries_mutex);
-	acache_update_refcount_nolock (key, identifiers, identifier_qty, -1);
+	acache_update_refcount_nolock (key, mids, mid_qty, -1);
 	g_static_mutex_unlock (&entries_mutex);
 
 	acache_expire ();
@@ -694,10 +681,10 @@ acache_checkin (const char          * key,
 ***/
 
 void
-acache_set_message (const char    * key,
-                    const char    * message_id,
-                    const char    * message,
-                    guint           message_len)
+acache_set_message (const char        * key,
+                    const PString     * message_id,
+                    const char        * message,
+                    guint               message_len)
 {
        	FILE * fp = NULL;
 	char filename[PATH_MAX];
@@ -705,8 +692,8 @@ acache_set_message (const char    * key,
 
 	/* sanity clause */
 	g_return_if_fail (is_nonempty_string(key));
+	g_return_if_fail (pstring_is_set(message_id));
 	g_return_if_fail (is_nonempty_string(message));
-	g_return_if_fail (is_nonempty_string(message_id));
 	g_return_if_fail (message_len > 0);
 
 	/* get the path */
@@ -817,9 +804,9 @@ acache_get_message_mem_stream (const AcacheEntry * entry)
 }
 
 GMimeMessage*
-acache_get_message  (const char          * path_key,
-                     MessageIdentifier  ** mids,
-                     int                   mid_qty)
+acache_get_message  (const char                * path_key,
+                     const PString * const     * mids,
+                     int                         mid_qty)
 {
 	int i;
 	int stream_qty;
@@ -829,14 +816,14 @@ acache_get_message  (const char          * path_key,
 	/* sanity clause */
 	g_return_val_if_fail (is_nonempty_string(path_key), NULL);
 	g_return_val_if_fail (mid_qty>0, NULL);
-	g_return_val_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)mids, mid_qty), NULL);
+	g_return_val_if_fail (mids!=NULL, NULL);
 
 	/* get streams */
 	streams = g_newa (GMimeStream*, mid_qty);
 	for (i=stream_qty=0; i<mid_qty; ++i)
 	{
 		GMimeStream * stream;
-		const AcacheEntry * entry = acache_lookup_entry (path_key, mids[i]->message_id);
+		const AcacheEntry * entry = acache_lookup_entry (path_key, mids[i]);
 		if (entry == NULL)
 			break;
 
@@ -861,15 +848,15 @@ acache_get_message  (const char          * path_key,
 }
 
 gboolean
-acache_has_message (const char * path_key,
-                    const char * message_id)
+acache_has_message (const char        * path_key,
+                    const PString     * message_id)
 {
 	gboolean retval = FALSE;
 	AcacheEntry * entry = NULL;
 
 	/* sanity clause */
 	g_return_val_if_fail (is_nonempty_string(path_key), FALSE);
-	g_return_val_if_fail (is_nonempty_string(message_id), FALSE);
+	g_return_val_if_fail (pstring_is_set(message_id), FALSE);
 
 	/* lookup the message */
 	entry = acache_lookup_entry (path_key, message_id);
@@ -980,7 +967,7 @@ acache_add_path (const char * key,
 	else
 	{
 		const char * fname;
-		char message_id[2048];
+		char str[2048];
 		char filename[PATH_MAX];
 
 		while ((fname = g_dir_read_name (dir)))
@@ -990,15 +977,19 @@ acache_add_path (const char * key,
 			g_snprintf (filename, sizeof(filename), "%s%c%s", path, G_DIR_SEPARATOR, fname);
 			if (stat (filename, &stat_p) == 0)
 			{
-				if (get_message_id_from_file (message_id, sizeof(message_id), filename))
+				int len = get_message_id_from_file (str, sizeof(str), filename);
+
+				if (len != 0)
 				{
+					const PString mid = pstring_shallow (str, len);
+
 					/*
 					 * FIXME: this may segfault if acache 
 					 * is shut down while this thread is 
 					 * still running.
 					 */
 
-					AcacheEntry * entry = acache_get_entry (key, message_id);
+					AcacheEntry * entry = acache_get_entry (key, &mid);
 					entry->path = apath;
 					entry->size = stat_p.st_size;
 					entry->date = stat_p.st_mtime;
