@@ -51,8 +51,9 @@
  */
 typedef struct
 {
-	gchar * subject;
 	gboolean is_reply;
+	int subject_len;
+	char * subject;
 	Article * a;
 }
 Norm;
@@ -62,9 +63,10 @@ Norm;
  *
  * 1. tearing out the numerator from multipart indicators
  *    (i.e., remove "21" from (21/42))
- *    (why?)
- * 2. convert it to lowercase so that we can use strcmp
- *    instead of g_strcasecmp, which is slower.
+ *    (for?)
+ * 2. convert it to lowercase so that, when sorting, we can
+ *    have case-insensitive sorting without having to use
+ *    a slower case-insensitive compare function.
  * 3. strip out all the leading noise that breaks sorting.
  *
  * When we're threading articles, it's much faster to
@@ -72,11 +74,11 @@ Norm;
  * normalizing them for each comparison.
  */
 static size_t
-normalize_subject (char * buf, const Article * a)
+normalize_subject (char * buf, const Article * a, gboolean strip_case)
 {
 	static gboolean _inited = FALSE;
 	static gboolean _keep_chars[UCHAR_MAX+1];
-	register const guchar * in = (guchar*) skip_reply_leader (a->subject);
+	register const guchar * in = (guchar*) skip_reply_leader (a->subject.str);
 	register char * out = buf;
 	const gboolean multipart = a->parts != 0;
 
@@ -108,7 +110,7 @@ normalize_subject (char * buf, const Article * a)
 
 		/* strip out junk that breaks sorting */
 		if (_keep_chars[*in])
-			*out++ = (char) tolower(*in);
+			*out++ = (char) (strip_case ? tolower(*in) : *in);
 
 		++in;
 	}
@@ -125,12 +127,13 @@ normalize_subject (char * buf, const Article * a)
 static void
 normalize_articles (Article    ** articles,
                     gint          qty,
+                    gboolean      strip_case,
                     Norm       ** alloc_and_setme_norm,
                     gchar      ** alloc_and_setme_str)
 {
 	gint i;
 	glong str_buf_idx;
-	glong str_len;
+	glong len;
 	gchar * str_buf;
 	Norm * norm_buf;
 
@@ -145,19 +148,20 @@ normalize_articles (Article    ** articles,
 	*alloc_and_setme_norm = norm_buf = g_new (Norm, qty);
 
 	/* alloc a buf for the subject */
-	str_len = 0;
+	len = 0;
 	for (i=0; i<qty; ++i)
-		str_len += strlen (articles[i]->subject) + 2;
-	*alloc_and_setme_str = str_buf = g_new (char, str_len);
+		len += articles[i]->subject.len + 2;
+	*alloc_and_setme_str = str_buf = g_new (char, len);
 	
 	/* normalize the articles */
 	str_buf_idx = 0;
 	for (i=0; i<qty; ++i) {
 		Article * a = articles[i];
 		norm_buf[i].a = a;
-		norm_buf[i].is_reply = skip_reply_leader (a->subject) != a->subject ? 1 : 0;
+		norm_buf[i].is_reply = skip_reply_leader (a->subject.str) != a->subject.str ? 1 : 0;
 		norm_buf[i].subject = str_buf + str_buf_idx;
-		str_buf_idx += normalize_subject (norm_buf[i].subject, a) + 1;
+		norm_buf[i].subject_len = normalize_subject (norm_buf[i].subject, a, strip_case);
+		str_buf_idx += norm_buf[i].subject_len + 1;
 	}
 }
 
@@ -176,15 +180,15 @@ compare_pA_to_pA_by_part (const void * va, const void * vb)
 		return a->part - b->part;
 
 	/* order by subject */
-	if ((value = strcmp (a->subject, b->subject)))
+	if ((value = pstring_compare (&a->subject, &b->subject)))
 		return value;
 
 	/* the rest is probably unnecessary. just carry-over from
 	 * compare_pA_to_pA_by_subject */
 
 	/* if one but not both is a reply, the reply goes second */
-	a_is_reply = skip_reply_leader (a->subject) != a->subject ? 1 : 0;
-	b_is_reply = skip_reply_leader (b->subject) != b->subject ? 1 : 0;
+	a_is_reply = skip_reply_leader (a->subject.str) != a->subject.str ? 1 : 0;
+	b_is_reply = skip_reply_leader (b->subject.str) != b->subject.str ? 1 : 0;
 	if (a_is_reply != b_is_reply)
 		return a_is_reply ? 1 : -1;
 
@@ -208,7 +212,7 @@ compare_pN_to_pN_by_subject (const void * va, const void * vb)
 	/* subject is the primary key, of course... */
 	if ((value = *a->subject - *b->subject))
 		return value;
-	if ((value = strcmp (a->subject, b->subject)))
+	if ((value = pan_strcmp_len (a->subject, a->subject_len, b->subject, b->subject_len)))
 		return value;
 
 	/* if one but not both is a reply, the reply goes second */
@@ -280,8 +284,8 @@ compare_ppA_to_ppA_by_action (const void * va, const void * vb)
 	if (ia != ib)
 		return ib - ia;
 
-	ia = acache_has_message (group_get_acache_key(a->group), a->message_id) ? 1 : 0;
-	ib = acache_has_message (group_get_acache_key(b->group), b->message_id) ? 1 : 0;
+	ia = acache_has_message (group_get_acache_key(a->group), &a->message_id) ? 1 : 0;
+	ib = acache_has_message (group_get_acache_key(b->group), &b->message_id) ? 1 : 0;
 	if (ia != ib)
 		return ib - ia;
 
@@ -340,16 +344,17 @@ compare_ppA_to_ppA_by_date (const void* va, const void* vb)
 }
 
 static int
-compare_ppA_to_ppA_by_message_id (const void* a, const void* b)
+compare_ppA_to_ppA_by_message_id (const void* va, const void* vb)
 {
-	const gchar * msg_id_a = (*(const Article**)a)->message_id;
-	const gchar * msg_id_b = (*(const Article**)b)->message_id;
-	return strcmp (msg_id_a, msg_id_b);
+	register const Article * a = *(const Article **)va;
+	register const Article * b = *(const Article **)vb;
+	return pstring_compare (&a->message_id, &b->message_id);
 }
 
 typedef struct
 {
 	char name[128];
+	int name_len;
 	Article * article;
 }
 ArticleStruct;
@@ -359,7 +364,7 @@ compare_pAS_to_pAS_by_data (const void * va, const void * vb)
 {
 	const ArticleStruct * a = (const ArticleStruct*)va;
 	const ArticleStruct * b = (const ArticleStruct*)vb;
-	return strcmp (a->name, b->name);
+	return pan_strcmp_len (a->name, a->name_len, b->name, b->name_len);
 }
 
 void
@@ -382,7 +387,7 @@ sort_articles (Article      ** buf,
 			for (i=0; i<article_qty; ++i)
 			{
 				as[i].article = buf[i];
-				article_get_short_author_str (buf[i], as[i].name, sizeof(as[i].name));
+				as[i].name_len = article_get_short_author_str (buf[i], as[i].name, sizeof(as[i].name));
 				g_strdown (as[i].name);
 			}
 			msort (as,
@@ -430,7 +435,7 @@ sort_articles (Article      ** buf,
 			gint i;
 			Norm * norm_buf = NULL;
 			char * str_buf = NULL;
-			normalize_articles (buf, article_qty, &norm_buf, &str_buf);
+			normalize_articles (buf, article_qty, TRUE, &norm_buf, &str_buf);
 			msort (norm_buf, article_qty, sizeof(Norm), compare_pN_to_pN_by_subject);
 			for (i=0; i<article_qty; ++i)
 				buf[i] = ARTICLE(norm_buf[i].a);
@@ -491,7 +496,7 @@ article_root_get_part_state (const Article * a)
 		retval = MULTIPART_STATE_SOME;
 
 	/* someone's posted a followup to a multipart */
-	else if (a->linecount<250 && !g_strncasecmp(a->subject,"Re:", 3))
+	else if (a->linecount<250 && !g_strncasecmp(a->subject.str,"Re:", 3))
 		retval = MULTIPART_STATE_NONE;
 
 	/* a multipart */
@@ -529,23 +534,22 @@ set_children_part_state (Article * a, const int state)
  * Thread the articles specified in list
  */
 void
-thread_articles (GPtrArray    * articles)
+thread_articles (Article   ** articles,
+                 guint        article_qty)
 {
 	guint i;
-	guint qty = articles->len;
 	Article search_a;
 	gchar * norm_str_buf;
 	Norm * norm;
 	Norm * sorted_norm;
 	GArray * buf = NULL;
-	GString * tmp_references;
-	GHashTable * ref_hash;
+	GHashTable * mid_to_article;
 
 	/* sanity clause */
 	g_return_if_fail (articles!=NULL);
-	g_return_if_fail (articles_are_valid ((const Article**)articles->pdata, articles->len));
+	g_return_if_fail (articles_are_valid ((const Article**)articles, article_qty));
 
-	if (qty<1 || !articles)
+	if (article_qty<1 || !articles)
 		return;
 
 	if (break_thread_when_subject_changes)
@@ -553,57 +557,57 @@ thread_articles (GPtrArray    * articles)
 
 	/* make a plausiably-legal article */
 	search_a.number = 1;
-	search_a.subject = "dummy subject";
-	search_a.references = NULL;
+	search_a.subject = pstring_shallow ("dummy subject", -1);
+	search_a.references = PSTRING_INIT;
 
 	/* make a message-id-sorted array of the articles */
-	ref_hash = g_hash_table_new (g_str_hash, g_str_equal);
-	for (i=0; i<qty; ++i) {
-		Article * a = ARTICLE (g_ptr_array_index (articles, i));
-		g_hash_table_insert (ref_hash, (gpointer)a->message_id, a);
-	}
+	mid_to_article = g_hash_table_new (pstring_hash, pstring_equal);
+	for (i=0; i<article_qty; ++i)
+		g_hash_table_insert (mid_to_article, &articles[i]->message_id, articles[i]);
 
 	/* normalize the articles */
 	norm = NULL;
 	norm_str_buf = NULL;
-	normalize_articles ((Article**)articles->pdata, qty, &norm, &norm_str_buf);
+	normalize_articles (articles, article_qty, FALSE, &norm, &norm_str_buf);
 
 	/* sort the normalized articles */
-	sorted_norm = g_memdup (norm, sizeof(Norm)*qty);
-	qsort (sorted_norm, qty, sizeof(Norm), compare_pN_to_pN_by_subject);
+	sorted_norm = g_memdup (norm, sizeof(Norm)*article_qty);
+	qsort (sorted_norm, article_qty, sizeof(Norm), compare_pN_to_pN_by_subject);
 
 	/* unthread the articles, just in case they were threaded before */
-	for (i=0; i!=qty; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+	for (i=0; i!=article_qty; ++i)
+	{
+		Article * a = articles[i];
 		a->parent = NULL;
-		a->unread_children = 0;
-		a->new_children = 0;
 		g_slist_free (a->threads);
 		a->threads = NULL;
 	}
 
 
 	/* thread the articles */
-       	tmp_references = g_string_new (NULL);
-	for (i=0; i!=qty; ++i)
+	for (i=0; i!=article_qty; ++i)
 	{
-		const gchar * references;
+		PString references;
 		Article * parent = NULL;
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+		Article * a = articles[i];
 		gint index = -1;
 
-		/* thread by reference
-		   (except for parts 2...n of multiparts, which need to be threaded by multipart) */
+		/* thread by reference.
+		   (except for parts 2...n of multiparts, which need to be threaded by multipart).
+		   We take the references header and work our way from the last message-id backwards
+		   to the first.  For each message-id in this series, the first one we find in our
+		   set of articles is flagged as the parent. */
 		references = a->references;
-		if (a->parts<2 && references!=NULL && *references=='<')
+		if (a->parts<2 && pstring_is_set(&a->references))
 		{
-			char * message_id;
-			g_string_assign (tmp_references, references);
-		       	message_id = strrchr (tmp_references->str, '<');
+			/* pull the last message-id from the references string. */
+			const char * cpch = pan_strrchr_len (references.str, references.len, '<');
+			PString message_id = pstring_substr_shallow (&references, cpch, NULL);
 
-			while (parent==NULL && is_nonempty_string(message_id))
+			while (parent==NULL && pstring_is_set (&message_id))
 			{
-				Article * match = (Article*) g_hash_table_lookup (ref_hash, message_id);
+				/* look for a match in our message-id hashtable */
+				Article * match = (Article*) g_hash_table_lookup (mid_to_article, &message_id);
 
 				/* if we found the ancestor & it's worthy, thread it */
 				if (match!=NULL && !is_child_of(match,a))
@@ -612,11 +616,10 @@ thread_articles (GPtrArray    * articles)
 
 					if (break_thread_when_subject_changes)
 					{
-						const char * new_subject = norm[i].subject;
-						const char * old_subject = article_get_subject (match);
-						g_array_set_size (buf, strlen(old_subject)+1);
-						normalize_subject (buf->data, match);
-						subject_changed = strcmp (buf->data, new_subject);
+						int buf_len;
+						g_array_set_size (buf, match->subject.len + 1);
+						buf_len = normalize_subject (buf->data, match, FALSE);
+						subject_changed = pan_strcmp_len (buf->data, buf_len, norm[i].subject, norm[i].subject_len);
 					}
 
 					if (!subject_changed)
@@ -624,13 +627,13 @@ thread_articles (GPtrArray    * articles)
 				}
 
 				/* if we couldn't find the ancestor, march up the References string */
-				*message_id = '\0';
-				g_strchomp (tmp_references->str);
-				message_id = strrchr (tmp_references->str, '<');
+				references.len -= message_id.len;
+				references = pstring_strstrip_shallow (&references);
+				cpch = pan_strrchr_len (references.str, references.len, '<');
+				message_id = pstring_substr_shallow (&references, cpch, NULL);
 			}
 		}
 
-
 		/* thread by multipart */
 		if (!parent && a->parts>1 && a->part>1)
 		{
@@ -641,17 +644,17 @@ thread_articles (GPtrArray    * articles)
 
 			index = lower_bound (&n,
 			                     sorted_norm,
-			                     qty,
+			                     article_qty,
 			                     sizeof(Norm),
 			                     compare_pN_to_pN_by_subject,
 			                     NULL);
 
-			if (0<=index && index<qty)
+			if (0<=index && index<article_qty)
 			{
 				Norm * match = &sorted_norm[index];
 				if ((match->a != a)
 					&& (match->a->parts == a->parts)
-					&& (!strcmp(match->subject,n.subject))
+					&& (!pan_strcmp_len(match->subject,match->subject_len,n.subject,n.subject_len))
 					&& (!is_child_of(match->a,a)))
 				{
 					parent = match->a;
@@ -660,7 +663,7 @@ thread_articles (GPtrArray    * articles)
 		}
 
 		/* thread by subject */
-		if (!parent && skip_reply_leader(a->subject)!=a->subject)
+		if (!parent && skip_reply_leader(a->subject.str)!=a->subject.str)
 		{
 			Norm n = norm[i];
 			search_a.part = 0;
@@ -670,21 +673,21 @@ thread_articles (GPtrArray    * articles)
 			index = lower_bound (
 				&n,
 				sorted_norm,
-				qty,
+				article_qty,
 				sizeof(Norm),
 				compare_pN_to_pN_by_subject,
 				NULL);
 
-			if (0<=index && index<qty && !is_child_of(sorted_norm[index].a,a))
+			if (0<=index && index<article_qty && !is_child_of(sorted_norm[index].a,a))
 			{
 				Norm * match = &sorted_norm[index];
 
-				if (!strcmp(match->subject,n.subject))
+				if (!pan_strcmp_len (match->subject, match->subject_len, n.subject, n.subject_len))
 				{
 					/* 1 original, 1 reply */
 					parent = match->a;
 				}
-				else if (!strcmp(match->subject, a->subject) && (match->a->date<a->date))
+				else if (!pan_strcmp_len (match->subject, match->subject_len, a->subject.str, a->subject.len) && (match->a->date<a->date))
 				{
 					/* 2 replies, no top --  oldest on top */
 					parent = match->a;
@@ -703,50 +706,30 @@ thread_articles (GPtrArray    * articles)
 	}
 
 	/* right now all the children are normalized; point to articles */
-	for (i=0; i!=qty; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+	for (i=0; i!=article_qty; ++i) {
+		Article * a = articles[i];
 		a->threads = g_slist_sort (a->threads, compare_pA_to_pA_by_part);
 	}
 
-	/**
-	***  Calculate new/unread child counts
-	**/
-
-	for (i=0; i!=qty; ++i)
-	{
-		Article * a;
-
-		a = ARTICLE(g_ptr_array_index(articles,i));
-		if (a->parent!=NULL && !article_is_read(a))
-			for (a=a->parent; a!=NULL; a=a->parent)
-				++a->unread_children;
-
-		a = ARTICLE(g_ptr_array_index(articles,i));
-		if (a->parent!=NULL && article_is_new(a))
-			for (a=a->parent; a!=NULL; a=a->parent)
-				++a->new_children;
-	}
-
 	/**
 	***  Calculate multipart states
 	**/
 
-	for (i=0; i!=articles->len; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+	for (i=0; i!=article_qty; ++i) {
+		Article * a = articles[i];
 		a->multipart_state = MULTIPART_STATE_NONE;
 	}
-	for (i=0; i!=articles->len; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+	for (i=0; i!=article_qty; ++i) {
+		Article * a = articles[i];
 		if (a->parent == NULL)
 			set_children_part_state (a, article_root_get_part_state(a));
 	}
 
 	/* cleanup */
-	g_hash_table_destroy (ref_hash);
+	g_hash_table_destroy (mid_to_article);
 	g_free (norm);
 	g_free (norm_str_buf);
 	g_free (sorted_norm);
-	g_string_free (tmp_references, TRUE);
 	if (buf != NULL)
 		g_array_free (buf, TRUE);
 }
