@@ -44,7 +44,8 @@ void sort_articles   (Article      ** buf,
                       int             sort_type,
                       gboolean        ascending);
 
-void thread_articles (GPtrArray     * articles);
+void thread_articles (Article      ** articles,
+                      guint           article_qty);
 
 
 #endif /* __ARTICLE_THREAD_H__ */
