@@ -78,15 +78,22 @@ article_destructor (Article * a)
 		dead_article.date             = ~0;
 		dead_article.threads          = DEAD_POINTER;
 		dead_article.parent           = DEAD_POINTER;
-		dead_article.xref             = DEAD_POINTER;
-		dead_article.subject          = DEAD_POINTER;
-		dead_article.author_addr      = DEAD_POINTER;
-		dead_article.author_real      = DEAD_POINTER;
-		dead_article.message_id       = DEAD_POINTER;
-		dead_article.references       = DEAD_POINTER;
+
+		dead_article.author_addr      = PSTRING_INIT;
+		dead_article.author_real      = PSTRING_INIT;
+		dead_article.references       = PSTRING_INIT;
+		dead_article.xref             = PSTRING_INIT;
+		dead_article.subject          = PSTRING_INIT;
+		dead_article.message_id       = PSTRING_INIT;
 	}
 
 	/* destroy the article */
+	pstring_clear (&a->author_addr);
+	pstring_clear (&a->author_real);
+	pstring_clear (&a->references);
+	pstring_clear (&a->xref);
+	pstring_clear (&a->subject);
+	pstring_clear (&a->message_id);
 	g_slist_free (a->threads);
 	*a = dead_article;
 }
@@ -121,15 +128,16 @@ article_new (Group * group)
 		empty_article.score_date       = (time_t)0;
 		empty_article.date             = (time_t)0;
 		empty_article.number           = (gulong)0;
-		empty_article.subject          = NULL;
-		empty_article.author_real      = NULL;
-		empty_article.author_addr      = NULL;
-		empty_article.message_id       = NULL;
-		empty_article.xref             = NULL;
-		empty_article.references       = NULL;
 		empty_article.parent           = NULL;
 		empty_article.threads          = NULL;
 		empty_article.group            = DEAD_POINTER;
+
+		empty_article.author_addr      = PSTRING_INIT;
+		empty_article.author_real      = PSTRING_INIT;
+		empty_article.references       = PSTRING_INIT;
+		empty_article.xref             = PSTRING_INIT;
+		empty_article.subject          = PSTRING_INIT;
+		empty_article.message_id       = PSTRING_INIT;
 	}
 
 	/* create the new article */
@@ -233,7 +241,7 @@ article_is_read (const Article * a)
 	g_return_val_if_fail (a!=NULL, FALSE);
 	g_return_val_if_fail (a->group!=NULL, FALSE);
 
-	return newsrc_is_article_read (group_get_newsrc(a->group), a->number);
+	return group_is_article_read (a->group, a->number);
 }
 
 void
@@ -243,7 +251,6 @@ articles_set_read_simple (Article    ** articles,
 {
 	guint i;
 	Group * g;
-	Newsrc * newsrc;
 	Article ** changed;
 	int changed_qty;
 
@@ -253,13 +260,12 @@ articles_set_read_simple (Article    ** articles,
 
 	/* prep local variables */
 	g = articles[0]->group;
-	newsrc = group_get_newsrc (g);
 	changed = g_new (Article*, article_qty);
 	changed_qty = 0;
 
 	/* find the articles that will be changed by this action */
 	for (i=0; i<article_qty; ++i)
-		if (!!read != !!newsrc_mark_article (newsrc, articles[i]->number, read))
+		if (!!read != !!group_mark_article_read (g, articles[i]->number, read))
 			changed[changed_qty++] = articles[i];
 
 	/**
@@ -300,12 +306,11 @@ articles_set_read_numbers_ghfunc (gpointer key, gpointer value, gpointer user_da
 	Group * g = GROUP(key);
 	GArray * numbers = (GArray*) value;
 	const gboolean read = user_data!=NULL;
-	Newsrc * newsrc = group_get_newsrc (g);
 
 	/* Mark each number as read/unread */
 	for (i=changed_qty=0; i<numbers->len; ++i) {
 		const gulong number = g_array_index (numbers, gulong, i);
-		if (read != newsrc_mark_article (newsrc, number, read))
+		if (read != group_mark_article_read (g, number, read))
 			++changed_qty;
 	}
 
@@ -537,6 +542,16 @@ article_get_root (const Article* article)
 	return (Article*) article;
 }
 
+gboolean
+articles_are_in_same_thread (const Article  * article_1,
+                             const Article  * article_2)
+{
+	if ((article_1!=NULL) != (article_2!=NULL))
+		return FALSE;
+
+	return article_get_root(article_1) == article_get_root(article_2);
+}
+
 char*
 article_get_thread_message_id (const Article * article)
 {
@@ -550,7 +565,7 @@ article_get_thread_message_id (const Article * article)
 	 * the References: header is broken by other newreaders. */
 	article = article_get_root (article);
 
-	refs = article->references;
+	refs = article->references.str;
 	if (is_nonempty_string(refs))
 	{
 		pch = get_next_token_str (refs, ' ', NULL);
@@ -572,134 +587,159 @@ article_get_thread_impl (const Article * top, GPtrArray * setme)
 		article_get_thread_impl (ARTICLE(l->data), setme);
 }
 
-void
-article_get_subthread (const Article * article, GPtrArray * setme)
-{
-	/* sanity clause */
-	g_return_if_fail (article_is_valid(article));
-	g_return_if_fail (setme!=NULL);
 
-	pan_g_ptr_array_reserve (setme, 128);
-	article_get_thread_impl (article, setme);
+void
+article_forall_in_subthread   (Article            * article,
+                               ArticleForall        forall_func,
+                               gpointer             forall_func_user_data)
+{
+	GPtrArray * a;
+
+	/* sanity check */
+	g_return_if_fail (article_is_valid (article));
+
+	/* forall */
+	a = g_ptr_array_sized_new (128);
+	article_get_thread_impl (article, a);
+	if (a->len)
+		(forall_func)((Article**)a->pdata, a->len, forall_func_user_data);
+	g_ptr_array_free (a, TRUE);
 }
 
 void
-article_get_thread (const Article * article, GPtrArray * setme)
+article_forall_in_thread    (Article           * article,
+                             ArticleForall       forall_func,
+                             gpointer            forall_user_data)
 {
-	/* sanity clause */
-	g_return_if_fail (article_is_valid(article));
-	g_return_if_fail (setme!=NULL);
-
-	article_get_subthread (article_get_root(article), setme);
+	article_forall_in_subthread (article_get_root(article), forall_func, forall_user_data);
 }
 
 void
-article_get_references (const Article * article, GPtrArray * setme)
+article_forall_in_references  (Article            * article,
+                               ArticleForall        forall_func,
+                               gpointer             forall_func_user_data)
 {
-	g_return_if_fail (article_is_valid(article));
-	g_return_if_fail (setme!=NULL);
+	GPtrArray * a;
 
-	pan_g_ptr_array_reserve (setme, 128);
+	g_return_if_fail (article_is_valid(article));
+
+	a = g_ptr_array_sized_new (128);
 	while (article != NULL) {
-		g_ptr_array_add (setme, (gpointer)article);
+		g_ptr_array_add (a, (gpointer)article);
 		article = article->parent;
 	}
+
+	if (a->len)
+		(forall_func)((Article**)a->pdata, a->len, forall_func_user_data);
+
+	g_ptr_array_free (a, TRUE);
 }
 
-GPtrArray*
-article_get_unique_threads (const GPtrArray    * articles,
-                            ThreadGet            thread_get)
+
+static void
+add_articles_to_direct_hash (Article        ** articles,
+                             guint             article_qty,
+                             gpointer          article_hash_gpointer)
 {
-	int i;
-	GPtrArray * thread;
-	GPtrArray * retval;
-	GHashTable * all;
+	guint i;
+	GHashTable * article_hash = (GHashTable*) article_hash_gpointer;
+	for (i=0; i!=article_qty; ++i)
+		g_hash_table_insert (article_hash, articles[i], articles[i]);
+}
+
+
+void
+article_forall_in_threads (Article           ** articles,
+                           guint                article_qty,
+                           ThreadGet            thread_get,
+                           ArticleForall        forall_func,
+                           gpointer             forall_user_data)
+{
+	guint i;
+	GHashTable * hash;
+	GPtrArray * array;
 	debug_enter ("articlelist_get_unique_threads");
 
-	/* sanity clause */
-	retval = g_ptr_array_sized_new (128);
-	g_return_val_if_fail (articles_are_valid_in_group ((const Article **)articles->pdata, articles->len), retval);
-
 	/* get the thread for each article passed in */
-	thread = g_ptr_array_sized_new (128);
-	all = g_hash_table_new (g_direct_hash, g_direct_equal);
-	for (i=0; i<articles->len; ++i)
+	hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	for (i=0; i!=article_qty; ++i)
 	{
-		int j;
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
+		Article * a = articles[i];
 
 		/* if we already have the article, then we have its thread, so skip */
-		if (g_hash_table_lookup (all, a) != NULL)
+		if (g_hash_table_lookup (hash, a) != NULL)
 			continue;
 
 		/* get the thread associated with sel... */
-		g_ptr_array_set_size (thread, 0);
 		if (thread_get == GET_SUBTHREAD)
-			article_get_subthread (a, thread);
+			article_forall_in_subthread (a, add_articles_to_direct_hash, hash);
 		else
-			article_get_thread (a, thread);
-
-		/* add the articles into "all" hash */
-		for (j=0; j<thread->len; ++j) {
-			Article * b = ARTICLE (g_ptr_array_index(thread,j));
-			g_hash_table_insert (all, b, b);
-		}
+			article_forall_in_thread (a, add_articles_to_direct_hash, hash);
 	}
 
+	/* forall */
+	array = g_ptr_array_new ();
+	pan_hash_to_ptr_array (hash, array);
+	if (array->len != 0u)
+		(forall_func)((Article**)array->pdata, array->len, forall_user_data);
+
 	/* cleanup */
-	pan_hash_to_ptr_array (all, retval);
-	g_hash_table_destroy (all);
-	g_ptr_array_free (thread, TRUE);
-	debug_exit ("article_get_unique_threads");
-	return retval;
+	g_ptr_array_free (array, TRUE);
+	g_hash_table_destroy (hash);
 }
 
 /***
 ****  XREF
 ***/
 
-static void
-xref_foreach (const char         * xref_string,
-              Server             * server,
-              ServerGroupsType     set,
-              Group              * skip,
-              ArticleXRefFunc      func,
-              gpointer             user_data)
+void
+xref_header_foreach (const char                * xref_string,
+                     const PString             * server_name,
+                     const PString             * skip_group,
+                     ArticleXRefStringFunc       func,
+                     gpointer                    user_data)
 {
-	/* sanity clause */
-	g_return_if_fail (server_is_valid(server));
+	PString run;
+	const char * march = NULL;
 
-	/* get the xref header */
-	if (is_nonempty_string(xref_string))
+	/* skip the servername; we've got the server already */
+	skip_next_token (xref_string, ' ', &march);
+
+	/* walk through the xrefs, of format "group1:number group2:number" */
+	while (get_next_token_pstring (march, ' ', &march, &run))
 	{
-		const char * march, * run_str = NULL;
-		int run_len;
-
-		/* skip the servername; we've got the server already */
-		skip_next_token (xref_string, ' ', &march);
-
-		/* walk through the xrefs, of format "group1:number group2:number" */
-		while (get_next_token_run (march, ' ', &march, &run_str, &run_len))
+		const char * delimit = pan_strchr_len (run.str, run.len, ':');
+		if (delimit != NULL)
 		{
-			const char * delimit = g_strstr_len (run_str, run_len, ":");
-			if (delimit != NULL)
-			{
-				char group_name[1024];
-				const int group_name_len = MIN (delimit-run_str, sizeof(group_name)-1);
-				const gulong number = strtoul (delimit+1, NULL, 10);
-				Group * group = NULL;
-
-				memcpy (group_name, run_str, group_name_len);
-				group_name[group_name_len] = '\0';
-
-				group = server_get_named_group_in_type (server, group_name, set);
-				if (group!=NULL && group!=skip)
-					(*func)(server, group, number, user_data);
-			}
+			const gulong number = strtoul (delimit+1, NULL, 10);
+			const PString group_name = pstring_substr_shallow (&run, NULL, delimit);
+			if (skip_group==NULL || !pstring_equal (&group_name, skip_group))
+				(*func)(server_name, &group_name, number, user_data);
 		}
 	}
 }
 
+typedef struct
+{
+	Server * server;
+	ArticleXRefFunc * func;
+	gpointer user_data;
+	ServerGroupsType set;
+}
+XRefForeachData;
+
+static void
+article_xref_foreach_impl (const PString             * server_name,
+                           const PString             * group_name,
+                           gulong                      number,
+                           gpointer                    user_data)
+{
+	XRefForeachData * data = (XRefForeachData*) user_data;
+	Group * group = server_get_named_group_in_type (data->server, group_name, data->set);
+	if (group!=NULL)
+		(data->func)(data->server, group, number, data->user_data);
+}
+
 void
 article_xref_foreach (const Article       * a,
                       ArticleXRefFunc       func,
@@ -707,9 +747,19 @@ article_xref_foreach (const Article       * a,
 		      ServerGroupsType      set,
                       gboolean              skip_group_a)
 {
+	XRefForeachData data;
+
 	g_return_if_fail (article_is_valid(a));
 
-	xref_foreach (a->xref, a->group->server, set, skip_group_a?a->group:NULL, func, user_data);
+	data.server = a->group->server;
+	data.func = func;
+	data.user_data = user_data;
+	data.set = set;
+
+	xref_header_foreach (a->xref.str,
+	                     &a->group->server->name,
+	                     skip_group_a ? &a->group->name : NULL,
+	                     article_xref_foreach_impl, &data);
 }
 
 /***
@@ -725,12 +775,13 @@ article_get_crosspost_qty (const Article  * article)
 
 	g_return_val_if_fail (article_is_valid (article), 0);
 
-	if (!is_nonempty_string (article->xref))
+	if (!pstring_is_set (&article->xref))
 		retval = 1;
 	else {
 		const char * pch;
+		const char * const end = article->xref.str + article->xref.len;
 		retval = 0;
-		for (pch=article->xref; *pch; ++pch)
+		for (pch=article->xref.str; pch!=end; ++pch)
 			if (*pch==':')
 				++retval;
 	}
@@ -766,95 +817,97 @@ article_header_is_extra (const char * key)
 }
 
 void
-article_set_subject (Article        * a,
-	             const char     * subject)
+article_set_xref (Article        * a,
+	          const PString  * xref)
 {
 	g_return_if_fail (a!=NULL);
-	g_return_if_fail (is_nonempty_string(subject));
+	g_return_if_fail (pstring_is_valid (xref));
 
-	a->subject = (char*) group_chunk_string (a->group, subject, TRUE);
+	pstring_copy (&a->xref, xref);
+}
+
+void
+article_set_subject (Article        * a,
+	             const PString   * subject)
+{
+	g_return_if_fail (a!=NULL);
+	g_return_if_fail (pstring_is_valid (subject));
+
+	pstring_copy (&a->subject, subject);
 }
  
 const char*
 article_get_subject (const Article * a)
 {
 	g_return_val_if_fail (article_is_valid(a), "");
-	return is_nonempty_string(a->subject) ? a->subject : "";
+
+	return pstring_is_set (&a->subject) ? a->subject.str : "";
 }
 
 void
 article_set_message_id (Article        * a,
-	                const char     * message_id)
+	                const PString  * message_id)
 {
 	g_return_if_fail (a!=NULL);
-	g_return_if_fail (is_nonempty_string(message_id));
+	g_return_if_fail (pstring_is_set (message_id));
 
-	a->message_id = (char*) group_chunk_string (a->group, message_id, TRUE);
+	pstring_copy (&a->message_id, message_id);
 }
 
 const char*
 article_get_message_id (const Article * a)
 {
 	g_return_val_if_fail (article_is_valid(a), "");
-	return a->message_id;
+
+	return pstring_is_set (&a->message_id) ? a->message_id.str : "";
 }
 
 void
 article_set_references (Article        * a,
-	                const char     * references)
+	                const PString  * references)
 {
 	g_return_if_fail (a!=NULL);
-	g_return_if_fail (is_nonempty_string(references));
+	g_return_if_fail (pstring_is_set (references));
 
-	a->references = (char*) group_chunk_string (a->group, references, TRUE);
+	pstring_copy (&a->references, references);
 }
 
-void
-article_set_xrefs (Article        * a,
-	           const char     * xrefs)
-{
-	g_return_if_fail (a!=NULL);
-	g_return_if_fail (is_nonempty_string(xrefs));
-
-	a->xref = (char*) group_chunk_string (a->group, xrefs, FALSE);
-}
-
-
 char*
-article_format_author_str (const char * author_addr,
-                           const char * author_real,
-                           char       * buf,
-                           int          bufsize)
+article_format_author_str (const PString * author_addr,
+                           const PString * author_real,
+                           char          * buf,
+                           int             bufsize)
 {
-	const int name_len = author_real ? strlen (author_real) : 0;
-	const int addr_len = author_addr ? strlen (author_addr) : 0;
-
 	*buf = '\0';
 
-	if (name_len && addr_len)
+	if (author_real->len && author_addr->len)
 	{
-		if (name_len + addr_len + 6 > bufsize)
-			g_snprintf (buf, bufsize, "\"%s\" <%s>", author_real, author_addr);
+		if (author_real->len + author_addr->len + 6 > bufsize)
+		{
+			g_snprintf (buf, bufsize, "\"%*.*s\" <%*.*s>",
+			            author_real->len, author_real->len, author_real->str,
+			            author_addr->len, author_addr->len, author_addr->str);
+		}
 		else
 		{
 			/* doing this by hand because profiling says it's a hotspot */
 			char * pch = buf;
 			*pch++ = '"';
-			memcpy (pch, author_real, name_len);
-			pch += name_len;
+			memcpy (pch, author_real->str, author_real->len);
+			pch += author_real->len;
 			*pch++ = '"';
 			*pch++ = ' ';
 			*pch++ = '<';
-			memcpy (pch, author_addr, addr_len);
-			pch += addr_len;
+			memcpy (pch, author_addr->str, author_addr->len);
+			pch += author_addr->len;
 			*pch++ = '>';
 			*pch++ = '\0';
 		}
 	}
-	else if (addr_len)
-		g_strlcpy (buf, author_addr, bufsize);
-	else if (name_len)
-		g_strlcpy (buf, author_real, bufsize);
+	else if (author_addr->len)
+		pan_strncpy_len (buf, bufsize, author_addr->str, author_addr->len);
+	else if (author_real->len)
+		pan_strncpy_len (buf, bufsize, author_real->str, author_real->len);
 
 	g_strstrip (buf);
 	return buf;
@@ -869,36 +922,46 @@ article_get_author_str (const Article * a, char * buf, int bufsize)
 	*buf = '\0';
 	g_return_val_if_fail (article_is_valid(a), buf);
 
-	return article_format_author_str (a->author_addr, a->author_real, buf, bufsize);
+	return article_format_author_str (&a->author_addr,
+	                                  &a->author_real,
+	                                  buf, bufsize);
 }
 
 int
-article_format_short_author_str (const char * author_addr,
-                                 const char * author_real,
-                                 char       * buf,
-                                 int          buflen)
+article_format_short_author_str (const PString * author_addr,
+                                 const PString * author_real,
+                                 char          * buf,
+                                 int             buflen)
 {
 	int retval;
+	static PString paddr;
+	static PString preal;
+	static gboolean init = FALSE;
 
 	/* sanity clause */
 	g_return_val_if_fail (buf!=NULL, 0);
 	g_return_val_if_fail (buflen>0, 0);
 
-	if (is_nonempty_string (author_real) && strcmp(author_real, default_incoming_name_real))
-	{
-		retval = g_strlcpy (buf, author_real, buflen);
+	/* setup */
+	if (!init) {
+		paddr = pstring_shallow (default_incoming_name_addr, -1);
+		preal = pstring_shallow (default_incoming_name_real, -1);
+		init = TRUE;
 	}
-	else if (is_nonempty_string (author_addr) && strcmp(author_addr,default_incoming_name_addr))
+
+	if (pstring_is_set(author_real) && !pstring_equal(author_real, &preal))
 	{
-		const char * author = author_addr;
-		const char * p = strchr (author, '@');
-		if (p == NULL)
-			p = author + strlen(author);
-		retval = g_strlcpy (buf, author, MIN(buflen,p-author));
+		retval = pan_strncpy_len (buf, buflen, author_real->str, author_real->len);
+	}
+	else if (pstring_is_set(author_addr) && !pstring_equal (author_addr, &paddr))
+	{
+		const char * cpch = pan_strchr_len (author_addr->str, author_addr->len, '@');
+		const PString name = pstring_substr_shallow (author_addr, author_addr->str, cpch);
+		retval = pan_strncpy_len (buf, buflen, name.str, name.len);
 	}
 	else
 	{
-		retval = g_strlcpy (buf, default_incoming_name_real, buflen);
+		retval = pan_strncpy_len (buf, buflen, preal.str, preal.len);
 	}
 
 	return retval;
@@ -907,14 +970,14 @@ article_format_short_author_str (const char * author_addr,
 int
 article_get_short_author_str (const Article * a, char * buf, int len)
 {
-	return article_format_short_author_str (a->author_addr,
-	                                        a->author_real,
+	return article_format_short_author_str (&a->author_addr,
+	                                        &a->author_real,
 	                                        buf, len);
 }
 
 void
 article_set_author (Article       * a,
-                    const char    * header_from)
+                    const PString * header_from)
 {
 	char addr_buf[512];
 	char real_buf[512];
@@ -926,11 +989,11 @@ article_set_author (Article       * a,
 	/* sanity clause */
 	g_return_if_fail (a!=NULL);
 	g_return_if_fail (a->group!=NULL);
-	g_return_if_fail (is_nonempty_string(header_from));
+	g_return_if_fail (pstring_is_set (header_from));
 
 	/* get the author address & real name */
 	charset = group_get_default_charset (a->group);
-	pch = pan_header_to_utf8 (header_from, -1, charset);
+	pch = pan_header_to_utf8 (header_from->str, header_from->len, charset);
 	gnksa_do_check_from (pch,
 	                     addr_buf, sizeof(addr_buf),
 	                     real_buf, sizeof(real_buf),
@@ -941,22 +1004,17 @@ article_set_author (Article       * a,
 	pch = addr;
 	if (!is_nonempty_string (pch))
 		pch = (char*)default_incoming_name_addr;
-	a->author_addr = group_chunk_string (a->group, pch, TRUE);
+	pstring_set (&a->author_addr, pch, -1);
 
 	/* use the real name, or fill in a default. */
 	if (is_nonempty_string(real)) {
 		gnksa_strip_realname (real);
-		a->author_real = group_chunk_string (a->group, real, TRUE);
-	}
-	else if (addr!=NULL && ((pch=strchr(addr,'@'))!=NULL)) {
-		char * tmp;
-		pan_strndup_alloca (tmp, addr, pch-addr);
-		a->author_real = group_chunk_string (a->group, tmp, TRUE);
-	}
-	else {
-		const char * cpch = (const char*)default_incoming_name_real;
-		a->author_real = group_chunk_string (a->group, cpch, TRUE);
+		pstring_set (&a->author_real, real, -1);
 	}
+	else if (addr!=NULL && ((pch=strchr(addr,'@'))!=NULL))
+		pstring_set (&a->author_real, addr, pch-addr);
+	else
+		pstring_set (&a->author_real, default_incoming_name_real, -1);
 }
 
 void
@@ -972,26 +1030,38 @@ article_set_from_g_mime_message   (Article         * a,
 
 	g_mime_message_get_date (msg, &a->date, NULL);
 
-	if (((cpch = g_mime_message_get_sender (msg))) != NULL)
-		article_set_author (a, cpch);
+	cpch = g_mime_message_get_sender (msg);
+	if (cpch != NULL) {
+		const PString author = pstring_shallow (cpch, -1);
+		article_set_author (a, &author);
+	}
 
-	if (((cpch = g_mime_message_get_subject (msg))) != NULL)
-		article_set_subject (a, cpch);
+	cpch = g_mime_message_get_subject (msg);
+	if (cpch != NULL) {
+		const PString subject = pstring_shallow (cpch, -1);
+		article_set_subject (a, &subject);
+	}
 
-	if (((cpch = g_mime_message_get_message_id (msg))) != NULL)
-		article_set_message_id (a, cpch);
+	cpch = g_mime_message_get_message_id (msg);
+	if (cpch != NULL) {
+		const PString message_id = pstring_shallow (cpch, -1);
+		article_set_message_id (a, &message_id);
+	}
 
-	if (((cpch = g_mime_message_get_header (msg, HEADER_REFERENCES))) != NULL)
-		article_set_references (a, cpch);
+	cpch = g_mime_message_get_header (msg, HEADER_REFERENCES);
+	if (cpch != NULL) {
+		const PString references = pstring_shallow (cpch, -1);
+		article_set_references (a, &references);
+	}
 	
 	/* fallback if no message-id */
-	if (!is_nonempty_string(a->message_id)) {
-		char * id;
+	if (!pstring_is_set (&a->message_id)) {
+		PString id = pstring_shallow (gnksa_generate_message_id_from_email_addr (a->author_addr.str), -1);
+		const PString default_subject = pstring_shallow (_("Unparseable Subject"), -1);
 		g_warning (_("Couldn't parse a Message-ID from the raw message!"));
-		id = gnksa_generate_message_id_from_email_addr (a->author_addr);
-		article_set_message_id (a, id);
-		g_free (id);
-		article_set_subject (a, _("Unparseable Subject"));
+		article_set_message_id (a, &id);
+		article_set_subject (a, &default_subject);
+		pstring_clear (&id);
 	}
 
 
@@ -1014,23 +1084,22 @@ article_is_valid (const Article * a)
 	g_return_val_if_fail (a->group->_articles_refcount>=0, FALSE);
 	g_return_val_if_fail (group_is_folder(a->group) || a->group->_articles_refcount>0, FALSE);
 
-	g_return_val_if_fail (a->message_id != DEAD_POINTER, FALSE);
-	g_return_val_if_fail (a->message_id!=NULL, FALSE);
-	g_return_val_if_fail (*a->message_id=='<', FALSE);
+	g_return_val_if_fail (pstring_is_valid (&a->xref), FALSE);
 
-	g_return_val_if_fail (a->subject != DEAD_POINTER, FALSE);
-	g_return_val_if_fail (is_nonempty_string(a->subject), FALSE);
+	g_return_val_if_fail (pstring_is_set (&a->subject), FALSE);
 
-	g_return_val_if_fail (a->references != DEAD_POINTER, FALSE);
-	g_return_val_if_fail (!a->references || *a->references=='<', FALSE);
+	g_return_val_if_fail (pstring_is_set (&a->message_id), FALSE);
+	g_return_val_if_fail (*a->message_id.str =='<', FALSE);
+
+	g_return_val_if_fail (pstring_is_valid (&a->references), FALSE);
+	g_return_val_if_fail (a->references.len==0 || *a->references.str =='<', FALSE);
 
 	g_return_val_if_fail (a->number!=0, FALSE);
 
 	g_return_val_if_fail (a->threads != DEAD_POINTER, FALSE);
 	g_return_val_if_fail (a->parent != DEAD_POINTER, FALSE);
-	g_return_val_if_fail (a->xref != DEAD_POINTER, FALSE);
-	g_return_val_if_fail (a->author_addr != DEAD_POINTER, FALSE);
-	g_return_val_if_fail (a->author_real != DEAD_POINTER, FALSE);
+	g_return_val_if_fail (pstring_is_valid (&a->author_addr), FALSE);
+	g_return_val_if_fail (pstring_is_valid (&a->author_real), FALSE);
 
 	return TRUE;
 }
