@@ -27,6 +27,7 @@
 #include <glib/garray.h>
 
 #include <pan/base/group.h>
+#include <pan/base/pstring.h>
 #include <pan/base/pan-callback.h>
 
 #define ARTICLE(a) ((Article *)a)
@@ -55,19 +56,22 @@ typedef struct _Article
 	gint16 parts;		      /* # of parts in multipart message */
 	gint16 score;		      /* scoring */
 	guint16 linecount;	      /* # of lines in the body */
-	guint16 unread_children;      /* articlelist: unread kids in thread */
-	guint16 new_children;         /* articlelist: new kids in thread */
 	gulong byte_qty;              /* # of bytes in the body */
 	time_t score_date;            /* when this article was last scored */
 	time_t date;                  /* date posted, as parsed into time_t */
 	GSList * threads;             /* articlelist: list of child threads */
 	struct _Article * parent;     /* articlelist: parent in threading */
-	const char * xref;            /* article's xref header from XOVER */
-	const char * subject;         /* article's subject */
-	const char * author_addr;     /* article's author -- address*/
-	const char * message_id;      /* article's message-id */
-	const char * author_real;     /* (maybe NULL) article's author -- real name */
-	const char * references;      /* (maybe NULL) references string for this article */
+	PString message_id;           /* articles message-id */
+	PString subject;              /* article's subject */
+	PString xref;                 /* article's xref header from XOVER.  May be empty. */
+	PString references;           /* references string for this article.  May be empty. */
+	PString author_addr;          /* article's author -- address*/
+	PString author_real;          /* article's author -- real name */
+
+	gint16 subthread_score;       /* ARTICLELIST ONLY */
+	gpointer header_pane_extra;   /* ARTICLELIST ONLY */
+	guint16 unread_children;      /* ARTICLELIST ONLY */
+	guint16 new_children;         /* ARTICLELIST ONLY */
 
 	/* At some point this may be an array for 'article-centric' */
 	gulong number;               /* (PROTECTED) number for article */
@@ -100,6 +104,7 @@ Article;
 #define PAN_CHARSET               "X-Pan-Internal-CharSet"
 #define PAN_NEWSGROUPS            "X-Pan-Internal-Sendlater-Newsgroups"
 #define PAN_TO                    "X-Pan-Internal-Sendlater-To"
+#define PAN_NO_MESSAGE_ID         "X-Pan-Internal-No-Message-Id"
 
 /**
 ***  PUBLIC LIFE CYCLE
@@ -134,34 +139,34 @@ void         article_set_error_flag           (Article        * article,
 
 const char*  article_get_subject              (const Article  * article);
 
+void         article_set_xref                 (Article        * article,
+                                               const PString  * xref);
+
 void         article_set_subject              (Article        * article,
-                                               const char     * subject);
+                                               const PString  * subject);
 
 const char*  article_get_message_id           (const Article  * article);
 
 void         article_set_message_id           (Article        * article,
-                                               const char     * message_id);
+                                               const PString  * message_id);
 
 void         article_set_references           (Article        * article,
-                                               const char     * references);
-
-void         article_set_xrefs                (Article        * article,
-                                               const char     * xrefs);
+                                               const PString  * references);
 
 void         article_set_author               (Article        * article,
-                                               const char     * header_from);
+                                               const PString  * pstring);
 
 char*        article_get_author_str           (const Article  * article,
                                                char           * buf,
                                                int              bufsize);
 
-char*        article_format_author_str        (const char     * author_addr,
-                                               const char     * author_real,
+char*        article_format_author_str        (const PString  * author_addr,
+                                               const PString  * author_real,
                                                char           * buf,    
                                                int              bufsize);
 
-int          article_format_short_author_str  (const char     * author_addr,
-                                               const char     * author_real,
+int          article_format_short_author_str  (const PString  * author_addr,
+                                               const PString  * author_real,
                                                char           * buf,    
                                                int              bufsize);
 
@@ -180,6 +185,9 @@ void         article_set_from_g_mime_message  (Article        * setme_article,
 
 Article*     article_get_root                 (const Article  * article);
 
+gboolean     articles_are_in_same_thread      (const Article  * article_1,
+                                               const Article  * article_2);
+
 /**
  * Returns TRUE if the header is one only meant for Pan's internal bookkeeping
  * and should never be shown to the user.
@@ -199,6 +207,17 @@ gboolean     article_header_is_extra          (const char     * key);
  */
 typedef void (ArticleXRefFunc)(Server*,Group*,gulong,gpointer);
 
+typedef void (ArticleXRefStringFunc)(const PString * server_name,
+                                     const PString * group_name,
+                                     gulong number,
+                                     gpointer user_data);
+
+void xref_header_foreach (const char                * xref_string,
+                          const PString             * server_name,
+                          const PString             * skip_group,
+                          ArticleXRefStringFunc       func,
+                          gpointer                    user_data);
+
 /**
  * For each cross-reference specified in Article a's Xref: header,
  * the specified ArticleXRefFunc is invoked.
@@ -271,30 +290,33 @@ void         articles_set_dirty               (Article       ** articles,
 ***  THREADS
 **/
 
-/**
- * @param article the article whose thread is to be returned
- * @param GPtrArray a container to hold all the articles found in
- *                  this article's thread.
- */
-void           article_get_thread            (const Article   * article,
-                                              GPtrArray       * setme);
-
-void           article_get_subthread         (const Article   * article,
-                                              GPtrArray       * setme);
-
-void           article_get_references        (const Article   * article,
-                                              GPtrArray       * setme);
-
 typedef enum
 {
 	GET_WHOLE_THREAD,
 	GET_SUBTHREAD
 }
 ThreadGet;
- 
-GPtrArray*     article_get_unique_threads    (const GPtrArray    * articles,
-                                              ThreadGet            thread_get);
- 
+
+typedef void (*ArticleForeach)(Article * article, gpointer user_data);
+typedef void (*ArticleForall)(Article ** articles, guint article_qty, gpointer user_data);
+
+void           article_forall_in_references  (Article            * article,
+                                              ArticleForall        forall_func,
+                                              gpointer             forall_func_user_data);
+
+void           article_forall_in_subthread   (Article            * article,
+                                              ArticleForall        forall_func,
+                                              gpointer             forall_func_user_data);
+
+void           article_forall_in_thread      (Article            * articles,
+                                              ArticleForall        forall_func,
+                                              gpointer             forall_func_user_data);
+
+void           article_forall_in_threads     (Article           ** articles,
+                                              guint                article_qty,
+                                              ThreadGet            thread_get,
+                                              ArticleForall        forall_func,
+                                              gpointer             forall_func_user_data);
 
 char*          article_get_thread_message_id (const Article*);
 
