@@ -25,7 +25,7 @@
 #include <pan/base/group.h>
 #include <pan/base/status-item.h>
 
-void     file_headers_server_name_changed  (const char * old_name, const char * new_name);
+void     file_headers_server_name_changed  (const PString * old_name, const PString * new_name);
 
 void     file_headers_destroy              (const Group * group);
 
@@ -35,6 +35,5 @@ void     file_headers_save                 (Group * group, StatusItem * status);
 
 void     file_headers_save_noref           (Group * group, StatusItem * status);
 
-long     get_string_offset                 (GHashTable *, MemChunk *, const char *, FILE *, long *);
 
 #endif
