@@ -41,11 +41,63 @@
 #include <pan/base/pan-callback.h>
 #include <pan/base/server.h>
 
-static void fire_groups_changed (Group **, gint qty);
+static GHashTable* group_get_articles (Group *);
+static GPtrArray* group_get_article_array (Group *);
 
+/***
+****
+****  EVENTS
+****
+***/
+ 
 /**
-***
+***  Groups Removed
 **/
+ 
+PanCallback*
+group_get_groups_changed_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
+
+static void
+fire_groups_changed (Group ** groups, int group_qty, guint change_type)
+{
+	struct GroupChangeEvent e;
+
+	/* sanity clause */
+	g_return_if_fail (groups!=NULL);
+	g_return_if_fail (group_qty>0);
+	g_return_if_fail (group_is_valid(groups[0]));
+
+	/* fire the event */
+	e.groups = groups;
+	e.group_qty = group_qty;
+	e.change_type = change_type;
+	pan_callback_call (group_get_groups_changed_callback(), &e, NULL);
+}
+
+PanCallback*
+group_get_articles_added_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
+
+PanCallback*
+group_get_articles_removed_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
+
+/***
+****
+***/
 
 Group*
 group_new (Server * server, const char * name)
@@ -74,28 +126,28 @@ group_constructor (Group                * g,
 	g->old_sort_style = (gint8)0;
 	g->new_sort_style = (gint8)0;
 	g->filter_bits = ~((guint16)STATE_FILTER_SCORE_IGNORED);
-	g->filter_show = FILTER_SHOW_MATCHES_AND_REFERENCES;
+	g->filter_show = FILTER_SHOW_MATCHES;
 	g->article_qty = (gint32)0;
 	g->article_read_qty = (gint32)0;
 	g->article_low = (gulong)0;
 	g->article_high_old = (gulong)0;
 	g->permission = '?';
 	g->article_high = (gulong)0;
-	g->name = g_strdup (name);
+	g->name = PSTRING_INIT;
 	g->filter_name = NULL;
 	g->identity_name = NULL;
 	g->description = NULL;
 	g->download_dir = NULL;
 	g->default_charset = NULL;
-	g->chunk = NULL;
 
-	g->_articles_refcount = (gint)0;
+	g->_articles_refcount = 0;
 	g->_article_chunk = NULL;
 	g->_newsrc = NULL;
 	g->_purged = NULL;
 	g->_articles = NULL;
 	g->_deleted_articles = NULL;
-	g->_one_big_chunk = NULL;
+
+	pstring_set (&g->name, name, -1);
 }
 
 static GMutex*
@@ -110,9 +162,9 @@ group_get_article_mutex (Group * g)
 }
 
 static void
-destroy_articles_ghfunc (gpointer key, gpointer val, gpointer user_data)
+destroy_value_article_ghfunc (gpointer key, gpointer value, gpointer user_data)
 {
-	article_destructor (ARTICLE(val));
+	article_destructor ((Article*)value);
 }
 
 static void
@@ -121,10 +173,8 @@ blow_article_memory (Group * g)
 	g_return_if_fail (g!=NULL);
 	pan_warn_if_fail (g->_articles_refcount >= 0);
 
-	replace_gstr (&g->_one_big_chunk, NULL);
-
 	if (g->_articles != NULL) {
-		g_hash_table_foreach (g->_articles, destroy_articles_ghfunc, NULL);
+		g_hash_table_foreach (g->_articles, destroy_value_article_ghfunc, NULL);
 		g_hash_table_destroy (g->_articles);
 		g->_articles = NULL;
 	}
@@ -135,11 +185,6 @@ blow_article_memory (Group * g)
 		g->_deleted_articles = NULL;
 	}
 
-	if (g->chunk != NULL) {
-		g_string_chunk_free (g->chunk);
-		g->chunk = NULL;
-	}
-
 	if (g->_article_chunk != NULL) {
 		memchunk_destroy (g->_article_chunk);
 		g->_article_chunk = NULL;
@@ -156,7 +201,7 @@ group_destructor (Group * o)
 	debug1 (DEBUG_PAN_OBJECT, "group_destructor: %p", g);
 
 	/* strings */
-	replace_gstr (&g->name, NULL);
+	pstring_clear (&g->name);
 	replace_gstr (&g->description, NULL);
 	replace_gstr (&g->download_dir, NULL);
 	replace_gstr (&g->default_charset, NULL);
@@ -170,7 +215,8 @@ group_destructor (Group * o)
 		pan_object_unref (PAN_OBJECT(g->_purged));
 
 	/* articles */
-	blow_article_memory (g);
+	if (g->_articles_refcount > 0)
+		blow_article_memory (g);
 	if (g->article_mutex != NULL) {
 		g_mutex_free (g->article_mutex);
 		g->article_mutex = NULL;
@@ -244,9 +290,8 @@ groups_set_subscribed (Group ** groups, int qty, gboolean subscribed)
 		server_save_grouplist_if_dirty (server, NULL);
 
 	if (changed_qty > 0)
-		fire_groups_changed (changed, changed_qty);
+		fire_groups_changed (changed, changed_qty, GROUP_CHANGED_SUBSCRIPTION_STATE);
 
-	/* cleanup */
 	g_free (changed);
 	debug_exit ("groups_set_subscribed");
 }
@@ -268,7 +313,7 @@ group_set_dirty (Group * group)
 }
 
 void
-group_set_download_dir (Group * group, const gchar* download_dir)
+group_set_download_dir (Group * group, const char* download_dir)
 {
 	g_return_if_fail (group_is_valid(group));
 
@@ -289,7 +334,6 @@ group_set_default_charset (Group * group,
 	replace_gstr (&group->default_charset, g_strdup(default_charset));
 
 	group_set_dirty (group);
-	fire_groups_changed (&group, 1);
 
 	debug_exit ("group_set_default_charset");
 }
@@ -322,7 +366,7 @@ void
 group_set_filter (Group         * group,
                   guint           filter_bits,
                   gulong          filter_show,
-                  const gchar   * filter_name)
+                  const char    * filter_name)
 {
 	g_return_if_fail (group_is_valid(group));
 
@@ -335,7 +379,7 @@ group_set_filter (Group         * group,
 
 void
 group_set_identity (Group       * group,
-                    const gchar * identity_name)
+                    const char  * identity_name)
 {
 	g_return_if_fail (group!=NULL);
 
@@ -355,6 +399,9 @@ group_set_sort_style (Group * group, int sort_style)
 	group_set_dirty (group);
 }
 
+static Newsrc* group_get_newsrc (Group *);
+static Newsrc* group_get_purged (Group *);
+
 void
 group_set_article_range (Group    * g,
                          gulong     article_low,
@@ -393,59 +440,72 @@ group_mark_new_article_number (Group         * group,
 	group->article_high_old = everything_above_this_is_new;
 }
 
-static void
+static guint
 group_set_article_qty_impl (Group * group, int article_qty)
 {
+	guint retval = 0;
 	debug_enter ("group_set_article_qty_impl");
 
-	g_return_if_fail (group_is_valid(group));
+	g_return_val_if_fail (group_is_valid(group), retval);
 
 	if (article_qty != group->article_qty)
 	{
 		group->article_qty = article_qty;
 		group_set_dirty (group);
+		retval |= GROUP_CHANGED_ARTICLE_QTY;
 	}
 
 	debug_exit ("group_set_article_qty_impl");
+	return retval;
 }
 
 void
-group_set_article_qty (Group * group, gint article_qty)
+group_set_article_qty (Group * group, int article_qty)
 {
+	guint change_type;
 	debug_enter ("group_set_article_qty");
 
-	group_set_article_qty_impl (group, article_qty);
-	fire_groups_changed (&group, 1);
+	change_type = group_set_article_qty_impl (group, article_qty);
+	if (change_type)
+		fire_groups_changed (&group, 1, change_type);
 
 	debug_exit ("group_set_article_qty");
 }
 
-static void
-group_set_article_read_qty_impl (Group * group, gint article_read_qty)
+static guint
+group_set_article_read_qty_impl (Group * group, int article_read_qty)
 {
+	guint retval = 0;
 	debug_enter ("group_set_article_read_qty_impl");
 
-	g_return_if_fail (group_is_valid(group));
+	g_return_val_if_fail (group_is_valid(group), retval);
 
 	article_read_qty = MAX (article_read_qty, 0);
-	group->article_read_qty = article_read_qty;
-	group_set_dirty (group);
+	if (article_read_qty != group->article_read_qty)
+	{
+		group->article_read_qty = article_read_qty;
+		group_set_dirty (group);
+		retval |= GROUP_CHANGED_ARTICLE_READ_QTY;
+	}
 
 	debug_exit ("group_set_article_read_qty_impl");
+	return retval;
 }
 
 void
-group_set_article_read_qty (Group * group, gint article_read_qty)
+group_set_article_read_qty (Group * group, int article_read_qty)
 {
+	guint change_type;
 	debug_enter ("group_set_article_read_qty");
 
-	group_set_article_read_qty_impl (group, article_read_qty);
-	fire_groups_changed (&group, 1);
+	change_type = group_set_article_read_qty_impl (group, article_read_qty);
+	if (change_type)
+		fire_groups_changed (&group, 1, change_type);
 
 	debug_exit ("group_set_article_read_qty");
 }
 void
-group_inc_article_read_qty (Group * group, gint inc)
+group_inc_article_read_qty (Group * group, int inc)
 {
 	debug_enter ("group_inc_article_read_qty");
 
@@ -457,11 +517,17 @@ group_inc_article_read_qty (Group * group, gint inc)
 	debug_exit ("group_inc_article_read_qty");
 }
 
+static void
+group_mark_articles_read (Group * group, Article ** articles, guint article_qty, gpointer user_data)
+{
+	if (article_qty > 0u)
+		articles_set_read (articles, article_qty, TRUE);
+}
+
 void
 group_mark_all_read (Group * g, gboolean read)
 {
 	Newsrc * newsrc;
-	GPtrArray * articles;
 	debug_enter ("group_mark_all_read");
 
 	/* sanity clause */
@@ -475,10 +541,7 @@ group_mark_all_read (Group * g, gboolean read)
 	   crossposts as read, etc... */
 
 	g_mutex_lock (group_get_article_mutex (g));
-	articles = group_get_article_array (g);
-	if (articles->len > 0)
-		articles_set_read  ((Article**)articles->pdata, articles->len, TRUE);
-	g_ptr_array_free (articles, TRUE);
+	group_article_forall (g, group_mark_articles_read, NULL);
 	g_mutex_unlock (group_get_article_mutex (g));
 
 	/**
@@ -520,7 +583,7 @@ group_is_valid (const Group * group)
 {
 	g_return_val_if_fail (group!=NULL, FALSE);
 	g_return_val_if_fail (server_is_valid (group->server), FALSE);
-	g_return_val_if_fail (is_nonempty_string(group->name), FALSE);
+	g_return_val_if_fail (pstring_is_set (&group->name), FALSE);
 
 	return TRUE;
 }
@@ -579,7 +642,7 @@ group_get_acache_key (const Group * group)
 void
 group_empty (Group * group, gboolean clear_counts)
 {
-	GPtrArray * a;
+	guint change_type = 0;
 
 	/* sanity clause */
 	g_return_if_fail (group_is_valid(group));
@@ -588,20 +651,19 @@ group_empty (Group * group, gboolean clear_counts)
 	file_headers_destroy (group);
 
 	/* remove the articles from memory */
-	a = group_get_article_array (group);
-	group_remove_articles (group, (Article**)a->pdata, a->len);
-	g_ptr_array_free (a, TRUE);
+	group_article_forall (group, (GroupArticleFunc)group_remove_articles, NULL);
 
 	/* reset the counters */
-	group_set_article_qty_impl (group, 0);
-	group_set_article_read_qty_impl (group, 0);
+	change_type |= group_set_article_qty_impl (group, 0);
+	change_type |= group_set_article_read_qty_impl (group, 0);
 	newsrc_mark_all (group_get_purged(group), FALSE);
 
 	/* if user wants to clear the range, do that too */
 	if (clear_counts)
 		group_set_article_range (group, 0, 0);
 
-	fire_groups_changed (&group, 1);
+	if (change_type)
+		fire_groups_changed (&group, 1, change_type);
 }
 
 /***
@@ -616,20 +678,21 @@ folder_add_message (Group            * folder,
 {
 	gboolean is_new;
 	char * text;
-	const char * message_id;
+	PString message_id;
 
 	g_return_if_fail (group_is_valid(folder));
 	g_return_if_fail (group_is_folder(folder));
 	g_return_if_fail (GMIME_IS_MESSAGE(message));
 
+	/* get the message-id */
+	message_id = pstring_shallow (g_mime_message_get_message_id(message), -1);
+
 	/* do we already have this message? */
-	message_id = g_mime_message_get_message_id (message);
-	is_new = acache_has_message (group_get_acache_key(folder), message_id);
+	is_new = acache_has_message (group_get_acache_key(folder), &message_id);
 
 	/* write the message */
 	text = g_mime_message_to_string (message);
-	acache_set_message (folder->name, message_id, text, strlen(text));
-	g_free (text);
+	acache_set_message (folder->name.str, &message_id, text, strlen(text));
 
 	/* update articles */
 	if (is_new)
@@ -654,30 +717,37 @@ folder_add_message (Group            * folder,
 			group_set_article_qty (folder, folder->article_qty+1);
 		}
 	}
+
+	/* cleanup */
+	g_free (text);
 }
 
 void
 folder_remove_message  (Group         * folder,
                         GMimeMessage  * message)
 {
-	const char * message_id;
+	PString message_id;
+	const PString * pmessage_id;
 
 	/* sanity clause */
 	g_return_if_fail (group_is_valid (folder));
 	g_return_if_fail (group_is_folder (folder));
 	g_return_if_fail (GMIME_IS_MESSAGE(message));
 
+	/* get the message-id */
+	message_id = pstring_shallow (g_mime_message_get_message_id (message), -1);
+	pmessage_id = &message_id;
+
 	/* delete the message from our cache */
-	message_id = g_mime_message_get_message_id (message);
-	acache_expire_messages (group_get_acache_key(folder), &message_id, 1);
+	acache_expire_messages (group_get_acache_key(folder), &pmessage_id, 1);
 
 	/* delete the message from the group's tables */
 	if (!group_ref_articles_if_loaded (folder))
 		group_set_article_qty (folder, folder->article_qty-1);
 	else {
-		Article * a = group_get_article_by_message_id (folder, message_id);
+		Article * a = group_get_article_by_message_id (folder, &message_id);
 		if (a != NULL)
-			group_remove_articles (folder, &a, 1);
+			group_remove_articles (folder, &a, 1u);
 		group_unref_articles (folder, NULL);
 	}
 }
@@ -687,6 +757,15 @@ folder_remove_message  (Group         * folder,
 ****
 ***/
 
+static void
+group_thread_articles (Group        * group,
+                       Article     ** articles,
+                       guint          article_qty,
+                       gpointer       user_data)
+{
+	thread_articles (articles, article_qty);
+}
+
 static void
 group_add_articles_impl (Group             * group,
                          const GPtrArray   * articles,
@@ -695,8 +774,8 @@ group_add_articles_impl (Group             * group,
                          GPtrArray         * ignored,
                          gboolean            lock_group)
 {
-	gint old_qty;
-	gint new_read_qty;
+	int old_qty;
+	int new_read_qty;
 	guint i;
 	gulong low;
 	gulong high;
@@ -731,22 +810,21 @@ group_add_articles_impl (Group             * group,
 	newsrc = group_get_newsrc (group);
 	purged = group_get_purged (group);
 
-	if (lock_group) {
+	if (lock_group)
 		g_mutex_lock (group_get_article_mutex (group));
-	}
 
 	ours = group_get_articles (group);
 	old_qty = (int) g_hash_table_size (ours);
 	for (i=0; i!=articles->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		const char * message_id = article_get_message_id (a);
+		const PString * message_id = &a->message_id;
 		const gboolean is_purged = newsrc_is_article_read(purged, a->number);
 
 		if (!is_purged && g_hash_table_lookup(ours,message_id)==NULL)
 		{
 			a->group = group;
-			g_hash_table_insert (ours, (gpointer)message_id, a);
+			g_hash_table_insert (ours, &a->message_id, a);
 			g_ptr_array_add (tmp_used, a);
 
 			if (a->number<low)
@@ -766,19 +844,15 @@ group_add_articles_impl (Group             * group,
 	***  Rethread the articles
 	**/
 
-	if (tmp_used->len != 0) {
-		GPtrArray * a = group_get_article_array (group);
-		thread_articles (a);
-		g_ptr_array_free (a, TRUE);
-	}
+	if (tmp_used->len != 0)
+		group_article_forall (group, group_thread_articles, NULL);
 
 	/**
 	***  Unlock the article mutex
 	**/
 
-	if (lock_group) {
+	if (lock_group) 
 		g_mutex_unlock (group_get_article_mutex (group));
-	}
 
 	/**
 	***  Now update our stats if articles were added
@@ -786,17 +860,21 @@ group_add_articles_impl (Group             * group,
 
 	if (tmp_used->len != 0)
 	{
+		guint change_type = 0u;
+
 		/* maybe update the article range */
 		if (high!=group->article_high || low!=group->article_low)
 			group_set_article_range (group, low, high);
 
 		/* update the group article stats */
-		group_set_article_qty_impl (group, g_hash_table_size(ours));
+		change_type |= group_set_article_qty_impl (group, g_hash_table_size(ours));
 		if (old_qty != 0)
 			new_read_qty += group->article_read_qty;
-		group_set_article_read_qty_impl (group, new_read_qty);
+		change_type |= group_set_article_read_qty_impl (group, new_read_qty);
 		group_set_articles_dirty (group);
-		fire_groups_changed (&group, 1);
+
+		if (change_type)
+			fire_groups_changed (&group, 1, change_type);
 
 		pan_callback_call (group_get_articles_added_callback(), group, tmp_used);
 	}
@@ -873,17 +951,55 @@ group_set_articles_dirty (Group * group)
 }
 
 gpointer
-group_get_article_by_message_id (Group * group, const char * message_id)
+group_get_article_by_message_id (Group * group, const PString * message_id)
 {
-	g_return_val_if_fail (group_is_valid(group), NULL);
-	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
-	g_return_val_if_fail (*message_id=='<', NULL);
+	g_return_val_if_fail (group_is_valid (group), NULL);
+	g_return_val_if_fail (pstring_is_set (message_id), NULL);
 
 	return g_hash_table_lookup (group_get_articles(group), message_id);
 }
 
+static void
+remove_articles_from_thread (Article    ** articles,
+                             guint         article_qty,
+                             gpointer      remove_articles_gptrarray)
+{
+	guint i;
+	GPtrArray * remove = (GPtrArray*) remove_articles_gptrarray;
+	GHashTable * rethread_hash;
+
+	/* build a hash table of the articles to rethread.
+	 * this is for easy removal of "remove" articles */
+       	rethread_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	for (i=0; i!=article_qty; ++i)
+		g_hash_table_insert (rethread_hash, articles[i], articles[i]);
+	for (i=0; i!=remove->len; ++i)
+		g_hash_table_remove (rethread_hash, g_ptr_array_index (remove, i));
+
+	/* update the thread information in the "remove" articles */
+	for (i=0; i!=remove->len; ++i) {
+		Article * a = ARTICLE(g_ptr_array_index(remove, i));
+		a->parent = NULL;
+		if (a->threads != NULL) {
+			g_slist_free (a->threads);
+			a->threads = NULL;
+		}
+	}
+
+	/* update the thread information in "keep" articles */
+	if (g_hash_table_size(rethread_hash) > 0) {
+		GPtrArray * rethread_a = g_ptr_array_new ();
+		pan_hash_to_ptr_array (rethread_hash, rethread_a);
+		thread_articles ((Article**)rethread_a->pdata, rethread_a->len);
+		g_ptr_array_free (rethread_a, TRUE);
+	}
+
+	/* cleanup */
+	g_hash_table_destroy (rethread_hash);
+}
+
 void
-group_remove_articles (Group * group, Article ** articles, int article_qty)
+group_remove_articles (Group * group, Article ** articles, guint article_qty)
 {
 	guint i;
 	int removed_read;
@@ -917,9 +1033,9 @@ group_remove_articles (Group * group, Article ** articles, int article_qty)
 	removed = g_ptr_array_sized_new (article_qty);
 	for (i=0; i!=tmp_remove->len; ++i)
 	{
-		Article * a = ARTICLE(g_ptr_array_index(tmp_remove,i));
-		const gchar * message_id = article_get_message_id (a);
-		Article * our_a = ARTICLE(g_hash_table_lookup (ours, message_id));
+		const Article * a = ARTICLE (g_ptr_array_index(tmp_remove,i));
+		const PString * message_id = &a->message_id;
+		Article * our_a = ARTICLE (g_hash_table_lookup (ours, message_id));
 
 		if (our_a != NULL)
 		{
@@ -940,47 +1056,16 @@ group_remove_articles (Group * group, Article ** articles, int article_qty)
 
 	if (removed->len)
 	{
-		GPtrArray * rethread_a = article_get_unique_threads (removed, GET_WHOLE_THREAD);
-		GHashTable * rethread_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
-
 		/* add removed articles to the 'deleted' list */
 		if (group->_deleted_articles == NULL)
 			group->_deleted_articles = g_ptr_array_new ();
 		pan_g_ptr_array_append (group->_deleted_articles, removed->pdata, removed->len);
 
-		/* build a hash table of the articles that need to be rethreaded */
-		for (i=0; i!=rethread_a->len; ++i) {
-			Article * a = ARTICLE(g_ptr_array_index(rethread_a, i));
-			g_hash_table_insert (rethread_hash, a, a);
-		}
+		/* rethread the affected branches */
+		article_forall_in_threads ((Article**)removed->pdata, removed->len, GET_WHOLE_THREAD,
+		                           remove_articles_from_thread, removed);
 
-		/* remove the deleted articles from this rethread list */
-		for (i=0; i!=removed->len; ++i) {
-			Article * a = ARTICLE(g_ptr_array_index(removed, i));
-			g_hash_table_remove (rethread_hash, a);
-		}
-
-		/* isolate the deleted articles, in case any clients still
-		 * have handles to these deleted articles, because of some
-		 * workproc, etc. */
-		for (i=0; i!=removed->len; ++i) {
-			Article * a = ARTICLE(g_ptr_array_index(removed, i));
-			a->parent = NULL;
-			if (a->threads != NULL) {
-				g_slist_free (a->threads);
-				a->threads = NULL;
-			}
-		}
-
-		/* rethread the affected articles */
-		if (g_hash_table_size(rethread_hash) > 0) {
-			pan_hash_to_ptr_array (rethread_hash, rethread_a);
-			thread_articles (rethread_a);
-		}
-
-		/* cleanup */
-		g_hash_table_destroy (rethread_hash);
-		g_ptr_array_free (rethread_a, TRUE);
+		group_set_articles_dirty (group);
 	}
 
 	/**
@@ -993,13 +1078,12 @@ group_remove_articles (Group * group, Article ** articles, int article_qty)
 	***  Update the stats
 	**/
 
-	if (removed->len != 0)
-	{
-		/* update the articles stats */
-		group->article_qty -= removed->len;
-		group->article_read_qty -= removed_read;
-		group_set_articles_dirty (group);
-		fire_groups_changed (&group, 1);
+	if (removed->len != 0) {
+		guint change_type = 0u;
+		change_type |= group_set_article_qty_impl (group, group->article_qty - removed->len);
+		change_type |= group_set_article_read_qty_impl (group, group->article_read_qty - removed_read);
+		if (change_type)
+			fire_groups_changed (&group, 1, change_type);
 	}
 
 	/* cleanup */
@@ -1049,7 +1133,7 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 ****
 ***/
 
-static GStaticMutex article_refcount_mutex = G_STATIC_MUTEX_INIT;
+static GStaticRecMutex article_refcount_mutex = G_STATIC_REC_MUTEX_INIT;
 
 gboolean
 group_ref_articles_if_loaded (Group * group)
@@ -1062,12 +1146,12 @@ group_ref_articles_if_loaded (Group * group)
 	g_return_val_if_fail (group->_articles_refcount>=0, FALSE);
 
 	/* ref if reffed */
-	g_static_mutex_lock (&article_refcount_mutex);
+	g_static_rec_mutex_lock (&article_refcount_mutex);
 	if (group->_articles_refcount >= 1) {
 		++group->_articles_refcount;
 		retval = TRUE;
 	}
-	g_static_mutex_unlock (&article_refcount_mutex);
+	g_static_rec_mutex_unlock (&article_refcount_mutex);
 
 	debug_exit ("group_ref_articles_if_loaded");
 	return retval;
@@ -1076,20 +1160,23 @@ group_ref_articles_if_loaded (Group * group)
 void
 group_ref_articles (Group * group, StatusItem * status)
 {
+	int refcount;
 	debug_enter ("group_ref_articles");
 
 	/* sanity clause */
 	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (group->_articles_refcount >= 0);
 
-	g_static_mutex_lock (&article_refcount_mutex);
-	if (++group->_articles_refcount == 1)
+	/* increment the refcount and load from disk if necessary */
+	g_static_rec_mutex_lock (&article_refcount_mutex);
+	refcount = ++group->_articles_refcount;
+	if (refcount == 1)
 	{
 		g_mutex_lock (group_get_article_mutex (group));
 		file_headers_load (group, status);
 		g_mutex_unlock (group_get_article_mutex (group));
 	}
-	g_static_mutex_unlock (&article_refcount_mutex);
+	g_static_rec_mutex_unlock (&article_refcount_mutex);
 
 	debug_exit ("group_ref_articles");
 }
@@ -1104,7 +1191,7 @@ group_unref_articles (Group * group, StatusItem * status)
 	g_return_if_fail (group->_articles_refcount > 0);
 
 	/* unref */
-	g_static_mutex_lock (&article_refcount_mutex);
+	g_static_rec_mutex_lock (&article_refcount_mutex);
 	if (group->_articles_refcount == 1)
 	{
 		g_mutex_lock (group_get_article_mutex (group));
@@ -1114,7 +1201,7 @@ group_unref_articles (Group * group, StatusItem * status)
 		g_mutex_unlock (group_get_article_mutex (group));
 	}
 	--group->_articles_refcount;
-	g_static_mutex_unlock (&article_refcount_mutex);
+	g_static_rec_mutex_unlock (&article_refcount_mutex);
 
 	/* cleanup */
 	debug_exit ("group_unref_articles");
@@ -1130,7 +1217,7 @@ group_get_name (const Group * group)
 {
 	g_return_val_if_fail (group_is_valid(group), _("(No Group)"));
 
-	return group->name;
+	return group->name.str;
 }
 
 void
@@ -1138,64 +1225,49 @@ group_get_collapsed_name (const Group   * group,
                           char          * buf,
                           size_t          bufsize)
 {
-	int pos = 0;
-	int len = 0;
-	const char * pch = NULL;
-	const char * name = group_get_name (group);
-	const char * march = name;
-	const char * last = "";
+	static const PString moderated = { "moderated", 9, 0u };
+	const char * name_str = group_get_name (group);
+	const char * bufend = buf + bufsize;
+	const char * bufbegin = buf;
+	const char * pch;
+	const PString * name = &group->name;
+	PString token = PSTRING_INIT;
+	PString long_token = PSTRING_INIT;
 
-	while (pos+2<bufsize && get_next_token_run (march, '.', &march, &pch, &len)) {
-		last = pch;
-		if (len > 0) {
-			buf[pos++] = *pch;
-			buf[pos++] = '.';
+	*buf = '\0';
+
+	/* find the long token -- use the last, unless that's "moderated" */
+	pch = pan_strrchr_len (name->str, name->len, '.');
+	if (pch == NULL)
+		long_token = pstring_shallow (name->str, name->len);
+	else {
+		token = pstring_substr_shallow (name, pch+1, NULL);
+		if (!pstring_equal (&moderated, &token))
+			long_token = token;
+		else {
+			token = pstring_substr_shallow (name, name->str, pch);
+			pch = pan_strrchr_len (token.str, token.len, '.');
+			if (pch != NULL)
+				long_token = pstring_substr_shallow (&token, pch+1, NULL);
 		}
 	}
-	if (pos != 0)
-		pos -= 2;
-	buf[pos] = '\0';
 
-	if (is_nonempty_string(last))
-		g_strlcpy (buf+pos, last, bufsize-pos);
-}
-
-
-/**
-***
-**/
-
-const gchar*
-group_chunk_string (Group         * group,
-                    const gchar   * chunkme,
-                    gboolean        share)
-{
-	gchar * retval = NULL;
-
-	g_return_val_if_fail (group_is_valid(group), NULL);
-
-	if (chunkme != NULL)
-	{
-		if (group->chunk == NULL)
-			group->chunk = g_string_chunk_new (81920);
-
-		if (share)
-			retval = g_string_chunk_insert_const (group->chunk, chunkme);
-		else
-			retval = g_string_chunk_insert (group->chunk, chunkme);
+	pch = name_str;
+	while (get_next_token_pstring (pch, '.', &pch, &token)) {
+		const int len = pstring_equal (&token, &long_token) ? token.len : 1;
+		buf += pan_strncpy_len (buf, bufend-buf, token.str, len);
+		buf += pan_strncpy_len (buf, bufend-buf, ".", 1);
 	}
 
-	return retval;
+	if (buf != bufbegin)
+		*--buf = '\0';
 }
 
-
 /***
 ****
-****
-****
 ***/
 
-Newsrc*
+static Newsrc*
 group_get_newsrc (Group * group)
 {
 	g_return_val_if_fail (group_is_valid(group), NULL);
@@ -1206,12 +1278,78 @@ group_get_newsrc (Group * group)
 	return group->_newsrc;
 }
 
-gboolean
-group_has_newsrc (const Group * group)
+void
+group_set_newsrc_read_string (Group            * group,
+                              const char       * range_string)
 {
-	g_return_val_if_fail (group_is_valid(group), FALSE);
+	g_return_if_fail (group_is_valid (group));
 
-	return group->_newsrc != NULL;
+	if (is_nonempty_string (range_string))
+	{
+		gulong newsrc_low = 0ul;
+		gulong newsrc_high = 0ul;
+		Newsrc * newsrc = group_get_newsrc (group);
+
+		/* initialize the newsrc */
+		newsrc_init_from_range_str (newsrc, range_string);
+
+		/* sync the newsrc and group ranges.*/
+		newsrc_get_group_range (newsrc, &newsrc_low,  &newsrc_high);
+		group_set_article_range (group,
+		                         MIN (newsrc_low, group->article_low),
+		                         MAX (newsrc_high, group->article_high));
+	}
+}
+
+gboolean
+group_is_article_read (const Group      * group,
+                       gulong             article_number)
+{
+	return newsrc_is_article_read (group_get_newsrc ((Group*)group), article_number);
+}
+                                                                                                                               
+gboolean
+group_mark_article_read (Group            * group,
+                         gulong             article_number,
+                         gboolean           read)
+{
+	return newsrc_mark_article (group_get_newsrc (group), article_number, read);
+}
+
+char*
+group_get_newsrc_export_string (const Group * group)
+{
+	char * retval = NULL;
+
+	g_return_val_if_fail (group_is_valid(group), NULL);
+
+	if (group->_newsrc == NULL)
+		retval = g_strdup ("");
+	else
+		retval = newsrc_export_line (group->_newsrc, group_get_name(group), group_is_subscribed(group));
+
+	return retval;
+}
+
+static char*
+get_newsrc_read_string (const Newsrc * newsrc)
+{
+	char * retval;
+
+	if (newsrc == NULL)
+		retval = g_strdup ("");
+	else
+		retval = newsrc_get_read_str (newsrc);
+
+	return retval;
+}
+
+char*
+group_get_newsrc_read_string (const Group * group)
+{
+	g_return_val_if_fail (group_is_valid(group), NULL);
+
+	return get_newsrc_read_string (group->_newsrc);
 }
 
 /**
@@ -1229,7 +1367,7 @@ group_has_newsrc (const Group * group)
  * the group's own low/high numbers are updated, in group_set_article_range(),
  * so that the deleted article list doesn't get too long when saved to disk.
  */
-Newsrc*
+static Newsrc*
 group_get_purged (Group * group)
 {
 	g_return_val_if_fail (group_is_valid(group), NULL);
@@ -1240,6 +1378,44 @@ group_get_purged (Group * group)
 	return group->_purged;
 }
 
+char*
+group_get_purged_string (const Group * group)
+{
+	g_return_val_if_fail (group_is_valid (group), NULL);
+
+	return get_newsrc_read_string (group->_purged);
+}
+
+                                                                                                                               
+void
+group_set_purged_string (Group            * group,
+                         const char       * range_string)
+{
+	g_return_if_fail (group_is_valid (group));
+
+	if (is_nonempty_string (range_string))
+	{
+		Newsrc * newsrc = group_get_purged (group);
+
+		newsrc_init_from_range_str (newsrc, range_string);
+
+		newsrc_set_group_range (newsrc, group->article_low, group->article_high);
+	}
+}
+
+void
+group_mark_article_purged (Group            * group,
+                           gulong             article_number)
+{
+	newsrc_mark_article (group_get_purged (group), article_number, TRUE);
+}
+
+
+
+/**
+***
+**/
+
 struct _Article*
 group_alloc_new_article (Group * group)
 {
@@ -1247,26 +1423,25 @@ group_alloc_new_article (Group * group)
 	g_return_val_if_fail (group_is_valid(group), NULL);
 
 	if (group->_article_chunk == NULL)
-		group->_article_chunk = memchunk_new (sizeof(Article), 500, FALSE);
+		group->_article_chunk = memchunk_new (sizeof(Article), 1024, FALSE);
 
 	a = (Article*) memchunk_alloc (group->_article_chunk);
 	a->group = group;
 	return a;
 }
 
-GHashTable*
+static GHashTable*
 group_get_articles (Group * group)
 {
 	g_return_val_if_fail (group_is_valid(group), NULL);
 
-	/* FIXME: shouldn't this load the articles ??? */
 	if (group->_articles == NULL)
-		group->_articles = g_hash_table_new (g_str_hash, g_str_equal);
+		group->_articles = g_hash_table_new (pstring_hash, pstring_equal);
 
 	return group->_articles;
 }
 
-GPtrArray*
+static GPtrArray*
 group_get_article_array (Group * group)
 {
 	GHashTable * hash;
@@ -1284,47 +1459,23 @@ group_get_article_array (Group * group)
 	return retval;
 }
 
-/***
-****
-****  EVENTS
-****
-***/
- 
-/**
-***  Groups Removed
-**/
- 
-PanCallback*
-group_get_groups_changed_callback (void)
+void
+group_article_forall      (Group               * group,
+                           GroupArticleFunc      func,
+                           gpointer              func_user_data)
 {
-	static PanCallback * cb = NULL;
-	if (cb==NULL) cb = pan_callback_new ();
-	return cb;
-}
-
-static void
-fire_groups_changed (Group ** g, gint qty)
-{
-	g_return_if_fail (g!=NULL);
-	debug_enter ("fire_groups_changed");
-
-	pan_callback_call (group_get_groups_changed_callback(), g, GINT_TO_POINTER(qty));
-
-	debug_exit ("fire_groups_changed");
-}
-
-PanCallback*
-group_get_articles_added_callback (void)
-{
-	static PanCallback * cb = NULL;
-	if (cb==NULL) cb = pan_callback_new ();
-	return cb;
-}
-
-PanCallback*
-group_get_articles_removed_callback (void)
-{
-	static PanCallback * cb = NULL;
-	if (cb==NULL) cb = pan_callback_new ();
-	return cb;
+	g_return_if_fail (group_is_valid (group));
+
+	if (group_ref_articles_if_loaded (group))
+	{
+		GPtrArray * a = group_get_article_array (group);
+		(func)(group, (Article**)a->pdata, a->len, func_user_data);
+		g_ptr_array_free (a, TRUE);
+
+		group_unref_articles (group, NULL);
+	}
+	else
+	{
+		(func)(group, NULL, 0u, func_user_data);
+	}
 }
