@@ -20,7 +20,7 @@
 #ifndef __MESSAGE_CHECK_H__
 #define __MESSAGE_CHECK_H__
 
-#include <glib.h>
+#include <glib/garray.h>
 #include <gmime/gmime-message.h>
 #include <pan/base/server.h>
 
