@@ -23,11 +23,11 @@
 #include <pan/base/server.h>
 
 void newsrc_import (Server          * server,
-                    const char      * filename,
+                    const PString   * filename,
                     gboolean          subscribed_only);
 
 void newsrc_export (Server          * server,
-                    const char      * filename,
+                    const PString   * filename,
                     gboolean          subscribed_only);
 
 #define __NEWSRC_PORT_H__
