@@ -52,12 +52,13 @@ typedef enum
 	IS_STRING,
 	IS_BOOL,
 	IS_INT,
-	IS_UNSIGNED_LONG
+	IS_UNSIGNED_LONG,
+	PAN_CONFIG_NODE_TYPE_QTY
 } PanConfigNodeType;
 
 typedef struct 
 {
-	char              * key;
+	PString key;
 	PanConfigNodeType   type;
 	union
 	{
@@ -77,28 +78,28 @@ static GNode       * config_root = NULL;
 static gboolean      config_is_dirty = FALSE;
 
 static PanConfigNode *
-config_create_empty_node (const char * key, int key_len)
+config_create_empty_node (const PString * key)
 {
 	PanConfigNode * node = g_new (PanConfigNode, 1);
-	node->key = g_strndup (key, key_len);
+	node->key = PSTRING_INIT;
+	pstring_copy (&node->key, key);
 	node->type = IS_NOTYPE;
 	node->value.val_int = 0;
 	return node;
 }
 
 static PanConfigNode *
-config_create_node (const char        * key,
-                    int                 key_len,
+config_create_node (const PString     * key,
 		    PanConfigNodeType   type,
 		    va_list             ap)
 {
 	PanConfigNode * node = g_new (PanConfigNode, 1);
 
 	g_return_val_if_fail (node != NULL, NULL);
-	g_return_val_if_fail (is_nonempty_string(key), NULL);
-	g_return_val_if_fail (key_len>0, NULL);
+	g_return_val_if_fail (pstring_is_set (key), NULL);
 
-	node->key = g_strndup (key, key_len);
+	node->key = PSTRING_INIT;
+	pstring_copy (&node->key, key);
 	node->type = type;
 	
 	switch (type)
@@ -135,7 +136,7 @@ config_node_deleter (GNode * node, gpointer data)
 
 	if (config->type == IS_STRING)
 		g_free (config->value.val_string);
-	g_free (config->key);
+	pstring_clear (&config->key);
 	g_free (config);
 
 	node->data = NULL;
@@ -161,13 +162,12 @@ config_delete_node (GNode * node)
 }
 
 static GNode *
-config_scan_children (GNode * node, const char * key, int key_len)
+config_scan_children (GNode * node, const PString * key)
 {
 	GNode * child = NULL;
 
 	g_return_val_if_fail (node!=NULL, NULL);
-	g_return_val_if_fail (is_nonempty_string(key), NULL);
-	g_return_val_if_fail (key_len>0, NULL);
+	g_return_val_if_fail (pstring_is_set (key), NULL);
 
 	child = g_node_first_child (node);
 
@@ -175,7 +175,7 @@ config_scan_children (GNode * node, const char * key, int key_len)
 	{
 		const PanConfigNode * data = (const PanConfigNode*) child->data;
 
-		if (!strncmp (key, data->key, key_len))
+		if (pstring_equal (key, &data->key))
 			break;
 
 		child = g_node_next_sibling (child);
@@ -189,16 +189,15 @@ config_search_node (const char * key)
 {
 	GNode    * node = config_root;
 	const char * march;
-	const char * str;
-	int str_len;
+	PString str = PSTRING_INIT;
 
 	g_return_val_if_fail (config_root!=NULL, NULL);
 	g_return_val_if_fail (is_nonempty_string(key), NULL);
 	g_return_val_if_fail (*key=='/', NULL);
 
 	march = key + 1;
-	while (node!=NULL && get_next_token_run (march, '/', &march, &str, &str_len))
-		node = config_scan_children (node, str, str_len);
+	while (node!=NULL && get_next_token_pstring (march, '/', &march, &str))
+		node = config_scan_children (node, &str);
 
 	return node;
 }
@@ -212,8 +211,7 @@ config_set_or_add_node (const char        * key,
 	GNode * node = config_root;
 	va_list ap;
 	const char * march = NULL;
-	const char * str = NULL;
-	int str_len = 0;
+	PString str = PSTRING_INIT;
 
 	/* sanity clause */
 	g_return_if_fail (config_root!=NULL);
@@ -223,23 +221,23 @@ config_set_or_add_node (const char        * key,
 	/* loop down the key */
 	march = key + 1;
 	va_start (ap, type);
-	while (get_next_token_run (march, '/', &march, &str, &str_len))
+	while (get_next_token_pstring (march, '/', &march, &str))
 	{
-		GNode * child = config_scan_children (node, str, str_len);
-		const gboolean is_bottom = str[str_len] == '\0';
+		GNode * child = config_scan_children (node, &str);
+		const gboolean is_bottom = str.str[str.len] == '\0';
 
 		if (child == NULL)
 		{
 			PanConfigNode * data = is_bottom
-				? config_create_node (str, str_len, type, ap)
-				: config_create_empty_node (str, str_len);
+				? config_create_node (&str, type, ap)
+				: config_create_empty_node (&str);
 
 			child = g_node_append (node, g_node_new(data));
 		}
 		else if (is_bottom)
 		{
 			config_node_deleter (child, NULL);
-			child->data = config_create_node (str, str_len, type, ap);
+			child->data = config_create_node (&str, type, ap);
 		}
 
 		node = child;
@@ -264,90 +262,53 @@ get_config_filename (char * buf, int buflen)
 ***  Saving PanConfig
 **/
 
-static guint last_depth = 0;	/* Not too crazy about this one */
-
-static void 
-newline_depth (GString * appendme, guint depth)
+static xmlNodePtr
+write_node_recursive (xmlDocPtr doc, GNode * gnode)
 {
-	guint i;
+	static const char * types [PAN_CONFIG_NODE_TYPE_QTY] = { "?", "s", "b", "n", "u" };
+	PanConfigNode * config;
+	xmlNodePtr xnode;
 
-	g_string_append_c (appendme, '\n');
-	for (i=0; i<depth; ++i)
-		g_string_append_c (appendme, '\t');
-}
+	/* sanity clause */
+	g_return_val_if_fail (doc!=NULL, NULL);
+	g_return_val_if_fail (gnode!=NULL, NULL);
+	g_return_val_if_fail (gnode->data!=NULL, NULL);
 
-static void
-close_previous_level (GString * appendme, guint last_level, guint level)
-{
-	guint i;
-
-	for (i=last_level; i>level; i--)
+	/* current */
+	config = (PanConfigNode*) gnode->data;
+	if (config->type == IS_NOTYPE) /* it's a section */
 	{
-		newline_depth (appendme, i-2);
-		g_string_append (appendme, "</section>");
+		xnode = xmlNewDocNode (doc, NULL, "section", NULL);
+		xmlSetProp (xnode, "key", config->key.str);
 	}
-}
-
-static void
-pan_g_string_append_escaped (GString * gstr, char * escapeme)
-{
-	char * pch = pan_str_escape (escapeme);
-	g_string_append (gstr, pch);
-	g_free (pch);
-}
-
-static gboolean
-write_node (GNode * node, gpointer data)
-{
-	char            types[] = "_sbn";
-	GString       * appendme = (GString *) data;
-	PanConfigNode * config = node->data;
-	guint           depth = g_node_depth (node);
-
-	if (depth == 1)
-		return FALSE;
-
-	close_previous_level (appendme, last_depth, depth);
-
-	newline_depth (appendme, depth-1);
-	if (config->type == IS_NOTYPE)
+	else /* it's a key */
 	{
-		g_string_append (appendme, "<section key=\"");
-		pan_g_string_append_escaped (appendme, config->key);
-		g_string_append (appendme, "\">");
-	}
-	else
-	{
-		g_string_append (appendme, "<value key=\"");
-		pan_g_string_append_escaped (appendme, config->key);
-		g_string_append_printf (appendme, "\" type=\"%c\">", types[config->type]);
+		char buf[64];
+		char * content = buf;
+		*buf = '\0';
 
-		switch (config->type)
-		{
-			case IS_STRING:
-				if (config->value.val_string != NULL)
-					pan_g_string_append_escaped (appendme, config->value.val_string);
-				break;
-			case IS_BOOL:
-				g_string_append_printf (appendme, "%s", config->value.val_int ? "true" : "false");
-				break;
-			case IS_INT:
-				g_string_append_printf (appendme, "%d", config->value.val_int);
-				break;
-			case IS_UNSIGNED_LONG:
-				g_string_append_printf (appendme, "%lu", config->value.val_ulong);
-				break;
-			case IS_NOTYPE:
-				pan_warn_if_reached ();
-				break;
+		switch (config->type) {
+			case IS_STRING:         content = config->value.val_string ? config->value.val_string : ""; break;
+			case IS_BOOL:           content = config->value.val_int ? "true" : "false"; break;
+			case IS_INT:            g_snprintf (buf, sizeof(buf), "%d", config->value.val_int); break;
+		        case IS_UNSIGNED_LONG:  g_snprintf (buf, sizeof(buf), "%lu", config->value.val_ulong); break;
+                        default:                pan_warn_if_reached (); break;
 		}
 
-		g_string_append (appendme, "</value>");
+		xnode = xmlNewDocNode (doc, NULL, "value", NULL);
+		xmlSetProp (xnode, "key", config->key.str);
+		xmlSetProp (xnode, "type", types[config->type]);
+		xmlNodeAddContent (xnode, content);
 	}
 
-	last_depth = depth;
+	/* children */
+	if (gnode->children != NULL) {
+		GNode * tmp;
+		for (tmp=gnode->children; tmp!=NULL; tmp=tmp->next)
+			xmlAddChild (xnode, write_node_recursive (doc, tmp));
+	}
 
-	return FALSE;
+	return xnode;
 }
 
 static void
@@ -355,24 +316,20 @@ pan_config_save (void)
 {
 	if (config_root != NULL)
 	{
+		GNode * tmp;
 		char filename[PATH_MAX];
-		GString * output = g_string_sized_new (4096);
+		xmlDocPtr doc;
 
-		/* build the datafile */
-		g_string_assign (output, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
-		                         "<!DOCTYPE config SYSTEM \"config.dtd\">\n"
-		                         "<config>");
-		last_depth = 0;
-		g_node_traverse (config_root, G_PRE_ORDER, G_TRAVERSE_ALL, -1, write_node, output);
-		close_previous_level (output, last_depth, 2);
-		g_string_append (output, "\n</config>");
+		/* build the tree */
+		doc = xmlNewDoc ("1.0");
+		doc->children = xmlNewDocNode (doc, NULL, "config", NULL);
+		for (tmp=config_root->children; tmp!=NULL; tmp=tmp->next)
+			xmlAddChild (doc->children, write_node_recursive (doc, tmp));
 
-		/* save the datafile */
+		/* write it out */
 		get_config_filename (filename, sizeof(filename));
-		pan_file_write_datafile (filename, output->str, output->len);
-
-		/* cleanup */
-		g_string_free (output, TRUE);
+		xmlSaveFormatFileEnc (filename, doc, "UTF-8", 1);
+		xmlFreeDoc (doc);
 	}
 }
 
@@ -501,21 +458,25 @@ pan_config_load_ini (void)
 		pch = strchr (line->str, '=');
 		if (pch != NULL)
 		{
+			char * freeme;
+			const char * val_utf8;
 			char * key = g_strndup (line->str, pch-line->str);
 			char * val = g_strdup (pch+1);
 
 			g_strstrip (key);
 			g_strstrip (val);
+			val_utf8 = pan_utf8ize (val, -1, &freeme);
 
 			if (strstr(key,"font") == NULL) /* omit pre-Pango font settings */
 			{
 				char * path = g_strdup_printf ("/Pan/%s/%s", category, key);
-				config_set_or_add_node (path, IS_STRING, val);
+				config_set_or_add_node (path, IS_STRING, val_utf8);
 				g_free (path);
 
 				++qty;
 			}
 
+			g_free (freeme);
 			g_free (val);
 			g_free (key);
 		}
@@ -533,9 +494,10 @@ pan_config_load_ini (void)
 static void
 pan_config_load (void)
 {
-	xmlDocPtr    doc;
-	xmlNodePtr   node;
-	char         fname[PATH_MAX];
+	const PString  root_string = pstring_shallow ("", 0);
+	xmlDocPtr      doc;
+	xmlNodePtr     node;
+	char           fname[PATH_MAX];
 
 	g_return_if_fail (config_root==NULL);
 
@@ -544,7 +506,7 @@ pan_config_load (void)
 		return ;
 	}
 
-	config_root = g_node_new (config_create_empty_node ("", 0));
+	config_root = g_node_new (config_create_empty_node (&root_string));
 
 	get_config_filename (fname, sizeof(fname));
 
@@ -574,7 +536,7 @@ pan_config_load (void)
 static char * prefix = NULL;
 
 static void
-compose_key (const char * key, char * buf, int buf_max)
+compose_key (char * buf, int buf_max, const char * key)
 {
 	if (prefix)
 		g_snprintf (buf, buf_max, "%s/%s", prefix, key);
@@ -585,216 +547,130 @@ compose_key (const char * key, char * buf, int buf_max)
 void
 pan_config_push_prefix (const char * new_prefix)
 {
-	if (prefix != NULL)
-		g_free (prefix);
+	int prefix_len;
 
-	prefix = g_strdup (new_prefix);
+	g_return_if_fail (new_prefix != NULL);
 
-	if (prefix[strlen(prefix)-1] == '/')
-		prefix[strlen(prefix)-1] = '\0';
+	replace_gstr (&prefix, NULL);
+
+	prefix_len = strlen (new_prefix);
+	prefix = g_memdup (new_prefix, prefix_len+1);
+	if (prefix[prefix_len-1] == '/')
+		prefix[prefix_len-1] = '\0';
 }
 
 void
 pan_config_pop_prefix (void)
 {
-	g_free (prefix);
-	prefix = NULL;
+	replace_gstr (&prefix, NULL);
 }
 
 
-/**
- ** Parsing defaults out of a key
- **/
-
-static char *
-parse_default_string (char * key)
-{
-	char   * val = NULL;
-	char   * pch = strrchr (key, '=');
-
-	if (pch != NULL)
-	{
-		val = g_strdup (pch+1);
-		*pch = '\0';
-	}
-
-	return val;
-}
-
-
-static gboolean
-parse_default_bool (char * key)
-{
-	gboolean   val = FALSE;
-	char    * pch = strrchr (key, '=');
-
-	if (pch != NULL)
-	{
-#if 1
-		char * tmp = g_ascii_strup (pch+1, -1);
-#else
-		char * tmp = g_strdup (pch+1);
-		g_strup (pch+1);
-#endif
-
-		val = tolower(tmp[0]) == 't';
-		*pch = '\0';
-
-		g_free (tmp);
-	}
-
-	return val;
-}
-
-
-static int
-parse_default_int (char * key)
-{
-	int    val = 0;
-	char * pch = strrchr (key, '=');
-
-	if (pch != NULL)
-	{
-		val = atoi (pch+1);
-		*pch = '\0';
-	}
-
-	return val;
-}
-
-static gulong
-parse_default_ulong (char * key)
-{
-	gulong retval = 0;
-	char * pch = strrchr (key, '=');
-
-	if (pch != NULL)
-	{
-		retval = strtoul (pch+1, NULL, 10);
-		*pch = '\0';
-	}
-
-	return retval;
-}
-
 /**
  ** GETTING 
  **/
 
-char*
-pan_config_get_string (const char * key)
+static void
+ensure_config_loaded (void)
 {
-	char            new_key[MAX_KEY_LEN];
-	char          * val;
-	GNode         * node;
-	PanConfigNode * data;
-	debug_enter ("pan_config_get_string");
-
 	if (config_root == NULL)
 		pan_config_load ();
+}
 
-	compose_key (key, new_key, sizeof(new_key));
-	val = parse_default_string (new_key);
+char*
+pan_config_get_string (const char * key, const char * default_value)
+{
+	char              full_key[MAX_KEY_LEN];
+	char            * val;
+	GNode           * node;
+	PanConfigNode   * data;
+	debug_enter ("pan_config_get_string");
 
-	node = config_search_node (new_key);
+	ensure_config_loaded ();
+
+	compose_key (full_key, sizeof(full_key), key);
+	node = config_search_node (full_key);
 	data = (PanConfigNode *) (node ? node->data : NULL);
 
-	if (data != NULL && data->type == IS_STRING)
-	{
-		g_free (val);
+	val = NULL;
+
+	if (data!=NULL && data->type == IS_STRING)
 		val = g_strdup (data->value.val_string);
-	}
+
+	if (!is_nonempty_string (val))
+		replace_gstr (&val, g_strdup (default_value));
 
-	/*g_message ("get [%s] returns string [%s]", key, val ? val : "(null)");*/
 	debug_exit ("pan_config_get_string");
 	return val;
 }
 
 gboolean
-pan_config_get_bool (const char * key)
+pan_config_get_bool (const char * key, gboolean default_value)
 {
-	char            new_key[MAX_KEY_LEN];
+	char            full_key[MAX_KEY_LEN];
 	gboolean        val;
 	GNode         * node;
 	PanConfigNode * data;
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
-	compose_key (key, new_key, sizeof(new_key));
-	val = parse_default_bool (new_key);
-
-	node = config_search_node (new_key);
+	compose_key (full_key, sizeof(full_key), key);
+	node = config_search_node (full_key);
 	data = (PanConfigNode *) (node ? node->data : NULL);
 
-	if (data != NULL) {
-	       if (data->type == IS_BOOL)
-		       val = data->value.val_bool;
-	       else if (data->type == IS_INT)
-		       val = data->value.val_int != 0;
-	       else if (data->type == IS_STRING)
-		       val = !pan_strcmp (data->value.val_string, "true");
-	}
+	if (data == NULL)                  val = default_value;
+	else if (data->type == IS_BOOL)    val = data->value.val_bool;
+	else if (data->type == IS_INT)     val = data->value.val_int != 0;
+	else if (data->type == IS_STRING)  val = data->value.val_string!=NULL && tolower(data->value.val_string[0])=='t';
+	else                               val = default_value;
 
-	/*g_message ("get [%s] returns bool [%d]", key, val);*/
 	return val;
 }
 
 
 int
-pan_config_get_int (const char * key)
+pan_config_get_int (const char * key, int default_value)
 {
-	int             val;
-	char            new_key[MAX_KEY_LEN];
-	GNode         * node;
-	PanConfigNode * data;
+	int                val;
+	char               full_key[MAX_KEY_LEN];
+	GNode            * node;
+	PanConfigNode    * data;
 	debug_enter ("pan_config_get_int");
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
-	compose_key (key, new_key, sizeof(new_key));
-	val = parse_default_int (new_key);
-
-	node = config_search_node (new_key);
+	compose_key (full_key, sizeof(full_key), key);
+	node = config_search_node (full_key);
 	data = (PanConfigNode *) (node ? node->data : NULL);
-	
-	if (data) {
-		if (data->type == IS_INT)
-			val = data->value.val_int;
-		else if (data->type == IS_STRING)
-			val = atoi (data->value.val_string);
-	}
 
-	/*g_message ("get [%s] returns int [%d]", key, val);*/
+	if (data == NULL)                   val = default_value;
+	else if (data->type == IS_INT)	    val = data->value.val_int;
+	else if (data->type == IS_STRING)   val = atoi (data->value.val_string);
+	else                                val = default_value;
+
 	debug_exit ("pan_config_get_int");
 	return val;
 }
 
 gulong
-pan_config_get_ulong (const char * key)
+pan_config_get_ulong (const char * key, gulong default_value)
 {
-	gulong          val;
-	char            new_key[MAX_KEY_LEN];
-	GNode         * node;
-	PanConfigNode * data;
+	gulong             val;
+	char               full_key[MAX_KEY_LEN];
+	GNode            * node;
+	PanConfigNode    * data;
 	debug_enter ("pan_config_get_ulong");
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
-	compose_key (key, new_key, sizeof(new_key));
-	val = parse_default_ulong (new_key);
-	
-	node = config_search_node (new_key);
+	compose_key (full_key, sizeof(full_key), key);
+	node = config_search_node (full_key);
 	data = (PanConfigNode *) (node ? node->data : NULL);
-	
-	if (data) {
-		if (data->type == IS_UNSIGNED_LONG)
-			val = data->value.val_ulong;
-		else if (data->type == IS_STRING)
-			val = strtoul (data->value.val_string, NULL, 10);
-	}
+
+	if (data == NULL)                          val = default_value;
+	else if (data->type == IS_UNSIGNED_LONG)   val = data->value.val_ulong;
+	else if (data->type == IS_STRING)          val = strtoul (data->value.val_string, NULL, 10);
+	else                                       val = default_value;
 
 	debug_exit ("pan_config_get_ulong");
 	return val;
@@ -807,8 +683,7 @@ pan_config_has_section (const char * section)
 	GNode         * node = NULL;
 	PanConfigNode * data = NULL;
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
 	node = config_search_node (section);
 	data = (PanConfigNode *) (node ? node->data : NULL);
@@ -824,54 +699,88 @@ pan_config_has_section (const char * section)
 void
 pan_config_set_string (const char * key, const char * value)
 {
-	char new_key[MAX_KEY_LEN];
+	char full_key[MAX_KEY_LEN];
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
 	if (value == NULL)
 		value = "";
 
-	compose_key (key, new_key, sizeof(new_key));
-	config_set_or_add_node (new_key, IS_STRING, value);
+	compose_key (full_key, sizeof(full_key), key);
+	config_set_or_add_node (full_key, IS_STRING, value);
 }
 
+void
+pan_config_set_string_if_different (const char * key, const char * value, const char * default_value)
+{
+	if (!pan_strcmp (value, default_value))
+		pan_config_clean_key (key);
+	else
+		pan_config_set_string (key, value);
+}
 
 void
 pan_config_set_bool (const char * key, gboolean value)
 {
-	char new_key[MAX_KEY_LEN];
+	char full_key[MAX_KEY_LEN];
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
-	compose_key (key, new_key, sizeof(new_key));
-	config_set_or_add_node (new_key, IS_BOOL, value);
+	compose_key (full_key, sizeof(full_key), key);
+	config_set_or_add_node (full_key, IS_BOOL, value);
+}
+
+void
+pan_config_set_bool_if_different (const char * key, gboolean value, gboolean default_value)
+{
+	value = value != 0;
+	default_value = default_value != 0;
+
+	if (value == default_value)
+		pan_config_clean_key (key);
+	else
+		pan_config_set_bool (key, value);
 }
 
 
 void
 pan_config_set_int (const char * key, int value)
 {
-	char new_key[MAX_KEY_LEN];
+	char full_key[MAX_KEY_LEN];
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
-	compose_key (key, new_key, sizeof(new_key));
-	config_set_or_add_node (new_key, IS_INT, value);
+	compose_key (full_key, sizeof(full_key), key);
+	config_set_or_add_node (full_key, IS_INT, value);
+}
+
+void
+pan_config_set_int_if_different (const char * key, int value, int default_value)
+{
+	if (value == default_value)
+		pan_config_clean_key (key);
+	else
+		pan_config_set_int (key, value);
 }
 
 void
 pan_config_set_ulong (const char * key, gulong value)
 {
-	char new_key[MAX_KEY_LEN];
+	char full_key[MAX_KEY_LEN];
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
-	compose_key (key, new_key, sizeof(new_key));
-	config_set_or_add_node (new_key, IS_UNSIGNED_LONG, value);
+	compose_key (full_key, sizeof(full_key), key);
+	config_set_or_add_node (full_key, IS_UNSIGNED_LONG, value);
+}
+
+void
+pan_config_set_ulong_if_different (const char * key, gulong value, gulong default_value)
+{
+	if (value == default_value)
+		pan_config_clean_key (key);
+	else
+		pan_config_set_ulong (key, value);
 }
 
 void
@@ -879,8 +788,7 @@ pan_config_clean_key (const char * key)
 {
 	GNode * node;
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
 	node = config_search_node (key);
 
@@ -898,8 +806,7 @@ pan_config_clean_section (const char * section)
 {
 	GNode * node;
 
-	if (config_root == NULL)
-		pan_config_load ();
+	ensure_config_loaded ();
 
 	node = config_search_node (section);
 
@@ -908,6 +815,3 @@ pan_config_clean_section (const char * section)
 		config_delete_node (node);
 	}
 }
-
-
-
