@@ -116,6 +116,79 @@ replace_gstr (char   ** oldaddr,
 	g_free (oldval);
 }
 
+int
+pan_strncpy_len (char        * target,
+                 const int     target_size,
+                 const char  * source_str,
+                 const int     source_len)
+{
+	int len;
+
+	/* sanity clause */
+	g_return_val_if_fail (target_size >= 0, 0);
+	g_return_val_if_fail (target!=NULL || target_size==0, 0);
+	g_return_val_if_fail (source_len >= 0, 0);
+	g_return_val_if_fail (source_str!=NULL || source_len==0, 0);
+
+	len = MIN (target_size-1, source_len);
+	memcpy (target, source_str, len);
+	target[len] = '\0';
+
+	return len;
+}
+
+char*
+pan_strchr_len (const char * haystack,
+                gssize       haystack_len,
+                char         needle)
+{
+	g_return_val_if_fail (haystack!=NULL || haystack_len==0, NULL);
+
+	if (haystack_len > 0)
+	{
+		register const char * end = haystack + haystack_len;
+		for (; haystack!=end; ++haystack)
+			if (*haystack == needle)
+				return (char*) haystack;
+	}
+
+	return NULL;
+}
+
+char*
+pan_strrchr_len (const char * haystack,
+                 gssize       haystack_len,
+                 char         needle)
+{
+	register const char * pch = NULL;
+
+	g_return_val_if_fail (haystack!=NULL || haystack_len==0, NULL);
+
+	if (haystack_len > 0)
+	{
+		pch = haystack + haystack_len - 1;
+		for (; pch!=haystack; --pch)
+			if (*pch == needle)
+				break;
+		if (*pch != needle)
+			pch = NULL;
+	}
+
+	return (char*) pch;
+}
+
+int
+pan_strcmp_len  (const char * str_a,
+                 int          str_a_len,
+                 const char * str_b,
+                 int          str_b_len)
+{
+	int retval = memcmp (str_a, str_b, MIN(str_a_len,str_b_len));
+        if (!retval)
+                retval = str_a_len - str_b_len;
+        return retval;
+}
+
 /**
  * Replaces the search string inside original with the replace string.
  * This should be safe with overlapping elements of text, though I haven't
@@ -321,15 +394,23 @@ pan_g_string_replace (GString       * string,
 void
 pan_g_string_strstrip (GString * string)
 {
+	int i;
+
+	/* sanity clause */
 	g_return_if_fail (string != NULL);
 
-	if (string->len != 0)
-	{
-		g_strstrip (string->str);
-		string->len = strlen (string->str);
-	}
-}
+	/* trim trailing space */
+	for (i=string->len-1; i>=0; --i)
+		if (!isspace ((guchar)string->str[i]))
+			break;
+	g_string_truncate (string, i+1);
 
+	/* trim leading space */
+	for (i=0; i<string->len; ++i)
+		if (!isspace ((guchar)string->str[i]))
+			break;
+	g_string_erase (string, 0, i);
+}
 
 /***
 ****
@@ -387,6 +468,25 @@ get_next_token_run (const char   * pch,
 	return is_nonempty_string(start) ? start : NULL;
 }
 
+gboolean
+get_next_token_pstring (const char   * pch,
+                        char           delimiter,
+                        const char  ** setme_next_token,
+                        PString      * setme_pstring)
+{
+	const char * start = pch;
+
+	if (pch!=NULL)
+		while (*pch && *pch!=delimiter)
+			++pch;
+
+	*setme_pstring = pstring_shallow (start, pch!=NULL ? pch-start : 0);
+
+	if (setme_next_token != NULL)
+		*setme_next_token = pch!=NULL && *pch==delimiter ? pch+1 : pch;
+
+	return is_nonempty_string(start);
+}
 
 glong
 get_next_token_long (const char   * pch,
@@ -899,7 +999,7 @@ pan_body_to_utf8 (GMimePart     * part,
 {
 	char * retval = NULL;
 	const char * content;
-	size_t content_len = 0ul;
+	guint content_len = 0u;
 
 	/* sanity clause */
 	g_return_val_if_fail (GMIME_IS_PART(part), NULL);
