@@ -89,6 +89,33 @@ pan_file_normalize_inplace (char * filename)
 	return filename;
 }
 
+char*
+pan_file_normalize (const char * filename, const char * fallback_absolute_path)
+{
+	char * pch;
+
+	g_return_val_if_fail (is_nonempty_string(filename), g_strdup(filename));
+
+	/* take care of paths beginning with ~ */
+	pch = filename[0]=='~' && filename[1]=='/'
+		? g_build_filename (pan_get_home_dir(), filename+2, NULL)
+		: g_strdup (filename);
+
+	g_strstrip (pch);
+
+	/* make sure the filename is absolute */
+	if (!g_path_is_absolute (pch))
+	{
+		if (fallback_absolute_path == NULL)
+			fallback_absolute_path = pan_get_home_dir ();
+		replace_gstr (&pch, g_build_filename (fallback_absolute_path, pch, NULL));
+	}
+
+	/* remove any unnecessary directory characters */
+	pan_file_normalize_inplace (pch);
+
+	return pch;
+}
 
 gboolean
 pan_file_exists (const char* filename)
@@ -121,7 +148,7 @@ pan_file_ensure_path_exists (const char *pathname)
 	if (!g_file_test (pathname, G_FILE_TEST_IS_DIR))
 	{
 		const char * in = pathname;
-		char * buf = g_alloca (strlen(pathname)+1);
+		char * buf = g_newa (char, strlen(pathname)+1);
 		char * out = buf;
 
 		*out++ = *in++;
@@ -166,8 +193,10 @@ pan_file_copy (const char * old_filename,
 	char buf [4096];
 	size_t size_read;
 
+	/* sanity clause */
 	g_return_val_if_fail (is_nonempty_string(old_filename), -1);
 	g_return_val_if_fail (is_nonempty_string(new_filename), -1);
+	g_return_val_if_fail (pan_file_exists (old_filename), -1);
 
 	in = fopen (old_filename, "rb");
 	if (in == NULL) {
@@ -194,8 +223,10 @@ gboolean
 pan_file_rename (const char * old_filename,
                  const char * new_filename)
 {
+	/* sanity clause */
 	g_return_val_if_fail (is_nonempty_string(old_filename), FALSE);
 	g_return_val_if_fail (is_nonempty_string(new_filename), FALSE);
+	g_return_val_if_fail (pan_file_exists (old_filename), FALSE);
 
 	if (rename (old_filename, new_filename)) {
 		if (!pan_file_copy (old_filename, new_filename))
@@ -227,7 +258,7 @@ pan_mkdir (const char * path, gulong mode)
 	else
 	{
 		errno = 0;
-#ifdef MKDIR_TAKES_ONE_ARG
+#ifdef G_OS_WIN32
 		retval = mkdir (path);
 #else
 		retval = mkdir (path, mode);
@@ -238,3 +269,86 @@ pan_mkdir (const char * path, gulong mode)
 	
 	return retval;
 }
+
+gboolean
+pan_file_swap_datafile (const char  * filename_tmp,
+                        const char  * filename)
+{
+	gboolean ok = TRUE;
+	char * filename_bak = NULL;
+
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(filename), FALSE);
+	g_return_val_if_fail (is_nonempty_string(filename_tmp), FALSE);
+	g_return_val_if_fail (pan_file_exists (filename_tmp), FALSE);
+
+	/* backup the old file */
+	if (ok && pan_file_exists(filename)) {
+		filename_bak = g_strdup_printf ("%s.bak", filename);
+		ok = pan_file_rename (filename, filename_bak);
+		if (!ok) {
+			log_add_va (LOG_ERROR, _("Can't backup \"%s\" as \"%s\": %s"),
+				    filename, filename_bak, g_strerror(errno));
+		}
+	}
+
+	/* replace the old file with the temp file */
+	if (ok) {
+		ok = pan_file_rename (filename_tmp, filename);
+		if (!ok) {
+			log_add_va (LOG_ERROR, _("Can't backup \"%s\" as \"%s\": %s"),
+				    filename, filename_bak, g_strerror(errno));
+			/* if we just made a backup in the previous paragraph,
+			 * restore from that backup.  Use copy() rather than rename()
+			 * so that even in the worst case we'll still have the backup. */
+			if (filename_bak != NULL)
+				pan_file_copy (filename_bak, filename);
+		}
+	}
+
+	/* cleanup */
+	g_free (filename_bak);
+
+	return ok;
+}
+
+gboolean
+pan_file_write_datafile (const char  * filename,
+                         const char  * data,
+                         gulong        data_len)
+{
+	char * filename_tmp;
+	FILE * fp;
+	gboolean ok;
+
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string (filename), FALSE);
+	g_return_val_if_fail (data_len > 0ul, FALSE);
+	g_return_val_if_fail (data != NULL, FALSE);
+
+	/* write the data to a temporary file */
+	filename_tmp = g_strdup_printf ("%s.tmp", filename);
+	fp = fopen (filename_tmp, "w+");
+	ok = fp != NULL;
+	if (!ok) {
+		log_add_va (LOG_ERROR, _("Can't write to \"%s\": %s"),
+		            filename_tmp, g_strerror (errno));
+	}
+	else {
+		const size_t written  = fwrite (data, sizeof(char), data_len, fp);
+		fclose (fp);
+		ok = data_len == written;
+		if (!ok) {
+			log_add_va (LOG_ERROR, _("Can't write to \"%s\": %s"),
+			            filename_tmp, g_strerror (errno));
+		}
+	}
+
+	/* move the temp file to the datafile */
+	if (ok)
+		pan_file_swap_datafile (filename_tmp, filename);
+
+	/* cleanup */
+	g_free (filename_tmp);
+	return ok;
+}
