@@ -1002,7 +1002,8 @@ pan_g_mime_message_mark_read (GMimeMessage * message)
 	{
 		GString * token = g_string_new (NULL);
 		Server * server = serverlist_get_active_server ();
-		const char * message_id = g_mime_message_get_message_id (message);
+		const char * message_id_str = g_mime_message_get_message_id (message);
+		const PString message_id = pstring_shallow (message_id_str, -1);
 
 		/* skip servername */
 		skip_next_token (xref, ' ', &xref);
@@ -1014,26 +1015,19 @@ pan_g_mime_message_mark_read (GMimeMessage * message)
 			char * delim = strchr (token->str, ':');
 			if (delim != NULL)
 			{
-				const char * group_name;
-				Group * group;
-
-				*delim = '\0';
-
 				/* if the group is subscribed, mark the article as read */
-				group_name = token->str;
-				group = server_get_named_group_in_type (server, group_name, SERVER_GROUPS_SUBSCRIBED);
+				const PString group_name = pstring_shallow (token->str, delim-token->str);
+				Group * group = server_get_named_group_in_type (server, &group_name, SERVER_GROUPS_SUBSCRIBED);
 				if (group != NULL)
 				{
 					if (group_ref_articles_if_loaded (group)) {
-						Article * article = group_get_article_by_message_id (group, message_id);
+						Article * article = group_get_article_by_message_id (group, &message_id);
 						if (article != NULL)
 							articles_set_read_simple (&article, 1, TRUE);
 						group_unref_articles (group, NULL);
 					}
 					else if (group_is_subscribed (group))
-						newsrc_mark_article (group_get_newsrc(group),
-								     strtoul (delim+1, NULL, 10),
-								     TRUE);
+						group_mark_article_read (group, strtoul (delim+1, NULL, 10), TRUE);
 
 				}
 			}
@@ -1064,17 +1058,22 @@ pan_g_mime_message_get_group (GMimeMessage * message)
 		const char * newsgroups = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
 		if (is_nonempty_string (newsgroups))
 		{
-			const char * march = newsgroups;
-			GString * group_name = g_string_new (NULL);
+			PString group_name;
+
 			for (;;)
 			{
-				if (!get_next_token_g_str (march, ',', &march, group_name))
+				int group_name_len;
+				const char * group_name_str;
+				const char * march = newsgroups;
+
+				if (!get_next_token_run (march, ',', &march, &group_name_str, &group_name_len))
 					break;
-				retval = server_get_named_group (server, group_name->str);
+
+				group_name = pstring_shallow (group_name_str, group_name_len);
+				retval = server_get_named_group (server, &group_name);
 				if (retval != NULL)
 					break;
 			}
-			g_string_free (group_name, TRUE);
 		}
 	}
 
@@ -1090,24 +1089,22 @@ get_charset_partfunc (GMimeObject * obj, gpointer charset_gpointer)
 {
 	GMimePart * part;
 	const GMimeContentType * type;
-	char ** charset;
+	const char ** charset;
 
 	if (!GMIME_IS_PART (obj))
 		return;
 
 	part = GMIME_PART (obj);
 	type = g_mime_object_get_content_type (GMIME_OBJECT (part));
-	charset = (char **) charset_gpointer;
-	if (g_mime_content_type_is_type (type, "text", "*")) {
-		*charset = (char *) g_mime_object_get_content_type_parameter (GMIME_OBJECT (part), "charset");
-		/*odebug1 ("found charset in article: %s", *charset);*/
-	}
+	charset = (const char **) charset_gpointer;
+	if (g_mime_content_type_is_type (type, "text", "*"))
+		*charset = g_mime_object_get_content_type_parameter (GMIME_OBJECT (part), "charset");
 }
 
 const char *
 pan_g_mime_message_get_charset (GMimeMessage * message)
 {
-	char * retval = NULL;
+	const char * retval = NULL;
 	g_return_val_if_fail (message!=NULL, NULL);
 
 	g_mime_message_foreach_part (message, get_charset_partfunc, &retval);
