@@ -41,13 +41,34 @@
 #include <pan/util.h>
 
 
-#define DEFAULT_FILTER_SHOW FILTER_SHOW_MATCHES_AND_REFERENCES
+#define DEFAULT_FILTER_SHOW FILTER_SHOW_MATCHES
 
 static FilterCurrentMediatorCtor _buttons;
-static gulong _filter_bits = ~STATE_FILTER_SCORE_LOW;
+static gulong _filter_bits = ~(gulong)STATE_FILTER_SCORE_IGNORED;
 static FilterShow _filter_show = DEFAULT_FILTER_SHOW;
 static void fire_user_changed_filter (void);
 
+static int _mute_feedback = 0;
+
+const static gulong NEW_MASK = STATE_FILTER_NEW | STATE_FILTER_UNREAD | STATE_FILTER_READ;
+const static gulong MINE_MASK = STATE_FILTER_MINE | STATE_FILTER_NOT_MINE;
+const static gulong BIN_MASK = STATE_FILTER_COMPLETE_BINARIES | STATE_FILTER_INCOMPLETE_BINARIES | STATE_FILTER_NONBINARIES;
+const static gulong CACHED_MASK = STATE_FILTER_CACHED | STATE_FILTER_NOT_CACHED;
+
+/**
+ * Holds the state of the new/unread/read menu buttons before the user
+ * clicks on the `only new' button in the toolbar, so that when they
+ * click it off we can set the menu buttons back to their old values.
+ */
+static gulong _menu_new_bits = 0ul;
+
+/**
+ * Holds the state of the score menu buttons before the user clicks on the
+ * `only watched' button in the toolbar, so that when they click it off we
+ * can set the menu buttons back to their old values.
+ */
+static gulong _menu_score_bits = 0ul;
+
 /**
 ***  REFRESH
 **/
@@ -61,185 +82,101 @@ refresh (void)
 	GtkCheckMenuItem * ckm;
 	GtkToggleButton * tb;
 	gboolean model;
-	gboolean view;
 
 	pan_lock ();
+	++_mute_feedback;
 
-	/**
-	***  New / Unread / Read
-	**/
-
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_new_ckm);
-	model = _filter_bits & STATE_FILTER_NEW ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
-
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_unread_ckm);
-	model = _filter_bits & STATE_FILTER_UNREAD ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
-
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_read_ckm);
-	model = _filter_bits & STATE_FILTER_READ ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
+	/* new, old, read */
 
+	model = (_filter_bits & NEW_MASK) == STATE_FILTER_NEW;
 	tb = GTK_TOGGLE_BUTTON(_buttons._match_only_new_tb);
-	model = (_filter_bits & STATE_FILTER_NEW)
-		&& !(_filter_bits & STATE_FILTER_UNREAD)
-		&& !(_filter_bits & STATE_FILTER_READ);
-	view = gtk_toggle_button_get_active (tb);
-	if (model != view)
-		gtk_toggle_button_set_active (tb, model);
+	gtk_toggle_button_set_active (tb, model);
 
-	/**
-	***  Binaries
-	**/
+	model = _filter_bits & STATE_FILTER_NEW;
+	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_only_new_ckm);
+	gtk_check_menu_item_set_active (ckm, model);
 
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_complete_ckm);
-	model = _filter_bits & STATE_FILTER_COMPLETE_BINARIES ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
+	model = _filter_bits & STATE_FILTER_UNREAD;
+	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_only_unread_ckm);
+	gtk_check_menu_item_set_active (ckm, model);
 
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_incomplete_ckm);
-	model = _filter_bits & STATE_FILTER_INCOMPLETE_BINARIES ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
+	model = _filter_bits & STATE_FILTER_READ;
+	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_only_read_ckm);
+	gtk_check_menu_item_set_active (ckm, model);
 
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_nonbinaries_ckm);
-	model = _filter_bits & STATE_FILTER_NONBINARIES ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
+	/* binaries */
 
+	model = (_filter_bits & BIN_MASK) == STATE_FILTER_COMPLETE_BINARIES;
+	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_only_complete_ckm);
+	gtk_check_menu_item_set_active (ckm, model);
 	tb = GTK_TOGGLE_BUTTON(_buttons._match_only_complete_tb);
-	model = (_filter_bits & STATE_FILTER_COMPLETE_BINARIES)
-		&& !(_filter_bits & STATE_FILTER_INCOMPLETE_BINARIES)
-		&& !(_filter_bits & STATE_FILTER_NONBINARIES);
-	view = gtk_toggle_button_get_active (tb);
-	if (model != view)
-		gtk_toggle_button_set_active (tb, model);
+	gtk_toggle_button_set_active (tb, model);
 
-	/**
-	***  Score
-	**/
+	/* text */
 
-	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_watched_ckm);
-	model = _filter_bits & STATE_FILTER_SCORE_WATCHED ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view )
-		gtk_check_menu_item_set_active (ckm, model);
+	model = (_filter_bits & BIN_MASK) == STATE_FILTER_NONBINARIES;
+	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_only_text_ckm);
+	gtk_check_menu_item_set_active (ckm, model);
 
-	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_high_ckm);
-	model = _filter_bits & STATE_FILTER_SCORE_HIGH ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
-
-	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_medium_ckm);
-	model = _filter_bits & STATE_FILTER_SCORE_MEDIUM ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
-
-	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_zero_ckm);
-	model = _filter_bits & STATE_FILTER_SCORE_ZERO ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
-
-	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_low_ckm);
-	model = _filter_bits & STATE_FILTER_SCORE_LOW ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view )
-		gtk_check_menu_item_set_active (ckm, model);
-
-	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_ignored_ckm);
-	model = _filter_bits & STATE_FILTER_SCORE_IGNORED ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view )
-		gtk_check_menu_item_set_active (ckm, model);
-
-	tb = GTK_TOGGLE_BUTTON(_buttons._match_only_watched_tb);
-	model = ((_filter_bits & STATE_FILTER_SCORE_MASK) == STATE_FILTER_SCORE_WATCHED);
-	view = gtk_toggle_button_get_active (tb);
-	if (model != view)
-		gtk_toggle_button_set_active (tb, model);
-
-	/**
-	***  Cached
-	**/
-
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_cached_ckm);
-	model = _filter_bits & STATE_FILTER_CACHED ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
-
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_noncached_ckm);
-	model = _filter_bits & STATE_FILTER_NOT_CACHED ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
+	/* cached */
 
+	model = (_filter_bits & CACHED_MASK) == STATE_FILTER_CACHED;
+	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_only_cached_ckm);
+	gtk_check_menu_item_set_active (ckm, model);
 	tb = GTK_TOGGLE_BUTTON(_buttons._match_only_cached_tb);
-	model = (_filter_bits & STATE_FILTER_CACHED)
-		&& !(_filter_bits & STATE_FILTER_NOT_CACHED);
-	view = gtk_toggle_button_get_active (tb);
-	if (model != view)
-		gtk_toggle_button_set_active (tb, model);
+	gtk_toggle_button_set_active (tb, model);
 
-	/**
-	***  Author
-	**/
+	/* author */
 
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_mine_ckm);
-	model = _filter_bits & STATE_FILTER_MINE ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
+	model = (_filter_bits & MINE_MASK) == STATE_FILTER_MINE;
+	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_only_mine_ckm);
+	gtk_check_menu_item_set_active (ckm, model);
+	tb = GTK_TOGGLE_BUTTON(_buttons._match_only_mine_tb);
+	gtk_toggle_button_set_active (tb, model);
 
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_others_ckm);
-	model = _filter_bits & STATE_FILTER_NOT_MINE ? 1 : 0;
-	view = ckm->active ? 1 : 0;
-	if (model != view)
-		gtk_check_menu_item_set_active (ckm, model);
+	/* score */
 
-	tb = GTK_TOGGLE_BUTTON(_buttons._show_only_my_threads_tb);
-	model = (_filter_bits & STATE_FILTER_MINE)
-		&& !(_filter_bits & STATE_FILTER_NOT_MINE)
-		&& (_filter_show == FILTER_SHOW_MATCHES_AND_REPLIES);
-	view = gtk_toggle_button_get_active (tb);
-	if (model != view)
-		gtk_toggle_button_set_active (tb, model);
+	model = (_filter_bits & STATE_FILTER_SCORE_MASK) == STATE_FILTER_SCORE_WATCHED;
+	tb = GTK_TOGGLE_BUTTON(_buttons._match_only_watched_tb);
+	gtk_toggle_button_set_active (tb, model);
 
-	/**
-	***  Show
-	**/
+	model = (_filter_bits & STATE_FILTER_SCORE_WATCHED);
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_only_watched_rmi);
+	gtk_check_menu_item_set_active (ckm, model);
 
-	switch (_filter_show) {
-		case FILTER_SHOW_MATCHES:
-			ckm = GTK_CHECK_MENU_ITEM(_buttons._show_articles_rmi);
-			break;
-		case FILTER_SHOW_THREAD:
-			ckm = GTK_CHECK_MENU_ITEM(_buttons._show_threads_rmi);
-			break;
-		case FILTER_SHOW_MATCHES_AND_REPLIES:
-			ckm = GTK_CHECK_MENU_ITEM(_buttons._show_articles_and_replies_rmi);
-			break;
-		case FILTER_SHOW_MATCHES_AND_REFERENCES:
-		default:
-			ckm = GTK_CHECK_MENU_ITEM(_buttons._show_articles_and_refs_rmi);
-			break;
-	}
+	model = (_filter_bits & STATE_FILTER_SCORE_HIGH);
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_only_high_rmi);
+	gtk_check_menu_item_set_active (ckm, model);
+
+	model = (_filter_bits & STATE_FILTER_SCORE_MEDIUM);
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_only_medium_rmi);
+	gtk_check_menu_item_set_active (ckm, model);
+
+	model = (_filter_bits & STATE_FILTER_SCORE_ZERO);
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_only_zero_rmi);
+	gtk_check_menu_item_set_active (ckm, model);
+
+	model = (_filter_bits & STATE_FILTER_SCORE_LOW);
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_only_low_rmi);
+	gtk_check_menu_item_set_active (ckm, model);
+
+	model = (_filter_bits & STATE_FILTER_SCORE_IGNORED);
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_only_ignored_rmi);
+	gtk_check_menu_item_set_active (ckm, model);
+
+
+	/* show */
+
+	if (_filter_show == FILTER_SHOW_THREAD)
+		ckm = GTK_CHECK_MENU_ITEM(_buttons._show_threads_rmi);
+	else if (_filter_show == FILTER_SHOW_SUBTHREADS)
+		ckm = GTK_CHECK_MENU_ITEM(_buttons._show_subthreads_rmi);
+	else
+		ckm = GTK_CHECK_MENU_ITEM(_buttons._show_articles_rmi);
 	if (!ckm->active)
 		gtk_check_menu_item_set_active (ckm, TRUE);
 
+	--_mute_feedback;
 	pan_unlock ();
 }
 
@@ -257,38 +194,36 @@ refresh_idle (gpointer unused)
  * Sets the current filter state and updates the menu to reflect the new state.
  */
 static void
-set (gulong value, FilterShow show)
+set (gulong bits, FilterShow show)
 {
-	static gboolean _mute_feedback = FALSE;
-
-	if (!_mute_feedback)
+	if (_mute_feedback == 0)
 	{
-		_mute_feedback = TRUE;
+		++_mute_feedback;
 
-		if (_filter_bits!=value || _filter_show!=show)
+		if (_filter_bits!=bits || _filter_show!=show)
 		{
 			_filter_show = show;
-			_filter_bits = value;
+			_filter_bits = bits;
 
 			gui_queue_add (refresh_idle, NULL);
 			fire_user_changed_filter ();
 		}
 
-		_mute_feedback = FALSE;
+		--_mute_feedback;
 	}
 }
 
 /**
- * Set the current filter's bit value (ie, what it matches)
+ * Set the current filter's bit bits (ie, what it matches)
  */
 static void
-set_value (gulong value)
+set_bits (gulong bits)
 {
-	set (value, _filter_show);
+	set (bits, _filter_show);
 }
 
 /**
- * Set the current filter's show value (ie, the FilterShow apply to the matched articles)
+ * Set the current filter's show bits (ie, the FilterShow apply to the matched articles)
  */
 static void
 set_show (FilterShow show)
@@ -302,99 +237,96 @@ set_show (FilterShow show)
 **/
 
 static void
-menu_button_toggled (GtkCheckMenuItem * item, gpointer user_data)
+set_mask_bits (gulong mask, gulong value)
 {
-	gulong value = _filter_bits;
-	gsize flag = GPOINTER_TO_SIZE (user_data);
+	gulong bits;
 
-	if (item->active)
-		value |= flag;
-	else
-		value &= ~flag;
+	g_assert (!(value & ~mask)); /* make sure everything in new_value is in mask */
 
-	set_value (value);
+	bits = _filter_bits;
+	bits &= ~mask; /* turn off everything in the mask */
+	bits |= value;
+
+	set_bits (bits);
 }
 
 static void
-show_only_new_cb (GtkToggleButton * tb, gpointer user_data)
-{
-	gulong value = _filter_bits;
-	const gboolean active = gtk_toggle_button_get_active (tb);
-
-	if (!active)
-		value |= STATE_FILTER_NEW | STATE_FILTER_UNREAD | STATE_FILTER_READ;
-	else {
-		value |= STATE_FILTER_NEW;
-		value &= ~STATE_FILTER_UNREAD;
-		value &= ~STATE_FILTER_READ;
-	}
-
-	set_value (value);
-}
-
-static void
-show_only_complete_cb (GtkToggleButton * tb, gpointer user_data)
-{
-	gulong value = _filter_bits;
-	const gboolean active = gtk_toggle_button_get_active (tb);
-
-	if (!active)
-		value |= STATE_FILTER_COMPLETE_BINARIES | STATE_FILTER_INCOMPLETE_BINARIES | STATE_FILTER_NONBINARIES;
-	else {
-		value |= STATE_FILTER_COMPLETE_BINARIES;
-		value &= ~STATE_FILTER_INCOMPLETE_BINARIES;
-		value &= ~STATE_FILTER_NONBINARIES;
-	}
-
-	set_value (value);
-}
-
-static void
-show_only_cached_cb (GtkToggleButton * tb, gpointer user_data)
-{
-	gulong value = _filter_bits;
-	const gboolean active = gtk_toggle_button_get_active (tb);
-
-	if (!active)
-		value |= STATE_FILTER_CACHED | STATE_FILTER_NOT_CACHED;
-	else
-		value &= ~STATE_FILTER_NOT_CACHED;
-
-	set_value (value);
-}
-
-static void
-show_only_watched_cb (GtkToggleButton * tb, gpointer user_data)
+show_only_new_tb_cb (GtkToggleButton * tb, gpointer user_data)
 {
 	const gboolean active = gtk_toggle_button_get_active (tb);
-	gulong value = _filter_bits & ~STATE_FILTER_SCORE_MASK;
 
 	if (active)
-		value |= STATE_FILTER_SCORE_WATCHED;
-	else
-		value |= (STATE_FILTER_SCORE_MASK & ~STATE_FILTER_SCORE_IGNORED);
+		_menu_new_bits = _filter_bits & NEW_MASK;
 
-	set_value (value);
+	set_mask_bits (NEW_MASK, active ? STATE_FILTER_NEW : _menu_new_bits);
 }
 
 static void
-show_only_mine_cb (GtkToggleButton * tb, gpointer user_data)
+show_only_complete_ckm_cb (GtkCheckMenuItem * item, gpointer user_data)
+{
+	const gboolean active = item->active;
+	set_mask_bits (BIN_MASK, active ? STATE_FILTER_COMPLETE_BINARIES  : BIN_MASK);
+}
+
+static void
+show_only_complete_tb_cb (GtkToggleButton * tb, gpointer user_data)
+{
+	const gboolean active = gtk_toggle_button_get_active (tb);
+	set_mask_bits (BIN_MASK, active ? STATE_FILTER_COMPLETE_BINARIES  : BIN_MASK);
+}
+
+static void
+show_only_text_ckm_cb (GtkCheckMenuItem * item, gpointer user_data)
+{
+	const gboolean active = item->active;
+	set_mask_bits (BIN_MASK, active ? STATE_FILTER_NONBINARIES  : BIN_MASK);
+}
+
+static void
+show_only_cached_ckm_cb (GtkCheckMenuItem * item, gpointer user_data)
+{
+	const gboolean active = item->active;
+	set_mask_bits (CACHED_MASK, active ? STATE_FILTER_CACHED  : CACHED_MASK);
+}
+
+static void
+show_only_cached_tb_cb (GtkToggleButton * tb, gpointer user_data)
+{
+	const gboolean active = gtk_toggle_button_get_active (tb);
+	set_mask_bits (CACHED_MASK, active ? STATE_FILTER_CACHED  : CACHED_MASK);
+}
+
+static void
+show_only_mine_ckm_cb (GtkCheckMenuItem * item, gpointer user_data)
+{
+	const gboolean active = item->active;
+	set_mask_bits (MINE_MASK, active ? STATE_FILTER_MINE  : MINE_MASK);
+}
+
+static void
+show_only_mine_tb_cb (GtkToggleButton * tb, gpointer user_data)
+{
+	const gboolean active = gtk_toggle_button_get_active (tb);
+	set_mask_bits (MINE_MASK, active ? STATE_FILTER_MINE  : MINE_MASK);
+}
+
+static void
+match_score_is_watched_tb_cb (GtkToggleButton * tb, gpointer unused)
 {
-	gulong value = _filter_bits;
-	FilterShow show = _filter_show;
 	const gboolean active = gtk_toggle_button_get_active (tb);
 
-	if (!active) {
-		value |= STATE_FILTER_MINE | STATE_FILTER_NOT_MINE;
-		show = DEFAULT_FILTER_SHOW;
-	}
-	else {
-		value |= STATE_FILTER_MINE;
-		value &= ~STATE_FILTER_NOT_MINE;
-		show = FILTER_SHOW_MATCHES_AND_REPLIES;
-	}
+	if (active)
+		_menu_score_bits = _filter_bits & STATE_FILTER_SCORE_MASK;
 
-	set (value, show);
+	set_mask_bits (STATE_FILTER_SCORE_MASK, active ? STATE_FILTER_SCORE_WATCHED : _menu_score_bits);
+}
+
+static void
+rmi_toggled_cb (GtkCheckMenuItem * item, gpointer bit_gpointer)
+{
+	const gboolean active = item->active;
+	const gulong bit = GPOINTER_TO_UINT(bit_gpointer);
+	set_mask_bits (bit, active ? bit : 0);
 }
 
 static void
@@ -445,6 +377,8 @@ rebuild_custom_filter_menu_with_filters (GPtrArray * filters)
 	/* build the new filter buttons */
 	if (filters->len)
 	{
+		int j;
+
 		_custom_filter_entries = g_new0 (GtkItemFactoryEntry, filters->len+1);
 
 		/* add the separator */
@@ -456,36 +390,41 @@ rebuild_custom_filter_menu_with_filters (GPtrArray * filters)
 		for (i=0; i<filters->len; ++i)
 		{
 			const FilterTop * top = FILTER_TOP(g_ptr_array_index(filters,i));
-			char * underscore_escaped_path = pan_substitute (top->name, "_", "__");
+			if (top->is_visible)
+			{
+				char * underscore_escaped_path = pan_substitute (top->name, "_", "__");
 
-			/* build an entry */
-			_custom_filter_entries[_custom_filter_entry_qty].path = g_strdup_printf ("/Filter/%s", underscore_escaped_path);
-			_custom_filter_entries[_custom_filter_entry_qty].item_type = "<ToggleItem>";
-			_custom_filter_entries[_custom_filter_entry_qty++].callback = custom_filter_selected_cb;
+				/* build an entry */
+				_custom_filter_entries[_custom_filter_entry_qty].path = g_strdup_printf ("/Filter/%s", underscore_escaped_path);
+				_custom_filter_entries[_custom_filter_entry_qty].item_type = "<ToggleItem>";
+				_custom_filter_entries[_custom_filter_entry_qty++].callback = custom_filter_selected_cb;
 
-			g_free (underscore_escaped_path);
+				g_free (underscore_escaped_path);
+			}
 		}
 
 		/* add the filters to the menu */
 		gtk_item_factory_create_items (_main_menu_factory, _custom_filter_entry_qty, _custom_filter_entries, NULL);
 
 		/* add the filter name as data item in the menu buttons.
-		 * This is used by custom_filter_selected_Cb() callback to
-		 * get a FilterTop* from the GtkMenuToggleButton */
-		for (i=0; i<filters->len; ++i)
+		 * This is used by custom_filter_selected_cb() callback to
+		 * get a FilterTop* from the GtkMenuToggleButton.
+		 * j starts at 1 because _custom_filter_entries[0] is
+		 * the menu separator  */
+		for (i=0, j=1; i<filters->len; ++i)
 		{
-			char * unescaped_path;
-			const char * top_name;
-			GtkWidget * w;
+			const FilterTop * top = FILTER_TOP(g_ptr_array_index(filters,i));
 
-		       	top_name = FILTER_TOP(g_ptr_array_index(filters,i))->name;
-			unescaped_path = g_strdup_printf ("/Filter/%s", top_name);
+			if (top->is_visible)
+			{
+				GtkWidget * w;
+				char path[1024];
 
-			w = gtk_item_factory_get_widget (_main_menu_factory, unescaped_path);
-			g_object_set_data_full (G_OBJECT(w), "filter_name", g_strdup(top_name), g_free);
-			_custom_filter_entries[i+1].extra_data = w;
-
-			g_free (unescaped_path);
+				g_snprintf (path, sizeof(path), "/Filter/%s", top->name);
+				w = gtk_item_factory_get_widget (_main_menu_factory, path);
+				g_object_set_data_full (G_OBJECT(w), "filter_name", g_strdup(top->name), g_free);
+				_custom_filter_entries[j++].extra_data = w;
+			}
 		}
 	}
 
@@ -532,69 +471,43 @@ filter_mediator_init (FilterCurrentMediatorCtor * ctor)
 
 	/* New / Read / Unread */
 
-	g_signal_connect (G_OBJECT(_buttons._match_new_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_NEW));
-	g_signal_connect (G_OBJECT(_buttons._match_unread_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_UNREAD));
-	g_signal_connect (G_OBJECT(_buttons._match_read_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_READ));
-	g_signal_connect (G_OBJECT(_buttons._match_only_new_tb), "toggled",
-	                  G_CALLBACK(show_only_new_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_new_ckm),    "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_NEW));
+	g_signal_connect (G_OBJECT(_buttons._match_only_unread_ckm), "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_UNREAD));
+	g_signal_connect (G_OBJECT(_buttons._match_only_read_ckm),   "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_READ));
+	g_signal_connect (G_OBJECT(_buttons._match_only_new_tb),     "toggled", G_CALLBACK(show_only_new_tb_cb), NULL);
 
 	/* Text / Complete / Incomplete */
 
-	g_signal_connect (G_OBJECT(_buttons._match_complete_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_COMPLETE_BINARIES));
-	g_signal_connect (G_OBJECT(_buttons._match_incomplete_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_INCOMPLETE_BINARIES));
-	g_signal_connect (G_OBJECT(_buttons._match_nonbinaries_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_NONBINARIES));
-	g_signal_connect (G_OBJECT(_buttons._match_only_complete_tb), "toggled",
-	                  G_CALLBACK(show_only_complete_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_complete_ckm), "toggled", G_CALLBACK(show_only_complete_ckm_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_text_ckm),     "toggled", G_CALLBACK(show_only_text_ckm_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_complete_tb),  "toggled", G_CALLBACK(show_only_complete_tb_cb), NULL);
 
 	/* Cached */
 
-	g_signal_connect (G_OBJECT(_buttons._match_cached_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_CACHED));
-	g_signal_connect (G_OBJECT(_buttons._match_noncached_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_NOT_CACHED));
-	g_signal_connect (G_OBJECT(_buttons._match_only_cached_tb), "toggled",
-	                  G_CALLBACK(show_only_cached_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_cached_ckm), "toggled", G_CALLBACK(show_only_cached_ckm_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_cached_tb),  "toggled", G_CALLBACK(show_only_cached_tb_cb), NULL);
 
 	/* Author */
 
-	g_signal_connect (G_OBJECT(_buttons._match_mine_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_MINE));
-	g_signal_connect (G_OBJECT(_buttons._match_others_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_NOT_MINE));
-	g_signal_connect (G_OBJECT(_buttons._show_only_my_threads_tb), "toggled",
-	                  G_CALLBACK(show_only_mine_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_mine_ckm), "toggled", G_CALLBACK(show_only_mine_ckm_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_mine_tb),  "toggled", G_CALLBACK(show_only_mine_tb_cb),  NULL);
 
 	/* Score */
 
-	g_signal_connect (G_OBJECT(_buttons._match_score_watched_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_WATCHED));
-	g_signal_connect (G_OBJECT(_buttons._match_score_high_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_HIGH));
-	g_signal_connect (G_OBJECT(_buttons._match_score_medium_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_MEDIUM));
-	g_signal_connect (G_OBJECT(_buttons._match_score_zero_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_ZERO));
-	g_signal_connect (G_OBJECT(_buttons._match_score_low_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_LOW));
-	g_signal_connect (G_OBJECT(_buttons._match_score_ignored_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_IGNORED));
-	g_signal_connect (G_OBJECT(_buttons._match_only_watched_tb), "toggled",
-	                  G_CALLBACK(show_only_watched_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_watched_tb),  "toggled", G_CALLBACK(match_score_is_watched_tb_cb), NULL);
+	g_signal_connect (G_OBJECT(_buttons._match_only_watched_rmi), "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_SCORE_WATCHED));
+	g_signal_connect (G_OBJECT(_buttons._match_only_high_rmi),    "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_SCORE_HIGH));
+	g_signal_connect (G_OBJECT(_buttons._match_only_medium_rmi),  "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_SCORE_MEDIUM));
+	g_signal_connect (G_OBJECT(_buttons._match_only_zero_rmi),    "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_SCORE_ZERO));
+	g_signal_connect (G_OBJECT(_buttons._match_only_low_rmi),     "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_SCORE_LOW));
+	g_signal_connect (G_OBJECT(_buttons._match_only_ignored_rmi), "toggled", G_CALLBACK(rmi_toggled_cb), GUINT_TO_POINTER(STATE_FILTER_SCORE_IGNORED));
 
 	/* show */
 
 	g_signal_connect (G_OBJECT(_buttons._show_articles_rmi), "toggled",
 	                  G_CALLBACK(show_toggled_cb), GSIZE_TO_POINTER(FILTER_SHOW_MATCHES));
-	g_signal_connect (G_OBJECT(_buttons._show_articles_and_replies_rmi), "toggled",
-	                  G_CALLBACK(show_toggled_cb), GSIZE_TO_POINTER(FILTER_SHOW_MATCHES_AND_REPLIES));
-	g_signal_connect (G_OBJECT(_buttons._show_articles_and_refs_rmi), "toggled",
-	                  G_CALLBACK(show_toggled_cb), GSIZE_TO_POINTER(FILTER_SHOW_MATCHES_AND_REFERENCES));
+	g_signal_connect (G_OBJECT(_buttons._show_subthreads_rmi), "toggled",
+	                  G_CALLBACK(show_toggled_cb), GSIZE_TO_POINTER(FILTER_SHOW_SUBTHREADS));
 	g_signal_connect (G_OBJECT(_buttons._show_threads_rmi), "toggled",
 	                  G_CALLBACK(show_toggled_cb), GSIZE_TO_POINTER(FILTER_SHOW_THREAD));
 
@@ -632,6 +545,9 @@ void
 filter_mediator_set_bits (gulong        bits,
                           FilterShow    show)
 {
+	_menu_new_bits = bits & NEW_MASK;
+	_menu_score_bits = bits & STATE_FILTER_SCORE_MASK;
+
 	set (bits, show);
 }
 
