@@ -107,4 +107,4 @@ gboolean     filter_aggregate_has_child       (FilterAggregate     * f,
 **/
 
 
-#endif /* __FILTER_H__ */
+#endif /* __FILTER_AGGREGATE_H__ */
