@@ -59,6 +59,4 @@ void  filter_manager_set_filters    (GPtrArray * filters);
  */
 PanCallback*   filter_manager_get_filters_changed_callback (void);
 
-extern const char * bozo_filter_name;
-extern const char * spam_filter_name;
 #endif
