@@ -50,6 +50,13 @@ typedef enum
 }
 PhraseMatchType;
 
+enum
+{
+	REGEX_NEED_COMPILE,
+	REGEX_ERR,
+	REGEX_COMPILED
+};
+
 struct _FilterPhrase
 {
 	/* Parent */
@@ -66,7 +73,7 @@ struct _FilterPhrase
 	char bmhs_skip[UCHAR_MAX];
 
 	regex_t key_regex;
-	gboolean regex_compiled;
+	int regex_state;
 	gboolean case_sensitive;
 };
 
@@ -74,15 +81,17 @@ struct _FilterPhrase
 ****  PUBLIC
 ***/
 
-Filter*      filter_phrase_new            (void);
+Filter*      filter_phrase_new               (void);
 
-void         filter_phrase_set            (FilterPhrase         * f,
-                                           PhraseMatchType        match_type,
-                                           PhraseKeyType          key_type,
-                                           const char           * key,
-                                           gboolean               case_sensitive);
+void         filter_phrase_set              (FilterPhrase         * f,
+                                             PhraseMatchType        match_type,
+                                             PhraseKeyType          key_type,
+                                             const char           * key,
+                                             gboolean               case_sensitive);
 
-char*        filter_phrase_create_regex   (const char           * string,
-                                           PhraseMatchType        match_type);
+char*        filter_phrase_create_regex     (const char           * string,
+                                             PhraseMatchType        match_type);
+
+char*        filter_phrase_validate_regex   (const char           * regex);
 
 #endif
