@@ -46,7 +46,11 @@ filter_score_get_score_mode (int score)
 static Filter*
 filter_score_dup (const Filter * f_old)
 {
-	return filter_score_new (FILTER_SCORE(f_old)->mode);
+	Filter *f_new = filter_score_new (FILTER_SCORE (f_old)->mode);
+	if (f_old->negate)
+		filter_negate (f_new);
+
+	return f_new;
 }
 
 static char*
@@ -101,7 +105,7 @@ filter_score_test_articles (Filter          * filter,
 	g_return_if_fail (articles_are_valid (articles, article_qty));
 
 	/* FIXME: casting away const */
-	ensure_articles_scored (articles[0]->group, (Article**)articles, article_qty);
+	ensure_articles_scored (articles[0]->group, (Article**)articles, article_qty, NULL);
 
 	for (i=0; i<article_qty; ++i)
 		passfail[i] = (mode & filter_score_get_score_mode (articles[i]->score)) != 0;
