@@ -28,8 +28,6 @@
 #include <string.h>
 #include <math.h>
 
-#include <regex.h>
-
 #include <glib.h>
 
 #include <pan/base/debug.h>
@@ -49,6 +47,9 @@
 
 #include <pan/prefs.h>
 
+#include <sys/types.h>
+#include <regex.h>
+
 /*********************
 **********************  Defines / Enumerated types
 *********************/
@@ -83,6 +84,7 @@ typedef struct
 
 	int value;
 	gboolean value_assign_flag;
+	gboolean stop_scoring;
 }
 Score;
 
@@ -184,7 +186,7 @@ score_get_main_scorefile_filename (void)
 	if (filename == NULL)
 	{
 		/* get the filename */
-		filename = pan_config_get_string (KEY_SCOREFILE_DEFAULT);
+		filename = pan_config_get_string (KEY_SCOREFILE, KEY_SCOREFILE_DEFAULT);
 		replace_gstr (&filename, pan_file_normalize (filename, NULL));
 	}
 
@@ -242,8 +244,27 @@ has_score_expired (unsigned const char *s, unsigned long today)
 *****
 ****/
 
+static void
+prep_section_regex (const PString * in, GString * out)
+{
+	const char * pch;
+	const char * end;
+
+	g_string_set_size (out, 0);
+
+	for (pch=in->str, end=pch+in->len; pch!=end; ++pch)
+	{
+		if ((*pch=='.' || *pch=='+') && (pch==in->str || pch[-1]!='\\'))
+			g_string_append_c (out, '\\');
+		else if (*pch=='*' && (pch==in->str || pch[-1]!='\\'))
+			g_string_append_c (out, '.');
+
+		g_string_append_c (out, *pch);
+	}
+}
+
 static Section*
-get_section (const char * name)
+get_section (const char * name, const char * filename, int line_number)
 {
 	GSList * l;
 	Section * retval = NULL;
@@ -260,8 +281,10 @@ get_section (const char * name)
 	/* if no such section, make one. */
 	if (retval == NULL)
 	{
-		int i;
-		char ** names;
+		GString * gname;
+		GString * prep;
+		PString token = PSTRING_INIT;
+		const char * march;
 		const gboolean negate = *name == '~';
 
 		/* make a new section */
@@ -275,22 +298,53 @@ get_section (const char * name)
 		/* break the groups apart by whitespace and compile each one as a regular expression */
 		if (negate)
 			++name;
-		names = g_strsplit (name, " ", -1);
-		for (i=0; names!=NULL && names[i]!=NULL; ++i)
-		       	/* just counting the qty */;
-		retval->regex_qty = i;
-		if (retval->regex_qty > 0) {
-			retval->regex = g_new0 (regex_t, retval->regex_qty);
-			for (i=0; i<retval->regex_qty; ++i) {
-				if (!pan_strcmp (names[i], "*"))
-					replace_gstr(&names[i], g_strdup(".*"));
-				regcomp (retval->regex+i, names[i], REG_EXTENDED|REG_ICASE);
+
+		/* normalize the section name's group delimiters and strip extra spaces */
+		gname = g_string_new (name);
+		pan_g_string_replace (gname, ",", " "); /* treat commas as spaces */
+		pan_g_string_replace (gname, "\t", " "); /* treat tabs as spaces */
+		while (strstr (gname->str, "  ")) /* strip out extra spaces */
+			pan_g_string_replace (gname, "  ", " ");
+
+		/* count the number of names */
+		march = gname->str;
+		while ((get_next_token_pstring (march, ' ', &march, &token)))
+			++retval->regex_qty;
+
+		/* compile the names into regex_t's */
+		prep = g_string_new (NULL);
+		if (retval->regex_qty)
+		{
+			int real_qty = 0;
+			retval->regex = g_new (regex_t, retval->regex_qty);
+			march = gname->str;
+
+			while ((get_next_token_pstring (march, ' ', &march, &token)))
+			{
+				int errcode;
+
+				prep_section_regex (&token, prep);
+
+				errcode = regcomp (retval->regex + real_qty, prep->str, REG_EXTENDED|REG_ICASE);
+				if (!errcode)
+					++real_qty;
+				else {
+					char errbuf[1024];
+					regerror (errcode, retval->regex + real_qty, errbuf, sizeof(errbuf));
+					log_add_va (LOG_ERROR, _("Can't use regular expression \"%s\" from file \"%s\", line %d: %s"),
+					                       prep->str, filename, line_number, errbuf);
+				}
 			}
+
+			retval->regex_qty = real_qty;
 		}
-		g_strfreev (names);
 
 		/* remember this section */
 		_sections = g_slist_append (_sections, retval);
+
+		/* cleanup */
+		g_string_free (prep, TRUE);
+		g_string_free (gname, TRUE);
 	}
 
 	return retval;
@@ -358,7 +412,7 @@ parse_score_file (ScorefileContext   * context,
 			g_strstrip (name);
 
 			/* get the section */
-			context->current_section = get_section (name);
+			context->current_section = get_section (name, filename, line_number);
 			context->current_score = NULL;
 			g_slist_free (context->aggregates);
 			context->aggregates = NULL;
@@ -412,6 +466,7 @@ parse_score_file (ScorefileContext   * context,
 			score->begin_line = score->end_line = line_number;
 			score->filter = NULL;
 			score->value_assign_flag = value_assign_flag;
+			score->stop_scoring = value_assign_flag;
 			score->value = value;
 			score->filter = filter_aggregate_new ();
 			filter_aggregate_set_type (FILTER_AGGREGATE(score->filter), aggregate_type);
@@ -774,6 +829,20 @@ unload_scores (void)
 	debug_exit ("unload_scores");
 }
 
+PanCallback*
+score_get_scorefile_invalidated_callback (void)
+{
+	static PanCallback * cb = NULL;
+	if (cb==NULL) cb = pan_callback_new ();
+	return cb;
+}
+
+static void
+fire_scorefile_invalidated (gboolean do_rescore_all)
+{
+	pan_callback_call (score_get_scorefile_invalidated_callback(), NULL, GINT_TO_POINTER(do_rescore_all));
+}
+
 static void
 score_invalidate_impl (void)
 {
@@ -782,7 +851,7 @@ score_invalidate_impl (void)
 	scorefile_time = 0;
 }
 void
-score_invalidate (void)
+score_invalidate (gboolean do_rescore_all)
 {
 	debug_enter ("score_invalidate");
 
@@ -792,6 +861,8 @@ score_invalidate (void)
 	}
 	g_static_mutex_unlock (&_score_mutex);
 
+	fire_scorefile_invalidated (do_rescore_all);
+
 	debug_exit ("score_invalidate");
 }
 
@@ -853,6 +924,12 @@ slrn_adds_these_even_if_turned_off (const char * key)
 	    || !g_ascii_strcasecmp (key, "Xref");
 }
 
+#ifdef G_OS_WIN32
+#define EOL "\r\n"
+#else
+#define EOL "\n"
+#endif
+
 static GString*
 make_scorefile_entry (const char           * score_name,
                       const char           * section_name,
@@ -867,10 +944,9 @@ make_scorefile_entry (const char           * score_name,
 	const time_t now = time (NULL);
 	GString * str = g_string_sized_new (1024);
 	debug_enter ("make_scorefile_entry");
-
-	g_string_append_printf (str, "\n%%BOS\n");
-	g_string_append_printf (str, "%%Score created by Pan on %s\n", ctime(&now));
-	g_string_append_printf (str, "[%s]\n",
+	g_string_append_printf (str, EOL "%%BOS" EOL);
+	g_string_append_printf (str, "%%Score created by Pan on %s" EOL, ctime(&now));
+	g_string_append_printf (str, "[%s]" EOL,
 		(is_nonempty_string(section_name) ? section_name : "*"));
 	g_string_append_printf (str, "Score%s %s%d",
 		(item_type==AGGREGATE_TYPE_OR ? "::" : ":"),
@@ -878,9 +954,9 @@ make_scorefile_entry (const char           * score_name,
 		value);
 	if (is_nonempty_string (score_name))
 		g_string_append_printf (str, " %% %s", score_name);
-	g_string_append_c (str, '\n');
+	g_string_append (str, EOL);
 	if (lifespan_days <= 0)
-		g_string_append_printf (str, "%%Expires: \n");
+		g_string_append_printf (str, "%%Expires: " EOL);
 	else {
 		int dd=0, mm=0, yyyy=0;
 		time_t expire_time_t = now + lifespan_days * 24 * 3600;
@@ -889,18 +965,18 @@ make_scorefile_entry (const char           * score_name,
 		dd = expire_tm.tm_mday;
 		mm = expire_tm.tm_mon + 1;
 		yyyy = expire_tm.tm_year + 1900;
-		g_string_append_printf (str, "Expires: %u/%u/%u\n", mm, dd, yyyy);
+		g_string_append_printf (str, "Expires: %u/%u/%u" EOL, mm, dd, yyyy);
 	}
 	for (i=0; i<item_qty; ++i) {
 		const ScoreAddItem * item = items + i;
 		const char * value = item->value ? item->value : "";
 		if (is_nonempty_string(item->value) && (item->on || slrn_adds_these_even_if_turned_off(item->key)))
-			g_string_append_printf (str, "%c\t%s%s: %s\n",
+			g_string_append_printf (str, "%c\t%s%s: %s" EOL,
 						(item->on?' ':'%'),
 						(item->negate?"~":""),
 						item->key, value);
 	}
-	g_string_append_printf (str, "%%EOS\n\n");
+	g_string_append_printf (str, "%%EOS" EOL EOL);
 
 	debug_exit ("make_scorefile_entry");
 	return str;
@@ -914,7 +990,8 @@ score_add (const char           * score_name,
            const int              lifespan_days,
            FilterAggregateType    item_type,
            const ScoreAddItem   * items,
-           const int              item_qty)
+           const int              item_qty,
+           const gboolean         do_rescore_all)
 {
 	int i;
 	debug_enter ("score_add");
@@ -950,6 +1027,8 @@ score_add (const char           * score_name,
 	}
 	g_static_mutex_unlock (&_score_mutex);
 
+	fire_scorefile_invalidated (do_rescore_all);
+
 	debug_exit ("score_add");
 }
 
@@ -977,18 +1056,21 @@ score_article_matchfunc (const Section  * section,
 }
 
 static gboolean
-does_section_match (const Section * section, const char * group_name)
+does_section_match (const Section * section, const PString * group_name)
 {
 	int i;
 	gboolean match = FALSE;
+	char buf[1024];
 
 	/* sanity clause */
 	g_return_val_if_fail (section!=NULL, FALSE);
-	g_return_val_if_fail (is_nonempty_string(group_name), FALSE);
+	g_return_val_if_fail (pstring_is_set (group_name), FALSE);
+
+	pan_strncpy_len (buf, sizeof(buf), group_name->str, group_name->len);
 
 	/* does it match? */
 	for (i=0; !match && i<section->regex_qty; ++i) {
-		match = !regexec (&section->regex[i], group_name, 0, NULL, 0);
+		match = !regexec (&section->regex[i], buf, 0, NULL, 0);
 		if (section->section_name_negate)
 			match = !match;
 	}
@@ -998,52 +1080,75 @@ does_section_match (const Section * section, const char * group_name)
 
 static void
 score_articles (Group              * group,
-                Article           ** articles,
-                int                  article_qty,
+                Article           ** articles_in,
+                int                  articles_in_qty,
                 ScoreMatchesFunc     match_func,
                 gpointer             match_func_user_data)
 {
 	GSList * l;
-	gboolean * match = g_new (gboolean, article_qty);
+	GPtrArray * articles;
+	GPtrArray * keep_scoring;
+	gboolean * match;
 	debug_enter ("score_articles");
 
+ 	articles = g_ptr_array_sized_new (articles_in_qty);
+	keep_scoring = g_ptr_array_sized_new (articles_in_qty);
+	match = g_new (gboolean, articles_in_qty);
+
+	pan_g_ptr_array_assign (articles, (gpointer*)articles_in, articles_in_qty);
+
 	/* walk through all the sections to see if any apply to this group */
 	for (l=_sections; l!=NULL; l=l->next)
 	{
 		GSList * l2;
 		const Section * section = (const Section*) l->data;
 
-		if (!does_section_match (section, group->name))
+		if (!does_section_match (section, &group->name))
 			continue;
 
+		/* test the articles against each score in the section */
 		for (l2=section->scores; l2!=NULL; l2=l2->next)
 		{
-			int i;
-
-			/* test the articles against each score in the section */
+			guint i;
 			const Score * score = (const Score*) l2->data;
 
-			if (score->expired) continue;
+			if (score->expired)
+				continue;
 
-			memset (match, 0, sizeof(gboolean) * article_qty);
-			filter_test_articles (score->filter, (const Article**)articles, article_qty, match);
+			/* prep the arrays */
+			g_ptr_array_set_size (keep_scoring, 0);
+			memset (match, 0, sizeof(gboolean) * articles->len);
 
-			/* if they passed, call the match func */
-			for (i=0; i<article_qty; ++i)
+			/* test the articles */
+			filter_test_articles (score->filter, (const Article**)articles->pdata, articles->len, match);
+
+			/* if they passed the test, call the match func. 
+			   if they didn't pass, or the entry doesn't say `stop scoring', Article continues to the next round. */
+			for (i=0u; i!=articles->len; ++i) {
+				Article * article = ARTICLE (g_ptr_array_index (articles, i));
 				if (match[i])
-					(*match_func)(section, score, articles[i], match_func_user_data);
+					(*match_func)(section, score, article, match_func_user_data);
+				if (!match[i] || !score->stop_scoring)
+					g_ptr_array_add (keep_scoring, article);
+			}
+
+			/* everything that didn't hit 'stop scoring' makes it through to the next round */
+			pan_g_ptr_array_assign (articles, keep_scoring->pdata, keep_scoring->len);
 		}
 	}
 
 	/* cleanup */
 	g_free (match);
+	g_ptr_array_free (articles, TRUE);
+	g_ptr_array_free (keep_scoring, TRUE);
 	debug_exit ("score_articles");
 }
 
 void 
 ensure_articles_scored  (Group          * group,
                          Article       ** articles_in,
-                         int              article_in_qty)
+                         int              article_in_qty,
+                         GPtrArray      * optional_changed)
 {
 	debug_enter ("ensure_articles_scored");
 
@@ -1099,8 +1204,11 @@ ensure_articles_scored  (Group          * group,
 				for (i=0; i<article_qty; ++i)
 					if (old_score[i] != articles[i]->score)
 						g_ptr_array_add (score_changed, articles[i]);
-				if (score_changed->len)
+				if (score_changed->len) {
 					articles_set_dirty ((Article**)score_changed->pdata, score_changed->len);
+					if (optional_changed)
+						pan_g_ptr_array_append (optional_changed, score_changed->pdata, score_changed->len);
+				}
 
 				/* cleanup */
 				g_free (old_score);
@@ -1219,7 +1327,8 @@ score_view_article (const Article * a)
 void
 score_remove_score  (const char * filename,
                      const int    begin_line,
-                     const int    end_line)
+                     const int    end_line,
+                     gboolean     do_rescore_all)
 {
 	/* sanity clause */
 	g_return_if_fail (is_nonempty_string (filename));
@@ -1325,4 +1434,40 @@ score_remove_score  (const char * filename,
 		g_free (filename_new);
 	}
 	g_static_mutex_unlock (&_score_mutex);
+
+	fire_scorefile_invalidated (do_rescore_all);
+}
+
+/***
+****
+***/
+
+static void
+add_group_name_to_section (const PString * server_name,
+                           const PString * group_name,
+                           gulong number,
+                           gpointer gstring_gpointer)
+{
+	GString * gstr = (GString*) gstring_gpointer;
+
+	g_string_append_len (gstr, group_name->str, group_name->len);
+	g_string_append (gstr, ", ");
+}
+char*
+score_create_section_str (const Article * article)
+{
+	GString * str;
+
+	/* sanity check */
+	g_return_val_if_fail (article_is_valid (article), g_strdup("[*]"));
+
+	/* build the section string */
+	str = g_string_new (NULL);
+	if (pstring_is_set (&article->xref)) /* add all xrefs */
+		xref_header_foreach (article->xref.str, &article->group->server->name, NULL, add_group_name_to_section, str);
+	else /* just use the current group */
+		add_group_name_to_section (&article->group->server->name, &article->group->name, article->number, str);
+	g_string_truncate (str, str->len-2);
+
+	return g_string_free (str, FALSE);
 }
