@@ -313,7 +313,7 @@ grouplist_rebuild_nolock (GPtrArray * groups, GPtrArray * sel)
 		pattern = g_pattern_spec_new (groupname_filter_string);
 	for (i=groups->len-1; i>=0; --i) {
 		Group * group = GROUP (g_ptr_array_index (groups, i));
-		if (!do_filter || g_pattern_match (pattern, strlen(group->name), group->name, NULL))
+		if (!do_filter || g_pattern_match (pattern, group->name.len, group->name.str, NULL))
 			grouplist_add_group_nolock (clist, group);
 	}
 	if (do_filter)
@@ -362,7 +362,7 @@ grouplist_update_ui_mainthread (gpointer unused)
 	}
 	else if (group_mode == GROUP_FOLDERS)
 	{
-		Server * s = serverlist_get_named_server(INTERNAL_SERVER_NAME);
+		Server * s = serverlist_get_named_server(&INTERNAL_SERVER_NAME);
 		groups = s==NULL
 			? g_ptr_array_new()
 			: server_get_groups (s, SERVER_GROUPS_ALL);
@@ -419,7 +419,7 @@ static Server *
 grouplist_get_visible_server (void)
 {
 	return group_mode==GROUP_FOLDERS
-		? serverlist_get_named_server(INTERNAL_SERVER_NAME)
+		? serverlist_get_named_server(&INTERNAL_SERVER_NAME)
 		: my_server;
 }
 
@@ -644,7 +644,7 @@ grouplist_add_subscribed_to_selection_nolock (void)
 ***/
 
 static void
-grouplist_update_group_nolock (GtkCList * clist, const Group * group)
+grouplist_update_group_nolock (GtkCList * clist, const Group * group, guint change_type)
 {
 	int row;
 
@@ -654,31 +654,29 @@ grouplist_update_group_nolock (GtkCList * clist, const Group * group)
        	row = gtk_clist_find_row_from_data (clist, (gpointer)group);
 	if (row != -1)
 	{
-		char buf[256] = { '\0' };
-		const char * utf8_name = NULL;
-		char * freeme;
-		const int total = group->article_qty;
-		const int unread = MAX(0, total-group->article_read_qty);
+		if (change_type & GROUP_CHANGED_SUBSCRIPTION_STATE)
+		{
+			if (group_is_subscribed (group))
+				gtk_clist_set_pixmap (clist, row, 0, sub_pixmap, sub_bitmap);
+			else if (!group_is_folder(group))
+				gtk_clist_set_text (clist, row, 0, "");
+		}
 
-		/* icon column */
-		if (group_is_subscribed (group))
-			gtk_clist_set_pixmap (clist, row, 0, sub_pixmap, sub_bitmap);
-		else if (!group_is_folder(group))
-			gtk_clist_set_text (clist, row, 0, "");
+		if (change_type & GROUP_CHANGED_ARTICLE_QTY)
+		{
+			char buf[256] = { '\0' };
+			g_snprintf (buf, sizeof(buf), "%d", group->article_qty);
+			gtk_clist_set_text (clist, row, 3, buf);
+		}
 
-		/* name column */
-		grouplist_get_group_name (group, buf, sizeof(buf));
-		utf8_name = pan_utf8ize (buf, -1, &freeme);
-		gtk_clist_set_text (clist, row, 1, utf8_name);
-		g_free (freeme);
-
-		/* total column */
-		g_snprintf (buf, sizeof(buf), "%d", total);
-		gtk_clist_set_text (clist, row, 3, buf);
-
-		/* unread column */
-		g_snprintf (buf, sizeof(buf), "%d", unread);
-		gtk_clist_set_text (clist, row, 2, buf);
+		if ((change_type & GROUP_CHANGED_ARTICLE_QTY)
+		    || (change_type & GROUP_CHANGED_ARTICLE_READ_QTY))
+		{
+			char buf[256] = { '\0' };
+			const int unread = group->article_qty - group->article_read_qty;
+			g_snprintf (buf, sizeof(buf), "%d", MAX(0,unread));
+			gtk_clist_set_text (clist, row, 2, buf);
+		}
 	}
 }
 
@@ -686,47 +684,45 @@ static int
 grouplist_groups_changed_idle (gpointer data)
 {
 	guint i;
-	GPtrArray * a = (GPtrArray*)data;
+	struct GroupChangeEvent * e = (struct GroupChangeEvent*) data;
+	const gboolean do_freeze = e->group_qty > 3u;
 	Server * server = grouplist_get_visible_server ();
-	const gboolean do_freeze = a->len > 3u;
 	GtkCList * clist = GTK_CLIST (Pan.group_tree);
 	debug_enter ("grouplist_groups_changed_idle");
 
 	pan_lock ();
+
 	if (do_freeze)
 		gtk_clist_freeze (clist);
-	for (i=0; i!=a->len; ++i) {
-		const Group * group = GROUP (g_ptr_array_index (a, i));
-		if (group_is_valid(group) && group->server==server)
-			grouplist_update_group_nolock (clist, group);
-	}
+	for (i=0; i!=e->group_qty; ++i)
+		if (group_is_valid(e->groups[i]) && e->groups[i]->server==server)
+			grouplist_update_group_nolock (clist, e->groups[i], e->change_type);
 	if (do_freeze)
 		gtk_clist_thaw (clist);
+
 	pan_unlock ();
 
 	/* cleanup */
-	g_ptr_array_free (a, TRUE);
+	g_free (e->groups);
+	g_free (e);
 	debug_exit ("grouplist_groups_changed_idle");
 	return 0;
 }
 
 static void
-grouplist_groups_changed_cb (gpointer gp_groups, gpointer gp_qty, gpointer unused2)
+grouplist_groups_changed_cb (gpointer change_event_gpointer, gpointer unused, gpointer unused2)
 {
-	int i;
-	const int qty = GPOINTER_TO_INT(gp_qty);
-	const Group ** groups = (const Group**) gp_groups;
-	GPtrArray * a;
+	struct GroupChangeEvent * e = (struct GroupChangeEvent*) change_event_gpointer;
+	struct GroupChangeEvent * my_e;
+	debug_enter ("grouplist_groups_changed_cb");
 
-	/* sanity clause */
-	g_return_if_fail (qty>0);
-	g_return_if_fail (groups!=NULL);
-	for (i=0; i<qty; ++i)
-		g_return_if_fail (group_is_valid (groups[i]));
+	my_e = g_new0 (struct GroupChangeEvent, 1);
+	my_e->groups = g_memdup (e->groups, sizeof(Group*) * e->group_qty);
+	my_e->group_qty = e->group_qty;
+	my_e->change_type = e->change_type;
+	gui_queue_add (grouplist_groups_changed_idle, my_e);
 
-	a = g_ptr_array_new ();
-	pan_g_ptr_array_assign (a, (gpointer*)groups, qty);
-	gui_queue_add (grouplist_groups_changed_idle, a);
+	debug_exit ("grouplist_groups_changed_cb");
 }
 
 /***
@@ -803,11 +799,22 @@ grouplist_refresh_filter_nolock (void)
 	g_free (new_filter);
 }
 
+static gboolean
+grab_focus_idle (gpointer w)
+{
+	pan_lock ();
+	gtk_widget_grab_focus (GTK_WIDGET(w));
+	pan_unlock ();
+	return FALSE;
+}
+
 static gboolean
 grouplist_filter_changed_cb_nolock (void)
 {
 	grouplist_refresh_filter_nolock ();
 
+	gui_queue_add (grab_focus_idle, Pan.group_tree);
+
 	return FALSE;
 }
 
@@ -921,8 +928,8 @@ grouplist_set_server_nolock (Server * server)
 			                            GTK_DIALOG_DESTROY_WITH_PARENT,
 			                            GTK_MESSAGE_QUESTION,
 			                            GTK_BUTTONS_YES_NO,
-			                            _("We don't have a list of groups for \"%s\".\nShall we get download it?"),
-			                            server->name);
+			                            _("We don't have a list of groups for \"%*.*s\".\nShall we download one?"),
+			                            server->name.len, server->name.len, server->name.str);
 			gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_YES);
 			g_signal_connect (w, "response", G_CALLBACK(grouplist_dialog_response_cb), server);
 			gtk_widget_show_all (w);
@@ -1094,10 +1101,12 @@ grouplist_create (void)
 	grouplist_mode_menu = grouplist_mode_menu_create();
 	gtk_box_pack_start (GTK_BOX(hbox), grouplist_mode_menu, FALSE, FALSE, 0);
 
-	w = gtk_label_new (_("Find:"));
+	w = gtk_label_new_with_mnemonic (_("F_ind:"));
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
-	groupname_filter_entry = w = gtk_entry_new ();
+	groupname_filter_entry = gtk_entry_new ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL(w), groupname_filter_entry);
+	w = groupname_filter_entry;
 	g_signal_connect (w, "focus_in_event", G_CALLBACK(entry_focus_in_cb), w);
 	g_signal_connect (w, "focus_out_event", G_CALLBACK(entry_focus_out_cb), w);
 	g_signal_connect (w, "focus_out_event", G_CALLBACK(grouplist_filter_changed_cb_nolock), w);
@@ -1181,8 +1190,7 @@ grouplist_update_font (void)
 
 static void
 grouplist_internal_select_row (int       row,
-                               gboolean  activate,
-                               gboolean  lock)
+                               gboolean  activate)
 {
         GtkCList* list = GTK_CLIST(Pan.group_tree);
 
@@ -1190,20 +1198,15 @@ grouplist_internal_select_row (int       row,
 	_modifiers = activate ? FALSE : TRUE;
 	_button_click_count = activate ? 2 : -1;
 
-	if (lock)
-		pan_lock ();
 	gtk_clist_freeze (list);
 	gtk_clist_unselect_all (list);
 	gtk_clist_select_row (list,row,-1);
 	gtk_clist_thaw (list);
-	if (lock)
-		pan_unlock ();
 }
 
 static gboolean
 grouplist_select_row_if_unread (int row,
-                                gboolean activate,
-                                gboolean lock)
+                                gboolean activate)
 {
         GtkCList * list = GTK_CLIST(Pan.group_tree);
 	Group * g = gtk_clist_get_row_data(list,row);
@@ -1212,7 +1215,7 @@ grouplist_select_row_if_unread (int row,
 	gboolean retval = FALSE;
 
 	if (read < total) {
-		grouplist_internal_select_row (row, activate, lock);
+		grouplist_internal_select_row (row, activate);
 		retval = TRUE;
 	}
 
@@ -1239,7 +1242,7 @@ grouplist_next_unread_group (gboolean activate)
 
 		/* walk through */
 		for ( i=0; i<row_qty; ++i, row=(row+1)%row_qty)
-			if (grouplist_select_row_if_unread (row, activate, 0))
+			if (grouplist_select_row_if_unread (row, activate))
 				break;
 	}
 
@@ -1275,7 +1278,7 @@ grouplist_next_group (gboolean activate)
 			row = (sel + 1) % row_qty;
 		}
 
-		grouplist_internal_select_row (row, activate, 0);
+		grouplist_internal_select_row (row, activate);
 	}
 
 	pan_unlock ();
