@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __GUI_HEADERS_H__
@@ -25,6 +22,7 @@
 
 #include <glib.h>
 #include <gtk/gtkwidget.h>
+#include <gmime/gmime-message.h>
 #include <pan/base/article.h>
 
 /**
@@ -59,11 +57,11 @@ extern gulong header_flags;
  * the untruncated header will be shown in a popup dialog.
  *
  * @param GtkTable the gtktable to use.  Its previous contents are removed.
- * @param Article the article whose headers to use.
+ * @param GMimeMessage the article whose headers to use.
  * @param header_fields which headers to show.
  */
 void gui_headers_set_nolock       (GtkWidget         * gtk_table,
-                                   const Article     * article,
+                                   GMimeMessage      * message,
                                    gulong              header_fields);
 
 /**
@@ -72,6 +70,6 @@ void gui_headers_set_nolock       (GtkWidget         * gtk_table,
  * don't have to query the configuration file by hand.
  */
 void gui_headers_set_default_nolock      (GtkWidget         * table,
-                                          const Article     * article);
+                                          GMimeMessage      * message);
 
 #endif /* __GUI_HEADERS_H__ */
