@@ -45,8 +45,13 @@ pack_widgets (GtkWidget * w1, GtkWidget * w2, GtkOrientation orient, gint uglyha
 	gboolean vertical = orient == GTK_ORIENTATION_VERTICAL;
 
 	if (w1!=NULL && w2!=NULL) {
-		const gchar * keys[] = { "/Pan/Geometry/vpaned=302", "/Pan/Geometry/hpaned=265" };
-		const gint pos = pan_config_get_int (keys[uglyhack_idx]);
+		int pos;
+
+		if (uglyhack_idx == 0)
+			pos = pan_config_get_int ("/Pan/Geometry/vpaned", 302);
+		else
+			pos = pan_config_get_int ("/Pan/Geometry/hpaned", 265);
+
 		if (vertical) {
 			w = gtk_vpaned_new ();
 			gtk_widget_set_usize (w1, -1, 50);
@@ -83,8 +88,6 @@ gui_paned_construct (void)
 	int i;
 	debug_enter ("gui_paned_construct");
 
-	pan_lock ();
-
 	gui_layout_clear_workarea ();
 
 	/**
@@ -153,10 +156,8 @@ gui_paned_construct (void)
 
 	Pan.viewmode = GUI_PANED;
 	gui_restore_column_widths (Pan.group_tree, "group");
-	gui_restore_column_widths (Pan.article_ctree, "header_pane");
 	gtk_widget_show_all (GTK_WIDGET(Pan.workarea));
 
-	pan_unlock ();
 	debug_exit ("gui_paned_construct");
 }
 
