@@ -57,8 +57,6 @@ typedef struct
 	GtkWidget * dialog;
 
 	GtkWidget * name_entry;
-	GtkWidget * default_news_tb;
-	GtkWidget * default_mail_tb;
 
 	GtkWidget * author_real_entry;
 	GtkWidget * author_addr_entry;
@@ -117,187 +115,62 @@ create_identity_tab (IdentityEditDialog *d)
 {
 	GtkWidget * t;
 	GtkWidget * w;
-	GtkWidget * l;
 	int row;
-	char buf[512];
 
 	row = 0;
-	t = gtk_table_new (4, 15, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER(t), 12);
-	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
+	t = pan_hig_workarea_create ();
 
-		/* title for profile info section */
-		g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Profile Information"));
-		l = gtk_label_new (buf);
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-		++row;
-
-		/* spacer to move the fields a little to the right of the name header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+3, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+3, 0, 0, 0, 0);
-
-		l = gtk_label_new_with_mnemonic (_("_Profile name:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+	pan_hig_workarea_add_section_title (t, &row, _("Profile Information"));
+	
+		pan_hig_workarea_add_section_spacer (t, row, 1);
 
 		w = d->name_entry = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("_Profile Name:"), w, NULL);
 
-		w = d->default_news_tb = gtk_check_button_new_with_mnemonic (_("Make this the default profile for _news"));
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
-		++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t, &row, _("Required Information"));
 
-		w = d->default_mail_tb = gtk_check_button_new_with_mnemonic (_("Make this the default profile for _mail"));
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
-		++row;
-
-	/* spacer between "profile information" and "required information" */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u);
-	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
-	++row;
-
-		/* title for profile info section */
-		g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Required Information"));
-		l = gtk_label_new (buf);
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-		++row;
-
-		/* spacer to move the fields a little to the right of the name header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+2, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+2, 0, 0, 0, 0);
-
-		l = gtk_label_new_with_mnemonic (_("_Full Name:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		pan_hig_workarea_add_section_spacer (t, row, 2);
 
 		w = d->author_real_entry = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = gtk_label_new_with_mnemonic (_("_Email Address:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		pan_hig_workarea_add_row (t, &row, _("_Full Name:"), w, NULL);
 
 		w = d->author_addr_entry = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_row (t, &row, _("_Email Address:"), w, NULL);
 
-	/* spacer between "profile information" and "required information" */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u);
-	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t, &row, _("Optional Information"));
 
-		/* title for profile info section */
-		g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Optional Information"));
-		l = gtk_label_new (buf);
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-		++row;
-
-		/* spacer to move the fields a little to the right of the name header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+2, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u);
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+2, 0, 0, 0, 0);
-
-		l = gtk_label_new_with_mnemonic (_("_Reply-To Header:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		pan_hig_workarea_add_section_spacer (t, row, 6);
 
 		w = d->reply_to_entry = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = gtk_label_new_with_mnemonic (_("Or_ganization Header:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Used when email replies to your posts should go somewhere other than the email address above.  Because Reply-To: is relatively difficult for spammers to harvest, it's polite give your email address here if you gave an invalid email address above."), "");
+		pan_hig_workarea_add_row (t, &row, _("_Reply-To Header:"), w, NULL);
 
 		w = d->organization_entry = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = gtk_label_new_with_mnemonic (_("Message-ID _Domain Name:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		pan_hig_workarea_add_row (t, &row, _("Or_ganization Header:"), w, NULL);
 
 		w = d->msg_id_fqdn_entry = gtk_entry_new ();
-		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, 
-			_("The domain name to use for Message-ID generation"), "");
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = gtk_label_new_with_mnemonic (_("_Attribution:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("When posting to Usenet, your article's Message-ID contains a domain name.  You can set a custom domain name here, or leave it blank to let Pan use the domain name from your email address."), "");
+		pan_hig_workarea_add_row (t, &row, _("Message-ID _Domain Name:"), w, NULL);
 
 		w = d->attribution_entry = gtk_entry_new ();
-		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, 
-			"%i for Message-ID\n"
-			"%a for Author and Address\n"
-			"%n for Author name\n%d for Date", "");
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = gtk_label_new_with_mnemonic (_("_Signature File:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("%i for Message-ID\n%a for Author and Address\n%n for Author name\n%d for Date"), "");
+		pan_hig_workarea_add_row (t, &row, _("_Attribution:"), w, NULL);
 
 		w = d->signature_entry = pan_file_entry_new (_("Select Signature File"));
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = gtk_label_new_with_mnemonic (_("C_haracter Set:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		pan_hig_workarea_add_row (t, &row, _("_Signature File:"), w, NULL);
 
 		w = d->posting_charset_entry = pan_charset_picker_new ("");
-		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, 
-			_("The default charset set for articles "
-			  "posted with this profile"), "");
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The default charset set for articles posted with this profile"), "");
+		pan_hig_workarea_add_row (t, &row, _("C_haracter Set:"), w, NULL);
 
 	return t;
 }
 
 static void
 populate_identity_tab (IdentityEditDialog * d,
-                       const Identity     * id,
-                       gboolean             default_news,
-                       gboolean             default_mail)
+                       const Identity     * id)
 {
 	gboolean    b;
 	GtkWidget * w;
@@ -308,14 +181,6 @@ populate_identity_tab (IdentityEditDialog * d,
 	w = d->name_entry;
 	pan_gtk_entry_set_text (w, b ? id->name : "");
 
-	/* default news */
-	w = d->default_news_tb;
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), default_news);
-
-	/* default mail */
-	w = d->default_mail_tb;
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), default_mail);
-
 	/* author name */
 	w = d->author_real_entry;
 	pan_gtk_entry_set_text (w, 
@@ -366,9 +231,7 @@ populate_identity_tab (IdentityEditDialog * d,
 
 static void
 identity_from_identity_tab (IdentityEditDialog * d, 
-                            Identity           * id,
-                            gboolean           * default_news,
-                            gboolean           * default_mail)
+                            Identity           * id)
 {
 	char * pch;
 	const char * cpch;
@@ -376,8 +239,6 @@ identity_from_identity_tab (IdentityEditDialog * d,
 
 	g_return_if_fail (d!=NULL);
 	g_return_if_fail (id!=NULL);
-	g_return_if_fail (default_news!=NULL);
-	g_return_if_fail (default_mail!=NULL);
 
 	/* Identity name */
 	w = d->name_entry;
@@ -385,14 +246,6 @@ identity_from_identity_tab (IdentityEditDialog * d,
 	replace_gstr (&id->name, pch);
 	g_strstrip (id->name);
 
-	/* default news */
-	w = d->default_news_tb;
-	*default_news = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
-
-	/* default mail */
-	w = d->default_mail_tb;
-	*default_mail = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w));
-
 	/* author name */
 	w = d->author_real_entry;
 	pch = gtk_editable_get_chars (GTK_EDITABLE (w), 0, -1);
@@ -594,9 +447,7 @@ dialog_destroy_cb (GtkDialog * dialog, gpointer data)
  
 GtkWidget*
 identity_edit_dialog_new (GtkWindow      * window,
-                          const Identity * id,
-                          gboolean         default_news,
-                          gboolean         default_mail)
+                          const Identity * id)
 {
 	GtkWidget *w, *n, *d_v;
 	IdentityEditDialog * d;
@@ -627,23 +478,21 @@ identity_edit_dialog_new (GtkWindow      * window,
 	gtk_box_pack_start (GTK_BOX(d_v), n, TRUE, TRUE, GUI_PAD_SMALL);
 
 	/* populate */
-	populate_identity_tab (d, id, default_news, default_mail);
+	populate_identity_tab (d, id);
 	populate_headers_tab (d, id);
 
 	return d->dialog;
 }
 
 Identity*
-identity_edit_dialog_get_identity (GtkWidget * w, 
-                                   gboolean  * default_news,
-                                   gboolean  * default_mail)
+identity_edit_dialog_get_identity (GtkWidget * w)
 {
 	Identity           * id = NULL;
 	IdentityEditDialog * d  = 
 		(IdentityEditDialog*) gtk_object_get_user_data (GTK_OBJECT(w));
 	
 	id = identity_new ();
-	identity_from_identity_tab (d, id, default_news, default_mail);
+	identity_from_identity_tab (d, id);
 	identity_from_headers_tab (d, id);
 
 	/* make sure user didn't skip on any of the essential parts */
