@@ -24,11 +24,14 @@
 
 #include <pan/base/argset.h>
 #include <pan/base/pan-i18n.h>
+#include <pan/base/log.h>
 
 #include <pan/message-check-ui.h>
 #include <pan/globals.h>
 #include <pan/util.h>
 
+#include <pan/xpm/pan-pixbufs.h>
+
 typedef struct
 {
 	GtkWidget * window;
@@ -70,8 +73,8 @@ post_warning_cb (gpointer user_data)
 	guint i;
 	GString * str;
 	GtkWidget * w;
+	GtkWidget * button;
 	GtkMessageType message_type;
-	GtkButtonsType buttons_type;
 	CheckedCallbackStruct * cbs = (CheckedCallbackStruct*) user_data;
 
 	/* build the error message */
@@ -81,7 +84,8 @@ post_warning_cb (gpointer user_data)
 		g_string_append (str, e);
 		g_string_append_c (str, '\n');
 	}
-	g_warning (str->str);
+	if (str->len) /* remove the last linefeed */
+		g_string_truncate (str, str->len-1);
 
 	if (cbs->goodness == WARN)
 	{
@@ -90,14 +94,12 @@ post_warning_cb (gpointer user_data)
 			? _("Post anyway, despite these problems?")
 			: _("Post anyway, despite this problem?");
 		g_string_append (str, "\n");
-			g_string_append (str, ques);
-			buttons_type = GTK_BUTTONS_YES_NO;
-			message_type = GTK_MESSAGE_QUESTION;
+		g_string_append (str, ques);
+		message_type = GTK_MESSAGE_QUESTION;
 	}
 	else
 	{
 		/* I'm not asking you, I'm _telling_ you. ;) */
-		buttons_type = GTK_BUTTONS_OK;
 		message_type = GTK_MESSAGE_ERROR;
 	}
 
@@ -105,11 +107,33 @@ post_warning_cb (gpointer user_data)
 	w = gtk_message_dialog_new (GTK_WINDOW(cbs->window),
 	                            GTK_DIALOG_DESTROY_WITH_PARENT,
 	                            message_type,
-	                            buttons_type,
+	                            GTK_BUTTONS_NONE,
 	                            "%s", str->str);
 	g_signal_connect (w, "response", G_CALLBACK(warning_response_cb), cbs);
 	gtk_widget_show_all (w);
 
+	if (message_type == GTK_MESSAGE_QUESTION)
+	{
+		GtkWidget *label, *image, *button, *hbox, *align;
+
+		button = gtk_button_new ();
+		label = gtk_label_new_with_mnemonic (_("_Post"));
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+		image = pan_gtk_image_new_from_inline_text (icon_compose_post, GTK_ICON_SIZE_BUTTON);
+		hbox = gtk_hbox_new (FALSE, GUI_PAD);
+		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+		gtk_container_add (GTK_CONTAINER (button), align);
+		gtk_container_add (GTK_CONTAINER (align), hbox);
+		gtk_widget_show_all (button);
+		gtk_dialog_add_action_widget (GTK_DIALOG(w), button, GTK_RESPONSE_YES);
+	}
+
+	button = gtk_dialog_add_button (GTK_DIALOG(w), GTK_STOCK_CANCEL, GTK_RESPONSE_NO);
+	gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_NO);
+	gtk_widget_grab_focus (button);
+
 	/* cleanup */
 	g_string_free (str, TRUE);
 	return 0;
