@@ -78,7 +78,7 @@ pan_file_entry_new (const char * title)
 	/* add the keys */
 	g_object_set_data_full (G_OBJECT(hbox), "title", g_strdup(title), g_free);
 	g_object_set_data (G_OBJECT(hbox), "entry", e);
-	pan_file_entry_set (hbox, g_get_home_dir());
+	pan_file_entry_set (hbox, pan_get_home_dir());
 
 	gtk_widget_show (e);
 	gtk_widget_show (b);
