@@ -24,7 +24,6 @@
 
 /* DISPLAY */
 #define PREFIX_DISPLAY "/Pan/Display/"
-#define KEY_THREAD_DATE_FORMAT                     PREFIX_DISPLAY  "thread_date_format"
 #define KEY_COLOR_UNREAD                           PREFIX_DISPLAY  "unread_color"
 #define KEY_COLOR_READ                             PREFIX_DISPLAY  "read_color"
 #define KEY_COLOR_URL                              PREFIX_DISPLAY  "text_url_color" 
@@ -45,17 +44,20 @@
 #define KEY_COLOR_IGNORED_BG                       PREFIX_DISPLAY  "score_color_ignored_background"
 #define KEY_COLOR_IGNORED_FG                       PREFIX_DISPLAY  "score_color_ignored_foreground"
 #define KEY_QUOTE_CHARS                            PREFIX_DISPLAY  "text_quote_chars"
-#define KEY_SMOOTH_SCROLLING_ENABLED               PREFIX_DISPLAY  "smooth_scrolling_enabled"
-#define KEY_SMOOTH_SCROLLING                       PREFIX_DISPLAY  "smooth_scrolling"
 #define KEY_GROUP_PANE_ENABLED                     PREFIX_DISPLAY  "group_pane_enabled"
 #define KEY_GROUP_PANE_COLLAPSE_GROUP_NAMES        PREFIX_DISPLAY  "group_pane_collapse_group_names"
 #define KEY_GROUP_PANE_CUSTOM_FONT_ENABLED         PREFIX_DISPLAY  "group_pane_custom_font_enabled"
 #define KEY_GROUP_PANE_CUSTOM_FONT                 PREFIX_DISPLAY  "group_pane_custom_font"
+#define KEY_HEADER_PANE_DATE_FORMAT                PREFIX_DISPLAY  "header_pane_date_format"
 #define KEY_HEADER_PANE_ENABLED                    PREFIX_DISPLAY  "header_pane_enabled"
 #define KEY_HEADER_PANE_CUSTOM_FONT_ENABLED        PREFIX_DISPLAY  "header_pane_custom_font_enabled"
 #define KEY_HEADER_PANE_CUSTOM_FONT                PREFIX_DISPLAY  "header_pane_custom_font"
-#define KEY_HEADER_PANE_THREADING_ENABLED          PREFIX_DISPLAY  "header_plane_threading_enabled"
+#define KEY_HEADER_PANE_THREADING_ENABLED          PREFIX_DISPLAY  "header_pane_threading_enabled"
+#define KEY_HEADER_PANE_COLUMNS                    PREFIX_DISPLAY  "header_pane_columns"
+#define KEY_HEADER_PANE_THREADS_EXPANDED           PREFIX_DISPLAY  "header_pane_threads_expanded"
+#define KEY_HEADER_PANE_HIDE_MULTIPARTS            PREFIX_DISPLAY  "header_pane_hide_multiparts"
 #define KEY_BODY_PANE_ENABLED                      PREFIX_DISPLAY  "body_pane_enabled"
+#define KEY_BODY_PANE_HEADERS                      PREFIX_DISPLAY  "body_pane_headers"
 #define KEY_BODY_PANE_MUTE_QUOTED_TEXT             PREFIX_DISPLAY  "body_pane_mute_quoted_text"
 #define KEY_BODY_PANE_SHOW_ALL_HEADERS             PREFIX_DISPLAY  "body_pane_show_all_headers"
 #define KEY_BODY_PANE_WRAP_ENABLED                 PREFIX_DISPLAY  "body_pane_wrap_enabled"
@@ -63,15 +65,27 @@
 #define KEY_BODY_PANE_CUSTOM_FONT                  PREFIX_DISPLAY  "body_pane_custom_font"
 #define KEY_BODY_PANE_MONOSPACE_FONT_ENABLED       PREFIX_DISPLAY  "body_pane_monospace_font_enabled"
 #define KEY_BODY_PANE_MONOSPACE_FONT               PREFIX_DISPLAY  "body_pane_monospace_font"
+#define KEY_BODY_PANE_SMOOTH_SCROLLING_ENABLED     PREFIX_DISPLAY  "body_pane_smooth_scrolling_enabled"
+#define KEY_BODY_PANE_SMOOTH_SCROLLING_SPEED       PREFIX_DISPLAY  "body_pane_smooth_scrolling_speed"
 #define KEY_COLOR_SUBJECT_COLUMN                   PREFIX_DISPLAY  "color_subject_column"
 #define KEY_COLOR_SCORE_COLUMN                     PREFIX_DISPLAY  "color_score_column"
 #define KEY_PANE_LAYOUT                            PREFIX_DISPLAY  "pane_layout"
 #define KEY_PANE_MODE                              PREFIX_DISPLAY  "pane_mode"
 #define KEY_BREAK_THREAD_WHEN_SUBJECT_CHANGES      PREFIX_DISPLAY  "break_thread_when_subject_changes"
-#define KEY_EXPAND_ALL_THREADS_BY_DEFAULT          PREFIX_DISPLAY  "expand_all_threads_by_default"
-#define KEY_HIDE_MULTIPART_CHILDREN                PREFIX_DISPLAY  "hide_multipart_children"
 #define KEY_WRAP_COLUMN                            PREFIX_DISPLAY  "wrap_column"
-#define KEY_ARTICLELIST_COLUMNS                    PREFIX_DISPLAY  "column_order"
+
+#define DEFAULT_VALUE_GROUP_PANE_ENABLED                     TRUE
+#define DEFAULT_VALUE_GROUP_PANE_COLLAPSE_NAMES              FALSE
+#define DEFAULT_VALUE_HEADER_PANE_ENABLED                    TRUE
+#define DEFAULT_VALUE_HEADER_PANE_IS_THREADED                TRUE
+#define DEFAULT_VALUE_BODY_PANE_ENABLED                      TRUE
+#define DEFAULT_VALUE_BODY_PANE_WRAP_ENABLED                 FALSE
+#define DEFAULT_VALUE_BODY_PANE_MONOSPACE_FONT_ENABLED       FALSE
+#define DEFAULT_VALUE_BODY_PANE_SHOW_ALL_HEADERS             FALSE
+#define DEFAULT_VALUE_BODY_PANE_MUTE_QUOTED                  FALSE
+#define DEFAULT_VALUE_PANE_MODE                              1
+#define DEFAULT_VALUE_WRAP_COLUMN                            74
+
 
 /* BEHAVIOR */
 #define PREFIX_BEHAVIOR "/Pan/behavior/"
@@ -84,19 +98,31 @@
 #define KEY_CACHE_FLUSH_ON_EXIT                    PREFIX_BEHAVIOR  "flush_cache_on_exit"
 #define KEY_CACHE_MAXIMUM_SIZE_MEGS                PREFIX_BEHAVIOR  "maximum_cache_size_megs"
 
-/* GENERAL */
-#define PREFIX_GENERAL "/Pan/general/"
+#define DEFAULT_VALUE_REMOVE_FAILED_TASKS          TRUE
 
 /* SCORING */
 #define PREFIX_SCORE "/Pan/scoring/"
 #define KEY_SCOREFILE                              PREFIX_SCORE "scorefile"
-#define KEY_SCOREFILE_DEFAULT                      PREFIX_SCORE "scorefile=News" G_DIR_SEPARATOR_S "Score"
+#define KEY_SCOREFILE_DEFAULT                      "News" G_DIR_SEPARATOR_S "Score"
+
+/* POSTING */
+#define PREFIX_POST "/Pan/posting/"
+#define KEY_POST_SPELLCHECK_ENABLED                PREFIX_POST "spellcheck_enabled"
+#define KEY_POST_USER_AGENT_HEADER                 PREFIX_POST "user_agent_header_enabled"
+#define KEY_POST_MESSAGE_ID_HEADER                 PREFIX_POST "message_id_header_enabled"
+#define DEFAULT_VALUE_POST_SPELLCHECK_ENABLED      TRUE
+#define DEFAULT_VALUE_POST_USER_AGENT_HEADER       TRUE
+#define DEFAULT_VALUE_POST_MESSAGE_ID_HEADER       TRUE
+
 
 /* APPS */
-#define PREFIX_APPS "/Pan/apps"
+#define PREFIX_APPS "/Pan/apps/"
 #define KEY_APP_BROWSER                            PREFIX_APPS "web_browser"
 #define KEY_APP_EDITOR                             PREFIX_APPS "editor"
 #define KEY_APP_SCOREFILE_EDITOR                   PREFIX_APPS "scorefile_editor"
+#define KEY_APP_PRINT_COMMAND                      PREFIX_APPS "print"
+
+#define DEFAULT_VALUE_APP_PRINT_COMMAND            "lpr"
 
 
 /**
@@ -117,7 +143,7 @@ extern gboolean                fetch_new_on_group_load;
 extern gboolean                expand_all_threads_by_default;
 extern gboolean                hide_mpart_child_nodes;
 extern gboolean                pan_mute;
-extern char                  * thread_date_format;
+extern char                  * header_pane_date_format;
 
 extern gboolean                header_pane_color_subject_column;
 extern gboolean                header_pane_color_score_column;
