@@ -79,8 +79,8 @@ Rule*        rule_dup                  (const Rule * r);
 ***  PUBLIC
 **/
 
-gboolean     rule_test_group           (const Rule * rule,
-                                        const gchar * groupname);
+gboolean     rule_test_group           (const Rule     * rule,
+                                        const PString  * groupname);
 
 /**
  * Apply the rule to a set of articles.
