@@ -22,9 +22,6 @@
 
 #include <glib.h>
 
-#include <pan/base/article.h>
-#include <pan/base/group.h>
-
 /**
  * @param Group* the group to use when fetching the articles
  *
@@ -33,7 +30,8 @@
  *             WILL FREE list, the sublist, and the Articles
  *             when done.
  */
-extern void save_attachment_as (Group*, GSList*);
+extern void save_attachment_as (GSList      * gslist_of_article_gslists,
+                                const char  * optional_download_dir);
 
 
 #endif /* __QUEUE_ITEM_BODY__H__ */
