@@ -231,7 +231,7 @@ get_value_for_match_type (GtkWidget * entry, GtkWidget * om, gboolean * negate)
 
 
 static char*
-add_this_to_scorefile (ScoreEditDialog * d)
+add_this_to_scorefile (ScoreEditDialog * d, gboolean do_rescore_all)
 {
 	gboolean value_assign_flag;
 	gboolean on;
@@ -314,9 +314,22 @@ add_this_to_scorefile (ScoreEditDialog * d)
 	str = g_string_new (NULL);
 	for (i=0; i<item_qty; ++i) {
 		on = on || items[i].on;
-		if (items[i].on && !is_nonempty_string(items[i].value)) {
-			g_string_append_printf (str, _("Criterion \"%s\" is selected, but empty."), items[i].key);
-			g_string_append_c (str, '\n');
+		if (items[i].on)
+		{
+			if (!is_nonempty_string(items[i].value))
+			{
+				g_string_append_printf (str, _("Criterion \"%s\" is selected, but empty."), items[i].key);
+				g_string_append_c (str, '\n');
+			}
+			else
+			{
+				char * regex_errmsg = filter_phrase_validate_regex (items[i].value);
+				if (regex_errmsg != NULL) {
+					g_string_append_printf (str, _("Can't use regular expression \"%s\": %s"), items[i].value, regex_errmsg);
+					g_string_append_c (str, '\n');
+					g_free (regex_errmsg);
+				}
+			}
 		}
 	}
 	if (!on) {
@@ -332,7 +345,7 @@ add_this_to_scorefile (ScoreEditDialog * d)
 
 	/* if no errors in the user's in put, add the score. */
 	if (!retval)
-		score_add (NULL, section, value, value_assign_flag, lifespan_days, AGGREGATE_TYPE_AND, items, item_qty);
+		score_add (NULL, section, value, value_assign_flag, lifespan_days, AGGREGATE_TYPE_AND, items, item_qty, do_rescore_all);
 
 	/* cleanup */
 	g_free (section);
@@ -350,7 +363,7 @@ score_edit_response_cb (GtkDialog * w, int response, gpointer user_data)
 
 	if (response == GTK_RESPONSE_APPLY || response == GTK_RESPONSE_OK)
 	{
-		char * error_message = add_this_to_scorefile (d);
+		char * error_message = add_this_to_scorefile (d, response==GTK_RESPONSE_APPLY);
 		if (error_message != NULL)
 		{
 			GtkWidget * dialog = gtk_message_dialog_new (GTK_WINDOW(w),
@@ -391,7 +404,7 @@ dialog_populate (ScoreEditDialog * d, const Article * a, ScoreAddMode mode)
 	d->message_id = a ? g_strdup (article_get_message_id (a)) : NULL;
 	d->top_message_id = a ? article_get_thread_message_id (a) : NULL;
 	d->group_string = a ? g_strdup (group_get_name (a->group)) : NULL;
-	d->xref_string = a ? g_strdup (a->xref) : NULL;
+	d->xref_string = a ? g_memdup (a->xref.str, a->xref.len+1) : NULL;
 
 	/* section */
 	/* FIXME: this is awkward, and gui.c's menu refresh does it too.
@@ -491,7 +504,6 @@ toggled_cb (GtkToggleButton * tb, gpointer user_data)
 GtkWidget*
 score_add_dialog (GtkWidget * window, Article * article, ScoreAddMode mode)
 {
-	char buf[512];
 	GtkObject * a;
 	GtkWidget * dialog;
 	GtkWidget * w;
@@ -544,9 +556,7 @@ score_add_dialog (GtkWidget * window, Article * article, ScoreAddMode mode)
 	**/
 
 	row = 0;
-	t = gtk_table_new (COLUMNS, ROWS, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER(t), GUI_PAD_BIG);
-	gtk_table_set_row_spacings (GTK_TABLE(t), GUI_PAD);
+	t = pan_hig_workarea_create ();
 	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), t, TRUE, TRUE, 0);
 
 	/**
@@ -572,25 +582,16 @@ score_add_dialog (GtkWidget * window, Article * article, ScoreAddMode mode)
 	***  Section
 	**/
 
-	/* title for which section of the scorefile to use */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Newsgroups"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, COLUMNS, row, row+1);
-	++row;
+	pan_hig_workarea_add_section_title (t, &row, _("Newsgroups"));
 
-		/* section label */
-		l = gtk_label_new (_("Group"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0, 0.5);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, CHECK_L, CHECK_R, row, row+1);
+		/* section */
+
+		l = pan_hig_workarea_add_label (t, row, _("Group"));
 
-		/* section menu */
 		w = d->section_menu = match_type_option_menu_new ();
 		gtk_table_attach_defaults (GTK_TABLE(t), w, CONTROL1_L, CONTROL1_R, row, row+1);
 		match_type_option_menu_set (w, PHRASE_MATCH_IS, FALSE);
 
-		/* section entry */
 		w = d->section_entry = gtk_entry_new ();
 		gtk_entry_set_width_chars (GTK_ENTRY(w), ENTRY_WIDTH_CHARS);
 		gtk_table_attach_defaults (GTK_TABLE(t), w, CONTROL2_L, CONTROL2_R, row, row+1);
@@ -600,19 +601,8 @@ score_add_dialog (GtkWidget * window, Article * article, ScoreAddMode mode)
 	***  Criteria
 	**/
 
-	/* spacer between "section" and "criteria" */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u); 
-	gtk_table_attach (GTK_TABLE(t), w, 0, COLUMNS, row, row+1, 0, 0, 0, 0);
-	++row;
-
-	/* title for which section of the scorefile to use */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Criteria"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, COLUMNS, row, row+1);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t,  &row, _("Criteria"));
 
 		/* subject */
 		l = d->subject_tb = gtk_check_button_new_with_mnemonic (_("Su_bject:"));
@@ -682,19 +672,8 @@ score_add_dialog (GtkWidget * window, Article * article, ScoreAddMode mode)
 		refresh_enabled_states (GTK_TOGGLE_BUTTON(l));
 		++row;
 
-	/* spacer between "criteria" and "score" */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u); 
-	gtk_table_attach (GTK_TABLE(t), w, 0, COLUMNS, row, row+1, 0, 0, 0, 0);
-	++row;
-
-	/* title for which section of the scorefile to use */
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Score"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, COLUMNS, row, row+1);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t,  &row, _("Score"));
 
 		/* Numeric */
 		l = d->score_tb = gtk_radio_button_new_with_mnemonic (NULL, _("Chan_ge Score:"));
@@ -727,18 +706,8 @@ score_add_dialog (GtkWidget * window, Article * article, ScoreAddMode mode)
 		gtk_table_attach_defaults (GTK_TABLE(t), l, CHECK_L, CHECK_R, row, row+1);
 		++row;
 
-	/* spacer between "score" and "lifespan" */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u); 
-	gtk_table_attach (GTK_TABLE(t), w, 0, COLUMNS, row, row+1, 0, 0, 0, 0);
-	++row;
-
-	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Lifespan"));
-	l = gtk_label_new (buf);
-	gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-	gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-	gtk_table_attach_defaults (GTK_TABLE(t), l, 0, COLUMNS, row, row+1);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t,  &row, _("Score"));
 
 		/* expire in N days */
 		l = d->expire_tb = gtk_radio_button_new_with_mnemonic (NULL, _("Expire in N _days:"));
