@@ -105,6 +105,8 @@ ServerListDialog;
 ************  Extern
 ***********/
 
+extern GtkTooltips * ttips;
+
 /***********
 ************  Public
 ***********/
@@ -168,9 +170,9 @@ edit_dialog_populate (ServerEditDialog * d, Server * s)
 	d->server = s;
 
 	/* server section */
-	cpch = s->name;
+	cpch = s->name.str;
 	gtk_entry_set_text (GTK_ENTRY(d->name_entry), cpch ? cpch : "");
-	cpch = s->address;
+	cpch = s->address.str;
 	gtk_entry_set_text (GTK_ENTRY(d->address_entry), cpch ? cpch : "");
 	a = gtk_spin_button_get_adjustment (GTK_SPIN_BUTTON(d->port_spin));
 	gtk_adjustment_set_value (a, s->port);
@@ -178,35 +180,43 @@ edit_dialog_populate (ServerEditDialog * d, Server * s)
 	gtk_adjustment_set_value (a, s->max_connections);
 
 	/* authorization */
-	cpch = s->username;
+	cpch = s->username.str;
 	gtk_entry_set_text (GTK_ENTRY(d->auth_username_entry), cpch ? cpch : "");
-	cpch = s->password;
+	cpch = s->password.str;
 	gtk_entry_set_text (GTK_ENTRY(d->auth_password_entry), cpch ? cpch : "");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(d->auth_checkbox), s->need_auth);
 	edit_dialog_sensitize_authorization (d, s->need_auth);
 
 	/* newsrc */
-	pan_file_entry_set (d->newsrc_filename_entry, s->newsrc_filename);
+	pan_file_entry_set (d->newsrc_filename_entry, s->newsrc_filename.str);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(d->newsrc_checkbox), s->use_newsrc);
 	edit_dialog_sensitize_newsrc (d, s->use_newsrc);
 }
 
 static void
-edit_dialog_populate_server (ServerEditDialog * d, Server * s)
+edit_dialog_populate_server (ServerEditDialog * d, Server * s, GError ** err)
 {
 	gboolean b;
+	PString name;
 	const char * cpch;
 
 	/* sanity clause */
 	g_return_if_fail (d!=NULL);
 	g_return_if_fail (GTK_IS_WIDGET(d->dialog));
 	g_return_if_fail (server_is_valid(s));
+	g_return_if_fail (err!=NULL);
+	g_return_if_fail (*err==NULL);
 
 	/* server section */
-	server_set_name (s, gtk_entry_get_text (GTK_ENTRY(d->name_entry)));
+	name = pstring_shallow (gtk_entry_get_text (GTK_ENTRY(d->name_entry)), -1);
+	server_set_name (s, &name, err);
 	cpch = gtk_entry_get_text (GTK_ENTRY(d->address_entry));
-	replace_gstr (&s->address, g_strdup(cpch));
-	g_strstrip (s->address);
+	pstring_set (&s->address, cpch, -1);
+	pstring_strstrip (&s->address);
+	if (!s->address.len) {
+		*err = g_error_new (g_quark_from_string("server-ui"), 1, _("You must specify an address."));
+		return;
+	}
 	s->port = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(d->port_spin));
 	s->max_connections = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(d->connection_limit_spin));
 
@@ -214,33 +224,53 @@ edit_dialog_populate_server (ServerEditDialog * d, Server * s)
 	b = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->auth_checkbox));
 	s->need_auth = b;
 	cpch = gtk_entry_get_text (GTK_ENTRY(d->auth_username_entry));
-	replace_gstr (&s->username, g_strdup(cpch));
-	g_strstrip (s->username);
+	pstring_set (&s->username, cpch, -1);
+	pstring_strstrip (&s->username);
+	if (s->need_auth && !s->username.len) {
+		*err = g_error_new (g_quark_from_string("server-ui"), 1, _("You must specify a username or disable authorization."));
+		return;
+	}
 	cpch = gtk_entry_get_text (GTK_ENTRY(d->auth_password_entry));
-	replace_gstr (&s->password, g_strdup(cpch));
+	pstring_set (&s->password, cpch, -1);
+	if (s->need_auth && !s->password.len) {
+		*err = g_error_new (g_quark_from_string("server-ui"), 1, _("You must specify a password or disable authorization."));
+		return;
+	}
 
 	/* newsrc */
 	b = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(d->newsrc_checkbox));
 	s->use_newsrc = b;
 	cpch = pan_file_entry_get (d->newsrc_filename_entry);
-	replace_gstr (&s->newsrc_filename, g_strdup(cpch));
-	g_strstrip (s->newsrc_filename);
+	pstring_set (&s->newsrc_filename, cpch, -1);
+	pstring_strstrip (&s->newsrc_filename);
+	if (s->use_newsrc && !s->newsrc_filename.len) {
+		*err = g_error_new (g_quark_from_string("server-ui"), 1, _("You must specify a .newsrc filename or disable newsrc support."));
+		return;
+	}
 }
 
 static void
 server_edit_response_cb (GtkDialog * w, int response, gpointer user_data)
 {
+	gboolean destroy = TRUE;
 	ServerEditDialog * d = (ServerEditDialog*) user_data;
 
 	if (response == GTK_RESPONSE_OK)
 	{
-		edit_dialog_populate_server (d, d->server);
+		GError * err = NULL;
+		edit_dialog_populate_server (d, d->server, &err);
 
-		if (d->new_server)
-			serverlist_add_server (d->server);
+		if (d->new_server && err==NULL)
+			serverlist_add_server (d->server, &err);
+
+		if (err == NULL)
+			queue_wakeup (); /* wake up the queue in case user changed "max connections" field */
+		else {
+			pan_error_dialog_parented (GTK_WINDOW(d->dialog), "%s", err->message);
+			g_error_free (err);
+			destroy = FALSE;
+		}
 
-		/* wake up the queue in case user changed "max connections" field */
-		queue_wakeup ();
 	}
 	else /* cancel, destroy */
 	{
@@ -248,18 +278,17 @@ server_edit_response_cb (GtkDialog * w, int response, gpointer user_data)
 			pan_object_unref (PAN_OBJECT(d->server));
 	}
 
-	gtk_widget_destroy (GTK_WIDGET(w));
+	if (destroy)
+		gtk_widget_destroy (GTK_WIDGET(w));
 }
 
 static GtkWidget*
 server_edit_dialog (GtkWidget * window, Server * server)
 {
-	char buf[512];
 	GtkObject * a;
 	GtkWidget * dialog;
 	GtkWidget * w;
 	GtkWidget * t;
-	GtkWidget * l;
 	ServerEditDialog * d;
 	int row;
 	const int DEFAULT_MAX_PER_SERVER = 4;
@@ -282,185 +311,68 @@ server_edit_dialog (GtkWidget * window, Server * server)
 	**/
 
 	row = 0;
-	t = gtk_table_new (4, 18, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER(t), 12);
-	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
+	t = pan_hig_workarea_create ();
 	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d->dialog)->vbox), t, TRUE, TRUE, 0);
+	pan_hig_workarea_add_section_title (t, &row, _("Server"));
 
-		/* title for name section */
-		g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Server"));
-		l = gtk_label_new (buf);
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-		++row;
-
-		/* spacer to move the fields a little to the right of the name header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u); 
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+5, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u); 
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+5, 0, 0, 0, 0);
-
-		l = gtk_label_new_with_mnemonic (_("_Name:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		pan_hig_workarea_add_section_spacer (t, row, 5);
 
 		w = d->name_entry = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = gtk_label_new_with_mnemonic (_("_Address:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("Short name for Pan to show, e.g. \"mynewsserver\""), NULL);
+		pan_hig_workarea_add_row (t, &row, _("_Name:"), w, NULL);
 
 		w = d->address_entry = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = gtk_label_new_with_mnemonic (_("Por_t:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The news server's actual address, e.g. \"news.mynewsserver.com\""), NULL);
+		pan_hig_workarea_add_row (t, &row, _("_Address:"), w, NULL);
 
 		a = gtk_adjustment_new (1.0, 1.0, ULONG_MAX, 1.0, 1.0, 1.0);
 		w = d->port_spin = gtk_spin_button_new (GTK_ADJUSTMENT(a), 1.0, 0u);
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, _("The news server's port.  This is almost always 119."), NULL);
+		pan_hig_workarea_add_row (t, &row, _("Por_t:"), w, NULL);
 
-	/* spacer between "name" and "connections" */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u); 
-	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t, &row, _("Connections"));
 
-		/* title for name section */
-		g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Connections"));
-		l = gtk_label_new (buf);
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-		++row;
-
-		/* spacer to move the fields a little to the right of the section header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u); 
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+3, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u); 
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+3, 0, 0, 0, 0);
-
-		l = gtk_label_new_with_mnemonic (_("Connection _Limit:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		pan_hig_workarea_add_section_spacer (t, row, 3);
 
 		a = gtk_adjustment_new (1.0, 1.0, DEFAULT_MAX_PER_SERVER, 1.0, 1.0, 1.0);
-		w = gtk_spin_button_new (GTK_ADJUSTMENT(a), 1.0, 0u);
-		d->connection_limit_spin = w;
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		d->connection_limit_spin = w = gtk_spin_button_new (GTK_ADJUSTMENT(a), 1.0, 0u);
+		pan_hig_workarea_add_row (t, &row, _("Connection _Limit:"), w, NULL);
 
-	/* spacer between "connections" and "authorization" */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u); 
-	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t, &row, _("Login"));
 
-		/* title for name section */
-		g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Login"));
-		l = gtk_label_new (buf);
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_section_spacer (t, row, 3);
 
-		/* spacer to move the fields a little to the right of the section header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u); 
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+3, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u); 
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+3, 0, 0, 0, 0);
-
-		w = d->auth_checkbox = gtk_check_button_new_with_mnemonic (_("Server requires authori_zation"));
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
+		w = d->auth_checkbox = pan_hig_workarea_add_wide_checkbutton (t, &row, _("Server requires authori_zation"), FALSE);
 		g_signal_connect (w, "toggled", G_CALLBACK(edit_dialog_auth_toggled_cb), d);
-		++row;
-
-		l = d->auth_username_label = gtk_label_new_with_mnemonic (_("_Username:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
 
 		w = d->auth_username_entry = gtk_entry_new ();
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
-
-		l = d->auth_password_label = gtk_label_new_with_mnemonic (_("_Password:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+		d->auth_username_label = pan_hig_workarea_add_row (t, &row, _("_Username:"), w, NULL);
 
 		w = d->auth_password_entry = gtk_entry_new ();
 		gtk_entry_set_visibility (GTK_ENTRY(w), FALSE);
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		d->auth_password_label = pan_hig_workarea_add_row (t, &row, _("_Password:"), w, NULL);
 
-	/* spacer between "authorization" and "newsrc" */
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 0u, 6u); 
-	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
-	++row;
+	pan_hig_workarea_add_section_divider (t, &row);
+	pan_hig_workarea_add_section_title (t, &row, _("Newsrc"));
 
-		/* title for name section */
-		g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Newsrc"));
-		l = gtk_label_new (buf);
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_label_set_use_markup (GTK_LABEL(l), TRUE);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 0, 4, row, row+1);
-		++row;
+		pan_hig_workarea_add_section_spacer (t, row, 2);
 
-		/* spacer to move the fields a little to the right of the section header */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 18u, 0u); 
-		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+2, 0, 0, 0, 0);
-
-		/* spacer between the controls and their labels */
-		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-		gtk_widget_set_usize (w, 12u, 0u); 
-		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+2, 0, 0, 0, 0);
-
-		w = d->newsrc_checkbox = gtk_check_button_new_with_mnemonic (_("Us_e .newsrc file"));
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
+		w = d->newsrc_checkbox = pan_hig_workarea_add_wide_checkbutton (t, &row, _("Us_e .newsrc file"), FALSE);
 		g_signal_connect (w, "toggled", G_CALLBACK(edit_dialog_newsrc_toggled_cb), d);
-		++row;
-
-		l = d->newsrc_filename_label = gtk_label_new_with_mnemonic (_("_Filename:"));
-		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
-		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
 
 		w = d->newsrc_filename_entry = pan_file_entry_new (_("Select .newsrc File"));
 		gtk_widget_set_usize (w, 300u, 0u); 
-		gtk_label_set_mnemonic_widget (GTK_LABEL(l), pan_file_entry_gtk_entry(w));
-		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
-		++row;
+		d->newsrc_filename_label = pan_hig_workarea_add_row (t, &row, _("_Filename:"), w, pan_file_entry_gtk_entry(w));
 
 	if (server == NULL) {
 		d->new_server = TRUE;
 		server = server_new ();
-		server->name = g_strdup (_("New Server"));
+		pstring_set (&server->name, _("New Server"), -1);
 		server->port = 119;
 	}
+
 	edit_dialog_populate (d, server);
 	gtk_widget_show_all (d->dialog);
 	return d->dialog;
@@ -505,7 +417,7 @@ get_public_server_array (void)
 
 	/* skip the internal folder... */
 	for (i=0; i<a->len; ++i)
-		if (!pan_strcmp (INTERNAL_SERVER_NAME, SERVER(g_ptr_array_index(a,i))->name))
+		if (pstring_equal (&INTERNAL_SERVER_NAME, &SERVER(g_ptr_array_index(a,i))->name))
 			break;
 	if (i!=a->len)
 		g_ptr_array_remove_index (a, i);
@@ -533,7 +445,7 @@ server_tree_view_refresh (ServerListDialog * d)
 
 		gtk_list_store_append (d->servers_store, &iter);
 		gtk_list_store_set (d->servers_store, &iter,
-				    SERVER_NAME_COLUMN, server->name,
+				    SERVER_NAME_COLUMN, server->name.str,
 				    SERVER_DATA_COLUMN, server,
 				    -1);
 		
