@@ -30,7 +30,15 @@
 #include <time.h> /* for time_t */
 
 #include <glib.h>
+
+#if defined(HAVE_GNET_1)
 #include <gnet/gnet.h>
+#define gnet_tcp_socket_get_io_channel gnet_tcp_socket_get_iochannel
+#elif defined(HAVE_GNET_2)
+#include <gnet.h>
+#else
+#error Must have GNet 1 or GNet t
+#endif
 
 #include <pan/base/debug.h>
 #include <pan/base/log.h>
@@ -56,21 +64,26 @@ struct _PanSocket
 
 	/* private fields */  
 	gboolean need_auth;
-	char * nntp_group_name;
-	char * nntp_username;
-	char * nntp_password;
+	PString nntp_group_name;
+	PString nntp_username;
+	PString nntp_password;
 	GString * line_buffer;
 	GTcpSocket * gnet_socket;
 	GIOChannel * gio_channel;
 
 	/* statistics */
-	char * host_address;
-	char * host_name;
+	PString host_address;
+	PString host_name;
 	guint bytes_read;
 	guint bytes_written;
 	time_t byte_count_start_time;
+
+	int consecutive_failed_operations;
 };
 
+
+#define MAX_CONSECUTIVE_FAILED_OPERATIONS 600
+
 /*********************
 **********************  Variables
 *********************/
@@ -82,15 +95,15 @@ static gulong total_bytes = 0;
 *********************/
 
 PanSocket*
-pan_socket_new (const char     * server_name,
-                const char     * server_address,
-                int              port)
+pan_socket_new (const PString     * server_name,
+                const PString     * server_address,
+                int                 port)
 {
 	PanSocket * sock;
 
 	/* sanity clause */
-	g_return_val_if_fail (is_nonempty_string (server_name), NULL);
-	g_return_val_if_fail (is_nonempty_string (server_address), NULL);
+	g_return_val_if_fail (pstring_is_set (server_name), NULL);
+	g_return_val_if_fail (pstring_is_set (server_address), NULL);
 	g_return_val_if_fail (port>=0, NULL);
 
 	/* normally SIGPIPE, from writes on closed sockets, kill the app */
@@ -109,42 +122,57 @@ pan_socket_new (const char     * server_name,
 void
 pan_socket_constructor (PanSocket             * sock,
 			PanObjectDestructor     destructor,
-			const char            * server_name,
-			const char            * server_address,
+			const PString         * server_name,
+			const PString         * server_address,
 			gint                    port)
 {
 	/* sanity clause */
 	g_return_if_fail (sock!=NULL);
-	g_return_if_fail (is_nonempty_string(server_name));
-	g_return_if_fail (is_nonempty_string(server_address));
+	g_return_if_fail (pstring_is_set (server_name));
+	g_return_if_fail (pstring_is_set (server_address));
 	g_return_if_fail (port>=0);
 
-	/* constructor */
+	/* constructor - zero things out */
 	pan_object_constructor (PAN_OBJECT(sock), destructor);
-	sock->host_address = g_strdup (server_address);
-	sock->host_name = g_strdup (server_name);
+	sock->host_address = PSTRING_INIT;
+	sock->host_name = PSTRING_INIT;
 	sock->bytes_read = 0u;
 	sock->bytes_written = 0u;
 	sock->byte_count_start_time = time(NULL);
+	sock->consecutive_failed_operations = 0;
 	sock->line_buffer = g_string_sized_new (512);
 	sock->error = FALSE;
 	sock->need_auth = FALSE;
-	sock->nntp_group_name = NULL;
-	sock->nntp_username = NULL;
-	sock->nntp_password = NULL;
+	sock->nntp_group_name = PSTRING_INIT;
+	sock->nntp_username = PSTRING_INIT;
+	sock->nntp_password = PSTRING_INIT;
+
+	/* set the host information */
+	pstring_copy (&sock->host_address, server_address);
+	pstring_copy (&sock->host_name, server_name);
 
 	/* set up the serv_addr struct */
 	errno = 0;
-	sock->gnet_socket = gnet_tcp_socket_connect (server_address, port);
+	sock->gnet_socket = gnet_tcp_socket_connect (server_address->str, port);
 	if (sock->gnet_socket==NULL)
-		log_add_va (LOG_ERROR, _("Connection to %s, port %d failed: %s"), server_address, port, g_strerror(errno));
-	else {
-		sock->gio_channel = gnet_tcp_socket_get_iochannel (sock->gnet_socket);
+	{
+		pan_socket_set_error_flag (sock, TRUE);
+
+		log_add_va (LOG_ERROR, _("Connection to %*.*s, port %d failed: %s"),
+		            server_address->len, server_address->len, server_address->str,
+		            port, g_strerror(errno));
+	}
+	else
+	{
+		sock->gio_channel = gnet_tcp_socket_get_io_channel (sock->gnet_socket);
 		if (g_io_channel_get_encoding (sock->gio_channel) != NULL)
 			g_io_channel_set_encoding (sock->gio_channel, NULL, NULL);
+		g_io_channel_set_flags (sock->gio_channel, G_IO_FLAG_NONBLOCK, NULL);
 		g_io_channel_set_buffered (sock->gio_channel, TRUE);
 		g_io_channel_set_line_term (sock->gio_channel, "\n", 1);
-		log_add_va (LOG_INFO, _("New connection %p for %s, port %d"), sock, server_address, port);
+		log_add_va (LOG_INFO, _("New connection %p for %*.*s, port %d"), sock,
+		            server_address->len, server_address->len, server_address->str,
+		            port);
 	}
 
         debug1 (DEBUG_PAN_OBJECT, "pan_socket_constructor: %p", sock);
@@ -162,7 +190,10 @@ pan_socket_destructor (PanObject *obj)
 	debug1 (DEBUG_SOCKET, "socket closed: %p", sock);
         debug1 (DEBUG_PAN_OBJECT, "pan_socket_destructor: %p", sock);
 
-	g_io_channel_unref (sock->gio_channel);
+#ifdef HAVE_GNET_1
+	if (sock->gio_channel != NULL)
+		g_io_channel_unref (sock->gio_channel);
+#endif
 	gnet_tcp_socket_delete (sock->gnet_socket);
 	g_string_free (sock->line_buffer, TRUE);
 
@@ -170,11 +201,11 @@ pan_socket_destructor (PanObject *obj)
 	sock->gnet_socket = DEAD_POINTER;
 	sock->line_buffer = DEAD_POINTER;
 
-	replace_gstr (&sock->nntp_group_name, DEAD_POINTER);
-	replace_gstr (&sock->nntp_username, DEAD_POINTER);
-	replace_gstr (&sock->nntp_password, DEAD_POINTER);
-	replace_gstr (&sock->host_address, DEAD_POINTER);
-	replace_gstr (&sock->host_name, DEAD_POINTER);
+	pstring_clear (&sock->nntp_group_name);
+	pstring_clear (&sock->nntp_username);
+	pstring_clear (&sock->nntp_password);
+	pstring_clear (&sock->host_address);
+	pstring_clear (&sock->host_name);
 
 	/* pass along to superclass */
 	pan_object_destructor (obj);
@@ -184,21 +215,21 @@ pan_socket_destructor (PanObject *obj)
 void
 pan_socket_set_nntp_auth (PanSocket            * socket,
                           gboolean               need_auth,
-                          const gchar          * username,
-                          const gchar          * password)
+                          const PString        * username,
+                          const PString        * password)
 {
 	g_return_if_fail (socket!=NULL);
 
 	socket->need_auth = need_auth;
-	replace_gstr (&socket->nntp_username, g_strdup(username));
-	replace_gstr (&socket->nntp_password, g_strdup(password));
+	pstring_copy (&socket->nntp_username, username);
+	pstring_copy (&socket->nntp_password, password);
 }
 
 
 /*
  * get_server - read from a socket until \n
  */
-int
+TaskStateEnum
 pan_socket_getline (PanSocket    * sock,
                     const char  ** setme)
 {
@@ -208,9 +239,10 @@ pan_socket_getline (PanSocket    * sock,
 	debug_enter ("pan_socket_getline");
 
 	/* sanity clause */
-	g_return_val_if_fail (sock!=NULL, -1);
-	g_return_val_if_fail (!sock->error, -1);
-	g_return_val_if_fail (setme!=NULL, -1);
+	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
+	g_return_val_if_fail (setme!=NULL, TASK_FAIL);
+	if (sock->error || sock->gio_channel==NULL)
+		return TASK_FAIL_NETWORK;
 
 	/* read the next line */
 	g_string_truncate (sock->line_buffer, 0);
@@ -219,19 +251,22 @@ pan_socket_getline (PanSocket    * sock,
 		status = g_io_channel_read_line_string (sock->gio_channel, sock->line_buffer, NULL,  &err);
 		switch (status) {
 			case G_IO_STATUS_AGAIN:
-				if (err != NULL)
-					g_error_free (err);
-				g_usleep (G_USEC_PER_SEC);
-				continue;
+				if (++sock->consecutive_failed_operations <= MAX_CONSECUTIVE_FAILED_OPERATIONS ) {
+					g_usleep (G_USEC_PER_SEC/10);
+					continue;
+				}
+				/* fall through */
 			case G_IO_STATUS_ERROR:
+				/* fall through */
 			case G_IO_STATUS_EOF:
 				if (err != NULL) {
 					log_add_va (LOG_ERROR, _("Error reading from socket: %s"), err->message);
 					g_error_free (err);
 				}
 				pan_socket_set_error_flag (sock, TRUE);
-				return -1;
+				return TASK_FAIL_NETWORK;
 			case G_IO_STATUS_NORMAL:
+				sock->consecutive_failed_operations = 0;
 				done = TRUE;
 				break;
 		}
@@ -246,25 +281,32 @@ pan_socket_getline (PanSocket    * sock,
 
 	*setme = (const char*) sock->line_buffer->str;
 	debug_exit ("pan_socket_getline");
-	return 0;
+	return TASK_OK;
 }
 
 
-int 
+TaskStateEnum 
 pan_socket_putline (PanSocket     * sock,
                     const char    * line)
 {
+	gsize old_buffer_size;
 	gsize nbytes;
 	gsize nleft;
 
 	/* sanity clause */
-	g_return_val_if_fail (is_nonempty_string(line), -1);
-	g_return_val_if_fail (sock!=NULL, -1);
-	g_return_val_if_fail (!sock->error, -1);
+	g_return_val_if_fail (is_nonempty_string(line), TASK_FAIL);
+	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
+	if (sock->error || sock->gio_channel==NULL)
+		return TASK_FAIL_NETWORK;
 
 	debug3 (DEBUG_SOCKET_OUTPUT,
 	        "[%ld][socket %p putline][%s]", time(0), (void*)sock, line);
 
+	/* force unbuffered writes */
+	/* (workaround for http://bugzilla.gnome.org/show_bug.cgi?id=119285) */
+	old_buffer_size = g_io_channel_get_buffer_size (sock->gio_channel);
+	g_io_channel_set_buffered (sock->gio_channel, FALSE);
+
 	/* try to write the string */
 	nleft = nbytes = strlen (line);
 	while (nleft>0)
@@ -278,40 +320,48 @@ pan_socket_putline (PanSocket     * sock,
 		switch (status)
 		{
 			case G_IO_STATUS_AGAIN:
-				if (err != NULL)
-					g_error_free (err);
-				g_usleep (G_USEC_PER_SEC);
+				if (++sock->consecutive_failed_operations <= MAX_CONSECUTIVE_FAILED_OPERATIONS) {
+					nleft -= bytes_written;
+					line += bytes_written;
+					g_usleep (G_USEC_PER_SEC/10);
+					continue;
+				}
 				/* fall through */
-			case G_IO_STATUS_NORMAL:
-				nleft -= bytes_written;
-				line += bytes_written;
-				continue;
 			case G_IO_STATUS_EOF:
+				/* fall through */
 			case G_IO_STATUS_ERROR:
 				if (err != NULL) {
 					log_add_va (LOG_ERROR, _("Error writing to socket: %s"), err->message);
 					g_error_free (err);
 				}
 				pan_socket_set_error_flag (sock, TRUE);
-				return -1;
+				return TASK_FAIL_NETWORK;
+
+			case G_IO_STATUS_NORMAL:
+				nleft -= bytes_written;
+				line += bytes_written;
+				sock->consecutive_failed_operations = 0;
+				continue;
 		}
 	}
 
 	sock->bytes_written += nbytes;
 	total_bytes += nbytes;
 	g_io_channel_flush (sock->gio_channel, NULL);
+	g_io_channel_set_buffered (sock->gio_channel, TRUE);
+	g_io_channel_set_buffer_size (sock->gio_channel, old_buffer_size);
 
-	return nleft;
+	return nleft ? TASK_FAIL : TASK_OK;
 }
 
-gboolean 
+TaskStateEnum 
 pan_socket_putline_va (PanSocket    *sock,
                        const gchar  *format,
                        ...)
 {
 	va_list args;
 	char* line;
-	int retval;
+	TaskStateEnum retval;
 
 	g_return_val_if_fail (format!=NULL, -1);
 
@@ -327,7 +377,7 @@ pan_socket_putline_va (PanSocket    *sock,
 
 
 /* Flush a socket; wait for a closing socket */
-void
+TaskStateEnum
 pan_socket_flush (PanSocket * ps)
 {
 	char buf[512];
@@ -341,17 +391,17 @@ pan_socket_flush (PanSocket * ps)
 		switch (status)
 		{
 			case G_IO_STATUS_EOF:
-				return;
+				return TASK_FAIL_NETWORK;
 			case G_IO_STATUS_AGAIN:
 				continue;
 			case G_IO_STATUS_NORMAL:
 				if (bytes_read == 0)
-					return;
+					return TASK_OK;
 				continue;
 			case G_IO_STATUS_ERROR:
 				log_add_va (LOG_ERROR, _("Error reading from socket: %s"), err->message);
 				g_error_free (err);
-				return;
+				return TASK_FAIL_NETWORK;
 		}
 	}
 }
@@ -366,34 +416,34 @@ pan_socket_needs_auth (const PanSocket * sock)
 	return sock->need_auth;
 }
 
-const char*
+const PString*
 pan_socket_get_username (const PanSocket * sock)
 {
-	return sock->nntp_username;
+	return &sock->nntp_username;
 }
 
-const char*
+const PString*
 pan_socket_get_password (const PanSocket * sock)
 {
-	return sock->nntp_password;
+	return &sock->nntp_password;
 }
 
-const char*
+const PString*
 pan_socket_get_current_group (const PanSocket * sock)
 {
-	return sock->nntp_group_name;
+	return &sock->nntp_group_name;
 }
 
 void
-pan_socket_set_current_group (PanSocket * sock, const char * group_name)
+pan_socket_set_current_group (PanSocket * sock, const PString * group_name)
 {
-	replace_gstr (&sock->nntp_group_name, g_strdup(group_name));
+	pstring_copy (&sock->nntp_group_name, group_name);
 }
 
-const char*
+const PString*
 pan_socket_get_server_name (const PanSocket * sock)
 {
-	return sock->host_name;
+	return &sock->host_name;
 }
 
 void
