@@ -23,6 +23,8 @@
 
 #include <config.h>
 
+#include <string.h>
+
 #include <gmime/gmime-message.h>
 
 #include <pan/base/article.h>
@@ -85,12 +87,20 @@ task_cancel_run (Task * task, PanSocket * sock)
 	TaskCancel * task_cancel = TASK_CANCEL(task);
 	StatusItem * status = STATUS_ITEM(task);
 	GMimeMessage * message = task_cancel->message;
-	const char * message_id = g_mime_message_get_message_id (message);
+	const char * cpch;
+	PString id;
+	TaskStateEnum val;
 
-        const int result = nntp_cancel (status, message_id, sock);
+	/* invoke nntp_cancel */
+ 	cpch = g_mime_message_get_message_id (message);
+	id = pstring_shallow (cpch, -1);
+        val = nntp_cancel (status, &id, sock);
 
-	/* if successful, then remove the article from our local headers */
-	if (result == TASK_SUCCESS)
+	/* did it work? */
+	task_state_set_health (&task->state, val);
+	if (val != TASK_OK)
+		task_state_set_work_need_socket (&task->state, task->server, task_cancel_run);
+	else
 	{
 		const char * subject = g_mime_message_get_subject (message);
 		const char * newsgroups = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
@@ -98,25 +108,23 @@ task_cancel_run (Task * task, PanSocket * sock)
 		guint i;
 
 		for (i=0; groups[i]; ++i) {
-			Group * group = server_get_named_group (task->server, groups[i]);
+			const PString group_name = pstring_shallow (groups[i], -1);
+			Group * group = server_get_named_group (task->server, &group_name);
 			group_ref_articles (group, status); {
-				Article * article = ARTICLE(group_get_article_by_message_id (group, message_id));
+				Article * article = ARTICLE(group_get_article_by_message_id (group, &id));
 				if (article)
 					group_remove_articles (group, &article, 1);
 			}
 			group_unref_articles (group, status);
 		}
 
+		log_add_va (LOG_INFO|LOG_URGENT, _("Article %s (%s) canceled"), subject, id.str);
+
+		task_state_set_work_completed (&task->state);
+
+		/* cleanup */
 		g_strfreev (groups);
-
-		log_add_va (LOG_INFO|LOG_URGENT, _("Article %s (%s) canceled"), subject, message_id);
 	}
-
-	/* update the task's state */
-	if (result == TASK_SUCCESS || result == TASK_FAIL_HOPELESS)
-		task_state_set (&task->state, result);
-	else
-		task_state_set_need_socket (&task->state, task->server, task_cancel_run);
 }
 
 static char*
@@ -163,7 +171,7 @@ task_cancel_new (Server * server, GMimeMessage * message)
 
 	/* initialize the parent class */
 	task_constructor (TASK(cancel), task_cancel_destructor, task_cancel_describe, server, TRUE);
-	task_state_set_need_socket (&TASK(cancel)->state, server, task_cancel_run);
+	task_state_set_work_need_socket (&TASK(cancel)->state, server, task_cancel_run);
 	TASK(cancel)->type = TASK_TYPE_POST;
 
 	/* initialize the task-cancel */
