@@ -61,7 +61,6 @@ typedef struct
 
 	guint          timeout_id;
 
-	gboolean       titlebar_needs_refresh;
 	gboolean       dampen_move_feedback_loop;
 }
 ManagerUI;
@@ -359,7 +358,7 @@ refresh_row_nolock (GtkListStore * model, GtkTreeIter * iter, Task * task)
 	/* build the strings */
 	text[0] = pan_utf8ize (statusbuf, -1, freeme+0);
 	text[1] = pan_utf8ize (percentbuf, -1, freeme+1);
-	text[2] = pan_utf8ize (task->server->name, -1, freeme+2);
+	text[2] = pan_utf8ize (task->server->name.str, -1, freeme+2);
 	text[3] = pan_utf8ize (xferbuf, -1, freeme+3);
 	text[4] = pan_utf8ize (description, -1, freeme+4);
 
@@ -433,6 +432,16 @@ ui_populate (ManagerUI *ui)
 ***  Queue Changes
 **/
 
+static void
+tasks_added_idle_cleanup (gpointer argset_gpointer)
+{
+	ArgSet * argset = (ArgSet*) argset_gpointer;
+	GSList * tasks = (GSList*) argset_get (argset, 1);
+
+	g_slist_foreach (tasks, (GFunc)pan_object_unref, NULL);
+	g_slist_free (tasks);
+	argset_free (argset);
+}
 static int
 tasks_added_idle (gpointer argset_gpointer)
 {
@@ -451,9 +460,7 @@ tasks_added_idle (gpointer argset_gpointer)
 	add_tasks_to_list (ui, tasks, index);
 
 	/* cleanup */
-	g_slist_foreach (tasks, (GFunc)pan_object_unref, NULL);
-	g_slist_free (tasks);
-	argset_free (argset);
+	tasks_added_idle_cleanup (argset_gpointer);
 	debug_exit ("tasks_added_idle");
 	return 0;
 }
@@ -461,22 +468,40 @@ tasks_added_idle (gpointer argset_gpointer)
 static void
 tasks_added_cb (gpointer call_obj, gpointer call_arg, gpointer client_arg)
 {
-	GSList * tasks;
 	int index;
+	GSList * tasks;
 	ManagerUI * ui;
 	debug_enter ("tasks_added_cb");
 
-	tasks = (GSList*) call_obj;
 	index = GPOINTER_TO_INT(call_arg);
+	tasks = (GSList*) call_obj;
 	ui = (ManagerUI*) client_arg;
 
 	tasks = g_slist_copy (tasks);
 	g_slist_foreach (tasks, (GFunc)pan_object_ref, NULL);
-	gui_queue_add (tasks_added_idle, argset_new3 (ui, tasks, GINT_TO_POINTER(index)));
+	gui_queue_add_full_to_g_object (tasks_added_idle,
+	                                argset_new3 (ui, tasks, GINT_TO_POINTER(index)),
+	                                tasks_added_idle_cleanup,
+	                                G_OBJECT(ui->window));
 
 	debug_exit ("tasks_added_cb");
 }
 
+/***
+****
+***/
+
+static void
+tasks_removed_idle_cleanup (gpointer argset_gpointer)
+{
+	ArgSet * argset = (ArgSet*) argset_gpointer;
+	GSList * tasks = (GSList*) argset_get (argset, 1);
+
+	g_slist_foreach (tasks, (GFunc)pan_object_unref, NULL);
+	g_slist_free (tasks);
+	argset_free (argset);
+}
+
 static int
 tasks_removed_idle (gpointer argset_gpointer)
 {
@@ -508,9 +533,7 @@ tasks_removed_idle (gpointer argset_gpointer)
 	pan_unlock ();
 
 	/* cleanup */
-	g_slist_foreach (tasks, (GFunc)pan_object_unref, NULL);
-	g_slist_free (tasks);
-	argset_free (argset);
+	tasks_removed_idle_cleanup (argset_gpointer);
 	debug_exit ("tasks_removed_idle");
 	return 0;
 }
@@ -522,11 +545,15 @@ tasks_removed_cb (gpointer call_obj, gpointer call_arg, gpointer client_arg)
 	ManagerUI * ui;
 	debug_enter ("tasks_removed_cb");
 
-	ui = (ManagerUI*) client_arg;
 	tasks = (GSList*) call_obj;
+	ui = (ManagerUI*) client_arg;
+
 	tasks = g_slist_copy (tasks);
 	g_slist_foreach (tasks, (GFunc)pan_object_ref, NULL);
-	gui_queue_add (tasks_removed_idle, argset_new2 (ui, tasks));
+	gui_queue_add_full_to_g_object (tasks_removed_idle,
+	                                argset_new2 (ui, tasks),
+	                                tasks_removed_idle_cleanup,
+	                                G_OBJECT(ui->window));
 
 	debug_exit ("tasks_removed_cb");
 }
@@ -648,11 +675,8 @@ periodic_update_timer (gpointer user_data)
 	pan_unlock ();
 
 	/* update the titlebar */	
-	if (ui->titlebar_needs_refresh) {
-		update_titlebar_lock (ui);
-		ui->titlebar_needs_refresh = FALSE;
-	}
-	
+	update_titlebar_lock (ui);
+
 	return 1;
 }
 
@@ -833,30 +857,12 @@ task_manager_build_ui (GtkWidget * window)
 	pan_callback_add (queue_tasks_removed, tasks_removed_cb, ui);
 	pan_callback_add (queue_tasks_moved, tasks_moved_cb, ui);
 
-	ui->titlebar_needs_refresh = TRUE;
 	ui->timeout_id = pan_timeout_add (2000, periodic_update_timer, ui);
 	ui->top = vbox;
         g_signal_connect (ui->top, "destroy", G_CALLBACK(ui_destroy_cb), ui);
 	return ui;
 }
 
-/**
-***  Titlebar dirty
-**/
-
-static void
-update_titlebar_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
-{
-	((ManagerUI*)(user_data))->titlebar_needs_refresh = TRUE;
-}
-static void
-remove_update_titlebar_callbacks_cb (GtkObject * o, gpointer user_data)
-{
-	pan_callback_remove (queue_tasks_added, update_titlebar_cb, user_data);
-	pan_callback_remove (queue_tasks_removed, update_titlebar_cb, user_data);
-	pan_callback_remove (queue_task_status_changed, update_titlebar_cb, user_data);
-}
-
 /************
 *************  PUBLIC ROUTINES
 ************/
@@ -879,14 +885,10 @@ task_manager_spawn (void)
 	manager_ui = task_manager_build_ui (gtk_window_new(GTK_WINDOW_TOPLEVEL));
 	ui_populate (manager_ui);
 
-	pan_callback_add (queue_tasks_added, update_titlebar_cb, manager_ui);
-	pan_callback_add (queue_tasks_removed, update_titlebar_cb, manager_ui);
-	pan_callback_add (queue_task_status_changed, update_titlebar_cb, manager_ui);
 	g_signal_connect (manager_ui->window, "delete_event",
 	                  G_CALLBACK(window_delete_event_cb), manager_ui);
-	g_signal_connect (manager_ui->window, "destroy",
-	                  G_CALLBACK(remove_update_titlebar_callbacks_cb), manager_ui);
 	gtk_window_set_title (GTK_WINDOW(manager_ui->window), _("Pan: Task Manager"));
+	gtk_window_set_role (GTK_WINDOW(manager_ui->window), "pan-task-manager");
 	gtk_container_add (GTK_CONTAINER(manager_ui->window), manager_ui->top);
 
 	gtk_window_set_default_size (GTK_WINDOW (manager_ui->window), 600, 400);
