@@ -145,6 +145,7 @@ static void
 task_save_destructor (PanObject* object)
 {
 	gpointer p;
+	const PString ** ids;
 	Task * task = TASK(object);
 	TaskSave * task_save = TASK_SAVE(object);
 	debug_enter ("task_save_destructor");
@@ -158,7 +159,10 @@ task_save_destructor (PanObject* object)
 	while ((p = g_queue_pop_head (task_save->download_mids)))
 		g_object_unref (G_OBJECT(p));
 	g_queue_free (task_save->download_mids);
-	acache_checkin (ACACHE_DEFAULT_KEY, (MessageIdentifier**)task->identifiers->pdata, task->identifiers->len);
+
+	ids = message_identifiers_get_id_array ((const MessageIdentifier**)task->identifiers->pdata, task->identifiers->len);
+	acache_checkin (ACACHE_DEFAULT_KEY, ids, task->identifiers->len);
+	g_free (ids);
 
 	/* destruct the superclass */
 	task_destructor (PAN_OBJECT(object));
@@ -173,6 +177,7 @@ task_save_new (Server              * server,
 	int i;
 	int lines_total = 0;
 	int lines_cached = 0;
+	const PString ** ids;
 	TaskSave * task;
 	debug_enter ("task_save_new");
 
@@ -182,7 +187,8 @@ task_save_new (Server              * server,
 	g_return_val_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)mids, mid_qty), FALSE);
 
 	/* checkout the article bodies */
-	acache_checkout (ACACHE_DEFAULT_KEY, mids, mid_qty);
+	ids = message_identifiers_get_id_array ((const MessageIdentifier**)mids, mid_qty);
+	acache_checkout (ACACHE_DEFAULT_KEY, ids, mid_qty);
 
 	/* create the task */
        	task = g_new0 (TaskSave, 1);
@@ -202,7 +208,7 @@ task_save_new (Server              * server,
 	task->download_mids = g_queue_new ();
 	for (i=0; i<mid_qty; ++i) {
 		lines_total += mids[i]->line_qty;
-		if (acache_has_message (ACACHE_DEFAULT_KEY, mids[i]->message_id))
+		if (acache_has_message (ACACHE_DEFAULT_KEY, ids[i]))
 			lines_cached += mids[i]->line_qty;
 		else {
 			g_object_ref (mids[i]);
@@ -213,9 +219,9 @@ task_save_new (Server              * server,
 
 	/* set the task status */
 	if (g_queue_is_empty (task->download_mids))
-		task_state_set_need_work (&TASK(task)->state, task_save_run_decode);
+		task_state_set_work_need_work (&TASK(task)->state, task_save_run_decode);
 	else
-		task_state_set_need_socket (&TASK(task)->state, server, task_save_run_download);
+		task_state_set_work_need_socket (&TASK(task)->state, server, task_save_run_download);
 
 	/* init the status-item */
 	status_item_emit_init_steps (STATUS_ITEM(task), lines_total);
@@ -224,6 +230,7 @@ task_save_new (Server              * server,
 
 	/* return the task */
 	debug_exit ("task_save_new");
+	g_free (ids);
 	return PAN_OBJECT(task);
 }
 
@@ -264,7 +271,7 @@ task_save_set_attachments (TaskSave      * task,
                            const char    * filename)
 {
 	MessageIdentifier * mid;
-	const char * group_name;
+	const PString * group_name;
 	debug_enter ("task_save_set_attachments");
 
 	/* sanity clause */
@@ -288,7 +295,7 @@ task_save_set_attachments (TaskSave      * task,
 
 	/* expand */
 	replace_gstr (&task->path_attachments,
-	              expand_download_dir (task->path_attachments, group_name));
+	              expand_download_dir (task->path_attachments, group_name->str));
 
 	debug_exit ("task_save_set_attachments");
 }
@@ -299,7 +306,7 @@ task_save_set_bodies (TaskSave      * task,
                       const char    * filename)
 {
 	MessageIdentifier * mid;
-	const char * group_name;
+	const PString * group_name;
 	debug_enter ("task_save_set_bodies");
 
 	g_return_if_fail (task!=NULL);
@@ -322,7 +329,7 @@ task_save_set_bodies (TaskSave      * task,
 
 	/* expand */
 	replace_gstr (&task->path_bodies,
-	              expand_download_dir (task->path_bodies,  group_name));
+	              expand_download_dir (task->path_bodies,  group_name->str));
 
 	debug_exit ("task_save_set_bodies");
 }
@@ -427,7 +434,7 @@ static void
 task_save_run_decode (Task * task, PanSocket * sock)
 {
 	TaskSave * task_save;
-	TaskStateEnum retval = TASK_SUCCESS;
+	TaskStateEnum retval = TASK_OK;
 
        	task_save = TASK_SAVE(task);
 	if (task_save->save_attachments)
@@ -449,7 +456,7 @@ task_save_run_decode (Task * task, PanSocket * sock)
 		/* if we failed to decode something that we think
 		 * we should've been able to decode, then fail. */
 		if (!decode_ok)
-			retval = TASK_FAIL_HOPELESS;
+			retval = TASK_FAIL;
 	}
 
 	if (task_save->save_bodies)
@@ -463,12 +470,14 @@ task_save_run_decode (Task * task, PanSocket * sock)
 			gboolean body_ok = TRUE;
 			MessageIdentifier * mid;
 			GMimeMessage * message;
+			const PString * id;
 			char * fname;
 			char * path;
 
 			/* get the message */
 			mid = (MessageIdentifier*) g_ptr_array_index (task->identifiers, i);
-			message = acache_get_message (ACACHE_DEFAULT_KEY, &mid, 1);
+			id = &mid->message_id;
+			message = acache_get_message (ACACHE_DEFAULT_KEY, &id, 1);
 
 			/* get the filename */
 			fname = g_strdup (task_save->filename_bodies);
@@ -509,16 +518,21 @@ task_save_run_decode (Task * task, PanSocket * sock)
 		}
 
 		if (!bodies_ok)
-			retval = TASK_FAIL_HOPELESS;
+			retval = TASK_FAIL;
 	}
 
-	task_state_set (&task->state, retval);
+	task_state_set_health (&task->state, retval);
+
+	if (retval == TASK_OK)
+		task_state_set_work_completed (&task->state);
+	else
+		task_state_set_work_need_work (&task->state, task_save_run_decode);
 }
 
 static void
 task_save_run_download (Task * task, PanSocket * sock)
 {
-	int result;
+	TaskStateEnum val;
 	MessageIdentifier * mid;
 	TaskSave * task_save = TASK_SAVE(task);
 	debug_enter ("task_save_run");
@@ -527,28 +541,20 @@ task_save_run_download (Task * task, PanSocket * sock)
 
 	/* let the queue know about any more parts left to downloads */
 	if (!g_queue_is_empty (task_save->download_mids)) {
-		task_state_set_need_socket (&task->state, task->server, task_save_run_download);
+		task_state_set_work_need_socket (&task->state, task->server, task_save_run_download);
 		queue_wakeup ();
 	}
 
 	/* try to download the part... */
-	result = nntp_article_download (STATUS_ITEM(task), sock, mid, &task->hint_abort, NNTP_VERBOSE_NEXT_STEP);
-	switch (result)
-	{
-		case TASK_SUCCESS:
-			g_object_unref (mid);
-			if (!--task_save->download_mids_qty)
-				task_state_set_need_work (&task->state, task_save_run_decode);
-			break;
-
-		case TASK_FAIL_HOPELESS:
-			task_state_set (&task->state, TASK_FAIL_HOPELESS);
-			break;
-
-		default:
-			g_queue_push_head (task_save->download_mids, mid);
-			task_state_set_need_socket (&TASK(task)->state, task->server, task_save_run_download);
-			break;
+	val = nntp_article_download (STATUS_ITEM(task), sock, mid, &task->hint_abort, NNTP_VERBOSE_NEXT_STEP);
+	task_state_set_health (&TASK(task)->state, val);
+	if (val == TASK_OK) {
+		g_object_unref (mid);
+		if (!--task_save->download_mids_qty)
+			task_state_set_work_need_work (&task->state, task_save_run_decode);
+	} else {
+		g_queue_push_head (task_save->download_mids, mid);
+		task_state_set_work_need_socket (&TASK(task)->state, task->server, task_save_run_download);
 	}
 
 	debug_exit ("task_save_run");
