@@ -48,15 +48,15 @@ main (void)
 	Article * a;
 	Article * a1;
 	Article * articles [512];
-	const char * cpch;
 	const char * out;
 	char buf[512];
+	PString pstr;
 
 	g_thread_init (NULL);
 	pan_init ();
 
 	s = server_new ();
-	s->name = g_strdup ("Server");
+	pstring_set (&s->name, "Server", -1);
 	g = group_new (s, "alt.test");
 	group_ref_articles (g, NULL);
 
@@ -73,11 +73,16 @@ main (void)
 	check (a1->date == 0)
 	check (a1->threads == NULL)
 	check (a1->parent == NULL)
-	check (a1->author_addr == NULL)
-	check (a1->author_real == NULL)
-	check (a1->subject == NULL)
-	check (a1->message_id == NULL)
-	check (a1->references == NULL)
+	check (pstring_is_valid (&a1->author_addr))
+	check (!pstring_is_set (&a1->author_addr))
+	check (pstring_is_valid (&a1->author_real))
+	check (!pstring_is_set (&a1->author_real))
+	check (pstring_is_valid (&a1->subject))
+	check (!pstring_is_set (&a1->subject))
+	check (pstring_is_valid (&a1->message_id))
+	check (!pstring_is_set (&a1->message_id))
+	check (pstring_is_valid (&a1->references))
+	check (!pstring_is_set (&a1->references))
 
 	/**
 	***  article_set_subject
@@ -89,14 +94,15 @@ main (void)
 
 	/* don't crash */
 	article_set_subject (a1, NULL);
-	check (a1->message_id == NULL)
+	check (pstring_is_valid (&a1->message_id) != FALSE)
+	check (pstring_is_set (&a1->message_id) == FALSE)
 
 	/* set the subject */
-	cpch = "Some Subject";
-	article_set_subject (a1, cpch);
-	check (a1->subject != NULL)
-	check (a1->subject != cpch)
-	check (!pan_strcmp (a1->subject, cpch))
+	pstr = pstring_shallow ("Some Subject", -1);
+	article_set_subject (a1, &pstr);
+	check (a1->subject.str != NULL)
+	check (a1->subject.str != pstr.str)
+	check (pstring_equal (&pstr, &a1->subject));
 
 	/**
 	*** article_set_message_id
@@ -109,24 +115,23 @@ main (void)
 
 	/* don't crash */
 	article_set_message_id (a1, NULL);
-	check (a1->message_id == NULL)
+	check (pstring_is_set (&a1->message_id) == FALSE)
 	check (!pan_strcmp (article_get_message_id (a1), ""));
 
 	/* don't crash */
-	cpch = "<asdf@foo.com>";
-	article_set_message_id (NULL, cpch);
+	pstr = pstring_shallow ("<asdf@foo.com>", -1);
+	article_set_message_id (NULL, &pstr);
 
 	/* set the message-id */
-	article_set_message_id (a1, cpch);
-	check (a1->message_id != NULL)
-	check (a1->message_id != cpch)
-	check (!pan_strcmp(a1->message_id, cpch))
+	article_set_message_id (a1, &pstr);
+	check (pstring_is_set (&a1->message_id))
+	check (pstring_equal (&pstr, &a1->message_id));
 
 	/* this is the last step in place for article_is_valid() to pass */
 	a1->number = 1;
-	cpch = "<querty@foo.com>";
-	article_set_message_id (a1, cpch);
-	check (!pan_strcmp (article_get_message_id(a1), cpch))
+	pstr = pstring_shallow ("<querty@foo.com>", -1);
+	article_set_message_id (a1, &pstr);
+	check (pstring_equal (&pstr, &a1->message_id));
 
 	/**
 	***  article_set_references
@@ -134,31 +139,35 @@ main (void)
 
 	article_set_references (NULL, NULL);
 	article_set_references (a1, NULL);
-	check (a1->references == NULL)
-	article_set_references (a1, cpch = "<asdf@foo.com> <1434@bar.com>");
-	check (a1->references != NULL)
-	check (a1->references != cpch)
-	check (!pan_strcmp (a1->references, cpch))
+	check (pstring_is_valid (&a1->references))
+	check (!pstring_is_set (&a1->references))
+	pstr = pstring_shallow ("<asdf@foo.com> <1434@bar.com>", -1);
+	article_set_references (a1, &pstr);
+	check (pstring_is_set (&a1->references))
+	check (!pan_strcmp (a1->references.str, pstr.str));
 
 	/**
 	***  article_set_xrefs
 	***  article_get_crosspost_qty
 	**/
 
-	article_set_xrefs (NULL, NULL);
+	article_set_xref (NULL, NULL);
 	check (article_get_crosspost_qty (NULL) == 0)
-	article_set_xrefs (a1, NULL);
-	check (a1->xref == NULL)
+	article_set_xref (a1, NULL);
+	check (pstring_is_valid (&a1->xref))
+	check (!pstring_is_set (&a1->xref))
 	check (article_get_crosspost_qty (a1) == 1)
-	article_set_xrefs (a1, cpch = "servername comp.lang.c:666");
-	check (a1->xref != NULL)
-	check (a1->xref != cpch)
-	check (!pan_strcmp (a1->xref, cpch))
+	pstr = pstring_shallow ("servername comp.lang.c:666", -1);
+	article_set_xref (a1, &pstr);
+	check (pstring_is_valid (&a1->xref))
+	check (a1->xref.str != pstr.str)
+	check (!pan_strcmp (a1->xref.str, pstr.str))
 	check (article_get_crosspost_qty (a1) == 1)
-	article_set_xrefs (a1, cpch = "servername comp.lang.c:666 alt.fan.rubber.duckie:999");
-	check (a1->xref != NULL)
-	check (a1->xref != cpch)
-	check (!pan_strcmp (a1->xref, cpch))
+	pstr = pstring_shallow ("servername comp.lang.c:666 alt.fan.rubber.duckie:999", -1);
+	article_set_xref (a1, &pstr);
+	check (pstring_is_valid (&a1->xref))
+	check (a1->xref.str != pstr.str)
+	check (!pan_strcmp (a1->xref.str, pstr.str))
 	check (article_get_crosspost_qty (a1) == 2)
 
 	/**
@@ -169,25 +178,29 @@ main (void)
 
 	a = a1;
 	a->number = 1;
-	article_set_author (a, "joe@foo.com");
+	pstr = pstring_shallow ("joe@foo.com", -1);
+	article_set_author (a, &pstr);
 	article_get_author_str (a, buf, sizeof(buf));
 	check (!pan_strcmp(buf, "\"joe\" <joe@foo.com>"))
 	article_get_short_author_str (a, buf, sizeof(buf));
 	check (!pan_strcmp(buf, "joe"))
 
-	article_set_author (a, "joe@foo.com (Joe Bob)");
+	pstr = pstring_shallow ("joe@foo.com (Joe Bob)", -1);
+	article_set_author (a, &pstr);
 	article_get_author_str (a, buf, sizeof(buf));
 	check (!pan_strcmp(buf, "\"Joe Bob\" <joe@foo.com>"))
 	article_get_short_author_str (a, buf, sizeof(buf));
 	check (!pan_strcmp(buf, "Joe Bob"))
 
-	article_set_author (a, "Joe Bob <joe@foo.com>");
+	pstr = pstring_shallow ("Joe Bob <joe@foo.com>", -1);
+	article_set_author (a, &pstr);
 	article_get_author_str (a, buf, sizeof(buf));
 	check (!pan_strcmp(buf, "\"Joe Bob\" <joe@foo.com>"))
 	article_get_short_author_str (a, buf, sizeof(buf));
 	check (!pan_strcmp(buf, "Joe Bob"))
 
-	article_set_author (a, "Joe Bob");
+	pstr = pstring_shallow ("Joe Bob", -1);
+	article_set_author (a, &pstr);
 	article_get_author_str (a, buf, sizeof(buf));
 	check (!pan_strcmp(buf, "\"Joe Bob\" <unknown@spammesenseless.com>"))
 	article_get_short_author_str (a, buf, sizeof(buf));
@@ -200,13 +213,17 @@ main (void)
 
 	a = articles[article_qty++] = article_new (g);
 	a->number = article_qty;
-	article_set_message_id (a, "<213.214.242.41.21@foo.com>");
-	article_set_subject (a, "Me and the Major");
+	pstr = pstring_shallow ("<213.214.242.41.21@foo.com>", -1);
+	article_set_message_id (a, &pstr);
+	pstr = pstring_shallow ("Me and the Major", -1);
+	article_set_subject (a, &pstr);
 
 	a = articles[article_qty++] = article_new (g);
 	a->number = article_qty;
-	article_set_message_id (a, "<213.214.242.41.21@foo.com>");
-	article_set_subject (a, "Like Dylan in the Movies");
+	pstr = pstring_shallow ("<213.214.242.41.21@foo.com>", -1);
+	article_set_message_id (a, &pstr);
+	pstr = pstring_shallow ("Like Dylan in the Movies", -1);
+	article_set_subject (a, &pstr);
 
 	articles_set_multipart_state (articles, 0, MULTIPART_STATE_SOME);
 	articles_set_multipart_state (NULL, 0, MULTIPART_STATE_SOME);
