@@ -65,6 +65,7 @@ init_pan (void)
 	g_message ("data_dir = `%s'", get_data_dir());
 
 	g_free (data_dir);
+	g_free (base);
 }
 
 static Server * server = NULL;
@@ -113,7 +114,7 @@ main (void)
 	server_saving_enabled = FALSE;
 
 	server = server_new ();
-	server->name = g_strdup ("Server");
+	pstring_set (&server->name, "Server", -1);
 	group = group_new (server, "alt.test");
 	group_ref_articles (group, NULL);
 
