@@ -74,21 +74,27 @@ int main (void)
 		/* substitution tests */
 		a = pan_substitute ("hello darkness my old friend", "darkness", "world ");
 		if (pan_strcmp (a, "hello world  my old friend")) return num;
+		g_free (a);
 		a = pan_substitute ("xoxoxxoo", "xo", "a");
 		if (pan_strcmp (a, "aaxao")) return num;
+		g_free (a);
 		a = pan_substitute ("coocoo to you", "o", "foo");
 		if (pan_strcmp (a, "cfoofoocfoofoo tfoo yfoou")) return num;
+		g_free (a);
 
 		/* feed it trash & don't crash */
 		a = pan_substitute (NULL, "xo", "a");
 		if (pan_strcmp (a, NULL)) return num;
+		g_free (a);
 		a = pan_substitute ("foo bar", NULL, "cow");
 		if (pan_strcmp (a, NULL)) return num;
+		g_free (a);
 		a = pan_substitute ("foo bar", "foo", NULL);
 		if (pan_strcmp (a, NULL)) return num;
+		g_free (a);
 		a = pan_substitute (NULL, NULL, NULL);
 		if (pan_strcmp (a, NULL)) return num;
-
+		g_free (a);
 
 		g_message ("PASS pan_substitute");
 	}
@@ -396,26 +402,41 @@ int main (void)
 	++num;
 	if (1)
 	{
+		char * pch;
 		const gchar * string = "Foo 10 2112 Bar 1 \t 2 ";
 		const gchar * march = string;
-		if (pan_strcmp(get_next_token_str(march, ' ', &march),"Foo")) return num;
-		if (pan_strcmp(get_next_token_str(march, ' ', &march),"10")) return num;
-		if (pan_strcmp(get_next_token_str(march, ' ', &march),"2112")) return num;
-		if (pan_strcmp(get_next_token_str(march, ' ', &march),"Bar")) return num;
-		if (pan_strcmp(get_next_token_str(march, ' ', &march),"1")) return num;
-		if (pan_strcmp(get_next_token_str(march, ' ', &march),"\t")) return num;
-		if (pan_strcmp(get_next_token_str(march, ' ', &march),"2")) return num;
-		if (pan_strcmp(get_next_token_str(march, ' ', &march),NULL)) return num;
+		if (pan_strcmp(pch=get_next_token_str(march, ' ', &march),"Foo")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, ' ', &march),"10")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, ' ', &march),"2112")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, ' ', &march),"Bar")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, ' ', &march),"1")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, ' ', &march),"\t")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, ' ', &march),"2")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, ' ', &march),NULL)) return num;
+		g_free (pch);
 		g_message ("PASS get_next_token_str");
 
 		string = "Foo\t\t\t10\t2112";
 		march = string;
-		if (pan_strcmp(get_next_token_str(march, '\t', &march),"Foo")) return num;
-		if (pan_strcmp(get_next_token_str(march, '\t', &march),"")) return num;
-		if (pan_strcmp(get_next_token_str(march, '\t', &march),"")) return num;
-		if (pan_strcmp(get_next_token_str(march, '\t', &march),"10")) return num;
-		if (pan_strcmp(get_next_token_str(march, '\t', &march),"2112")) return num;
-		if (pan_strcmp(get_next_token_str(march, '\t', &march),NULL)) return num;
+		if (pan_strcmp(pch=get_next_token_str(march, '\t', &march),"Foo")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, '\t', &march),"")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, '\t', &march),"")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, '\t', &march),"10")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, '\t', &march),"2112")) return num;
+		g_free (pch);
+		if (pan_strcmp(pch=get_next_token_str(march, '\t', &march),NULL)) return num;
+		g_free (pch);
 		g_message ("PASS get_next_token_str 2");
 	}
 
